------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Assets             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local world_size = engine.platform.constants.scene.world_size

--------------
-- textures --
--------------

local textures_theme = engine.paths.textures_theme

textures = {}

-- foreground --
textures.foreground = { path = textures_theme .. "foreground", size = world_size }

-- background --
textures.background = { path = textures_theme .. "background", size = world_size }

-- wall --
textures.wall = { path = textures_theme .. "wall" }

-- hud --
textures.pause_menu = { path = textures_theme .. "hud/pause_menu", size = v2f( 320.0, 377.0 ) }

-- bonus --
textures.bonus_troll    = { path = textures_theme .. "bonus/troll", size = v2f( 86.0, 128.0 ), collides = true }
textures.bonus_boulder  = { path = textures_theme .. "bonus/boulder", size = v2f( 55.5, 61.0 ), collides = true }
textures.bonus_catapult = { path = textures_theme .. "bonus/catapult", size = v2f( 216.0, 50.0 ) }
textures.bonus_dragon   = { path = textures_theme .. "bonus/dragon", size = v2f( 158.5, 141.0 ), collides = true }
textures.bonus_gargoyle = { path = textures_theme .. "bonus/gargoyle", size = v2f( 110.5, 128.0 ), collides = true }
textures.bonus_archer   = { path = textures_theme .. "bonus/archer", size = v2f( 63.0, 72.0 ), collides = true }

-- enemies --
textures.new_high_score   = { path = textures_theme .. "enemies/new_high_score", size = v2f( 320.0, 56.5 ) }
textures.drawbridge       = { path = textures_theme .. "enemies/drawbridge", size = v2f( 114.0, 57.0 ), collides = true }
textures.knight           = { path = textures_theme .. "enemies/knight", size = v2f( 46.0, 65.0 ), collides = true }
textures.horse            = { path = textures_theme .. "enemies/horse", size = v2f( 64.0, 96.0 ), collides = true }
textures.chain            = { path = textures_theme .. "enemies/chain", size = v2f( 256.0, 6.0 ) }
textures.troll            = { path = textures_theme .. "enemies/troll", size = v2f( 64.0, 56.5 ), collides = true }
textures.bridge           = { path = textures_theme .. "enemies/bridge", size = v2f( 1.0, 1.0 ) }
textures.archer           = { path = textures_theme .. "enemies/archer", size = v2f( 57.0, 83.5 ), collides = true }
textures.arrow            = { path = textures_theme .. "enemies/arrow", size = v2f( 45.5, 12.5 ), collides = true }
textures.gargoyle_frozen  = { path = textures_theme .. "enemies/gargoyle_frozen", size = v2f( 78.0, 85.0 ) }
textures.gargoyle         = { path = textures_theme .. "enemies/gargoyle", size = v2f( 60.0, 97.5 ), collides = true }
textures.catapult         = { path = textures_theme .. "enemies/catapult", size = v2f( 89.5, 97.5 ), collides = true }
textures.boulder          = { path = textures_theme .. "enemies/boulder", size = v2f( 24.5, 22.5 ), collides = true }
textures.dragon           = { path = textures_theme .. "enemies/dragon", size = v2f( 201.0, 182.0 ) }
textures.dragon_wing      = { path = textures_theme .. "enemies/dragon_wing", size = v2f( 199.5, 128.0 ) }
textures.fireball         = { path = textures_theme .. "enemies/fireball", size = v2f( 50.0, 50.0 ), collides = true }
textures.new_high_score_locale  = { path = textures_theme .. "enemies/new_high_score_locale", size = v2f( 320.0, 59.0 ) }

-- particles --
textures.particles_smoke    = { path = textures_theme .. "particles/smoke", size = v2f( 47.5, 41.5 ) }
textures.particles_fireball = { path = textures_theme .. "particles/fireball", size = v2f( 32.0, 32.0 ) }
textures.particles_feather  = { path = textures_theme .. "particles/feather", size = v2f( 11.5, 32.0 ) }
textures.particles_splinter = { path = textures_theme .. "particles/splinter", size = v2f( 21.5, 3.5 ) }
textures.particles_splinter = { path = textures_theme .. "particles/splinter", size = v2f( 21.5, 3.5 ) }
textures.particles_rock1    = { path = textures_theme .. "particles/rock1", size = v2f( 7, 7.5 ) }
textures.particles_rock2    = { path = textures_theme .. "particles/rock2", size = v2f( 8.5, 7.5 ) }
textures.particles_club     = { path = textures_theme .. "particles/club", size = v2f( 32.0, 26.0 ) }
textures.particles_gargoyle = { path = textures_theme .. "particles/gargoyle", size = v2f( 32.0, 22.0 ) }

textures.email_dx = { path = textures_theme .. "email_dx", size = v2f(320.0, 320.0) }

------------
-- meshes --
------------

local meshes_path = engine.paths.meshes

meshes = {}

-- backgrounds
meshes.bg_back   = { path = meshes_path .. "bg_back" }
meshes.bg_middle = { path = meshes_path .. "bg_middle" }
meshes.bg_front  = { path = meshes_path .. "bg_front" }
meshes.bridge    = { path = meshes_path .. "bridge" }

------------
-- sounds --
------------

local sounds_theme = engine.paths.sounds_theme

sounds = {}

-- bonuses --
sounds.bonus_gargoyle = { path = sounds_theme .. "bonus_gargoyle.ogg", priority = 100.0 }
sounds.bonus_catapult = { path = sounds_theme .. "bonus_catapult.ogg", priority = 100.0 }
sounds.bonus_troll    = { path = sounds_theme .. "bonus_troll.ogg",    priority = 100.0 }
sounds.bonus_dragon   = { path = sounds_theme .. "bonus_dragon.ogg",   priority = 100.0 }
sounds.bonus_archer   =
{
	{ path = sounds_theme .. "archer-bonus-1.ogg", priority = 100.0 },
	{ path = sounds_theme .. "archer-bonus-2.ogg", priority = 100.0 },
	{ path = sounds_theme .. "archer-bonus-3.ogg", priority = 100.0 }
}

-- enemies --
sounds.whinny =
{
	{ path = sounds_theme .. "whinny1.ogg" },
	{ path = sounds_theme .. "whinny2.ogg" },
	{ path = sounds_theme .. "whinny3.ogg" }
}
sounds.dragon_enter =
{
	{ path = sounds_theme .. "dragon_enter1.ogg" },
	{ path = sounds_theme .. "dragon_enter2.ogg" },
	{ path = sounds_theme .. "dragon_enter3.ogg" }
}
sounds.dragon_spit =
{
	{ path = sounds_theme .. "dragon_spit1.ogg" },
	{ path = sounds_theme .. "dragon_spit2.ogg" },
	{ path = sounds_theme .. "dragon_spit3.ogg" }
}
sounds.hit_fireball =
{
	{ path = sounds_theme .. "hit_fireball1.ogg" },
	{ path = sounds_theme .. "hit_fireball2.ogg" },
	{ path = sounds_theme .. "hit_fireball3.ogg" }
}
sounds.catapult =
{
	{ path = sounds_theme .. "catapult1.ogg" },
	{ path = sounds_theme .. "catapult2.ogg" },
	{ path = sounds_theme .. "catapult3.ogg" }
}
sounds.hit_boulder =
{
	{ path = sounds_theme .. "hit_boulder1.ogg" },
	{ path = sounds_theme .. "hit_boulder2.ogg" },
	{ path = sounds_theme .. "hit_boulder3.ogg" }
}
sounds.shoot_arrow =
{
	{ path = sounds_theme .. "shoot_arrow1.ogg" },
	{ path = sounds_theme .. "shoot_arrow2.ogg" },
	{ path = sounds_theme .. "shoot_arrow3.ogg" }
}
sounds.hit_arrow =
{
	{ path = sounds_theme .. "hit_arrow1.ogg" },
	{ path = sounds_theme .. "hit_arrow2.ogg" },
	{ path = sounds_theme .. "hit_arrow3.ogg" }
}
sounds.troll_growl =
{
	{ path = sounds_theme .. "troll_growl1.ogg" },
	{ path = sounds_theme .. "troll_growl2.ogg" },
	{ path = sounds_theme .. "troll_growl3.ogg" }
}
sounds.hit_troll =
{
	{ path = sounds_theme .. "hit-troll-1.ogg" },
	{ path = sounds_theme .. "hit-troll-2.ogg" },
	{ path = sounds_theme .. "hit-troll-3.ogg" }
}
sounds.gargoyle_awake =
{
	{ path = sounds_theme .. "gargoyle_awake_1.ogg" },
	{ path = sounds_theme .. "gargoyle_awake_2.ogg" },
	{ path = sounds_theme .. "gargoyle_awake_3.ogg" }
}
sounds.hit_gargoyle =
{
	{ path = sounds_theme .. "hit-gargoyle-1.ogg" },
	{ path = sounds_theme .. "hit-gargoyle-2.ogg" },
	{ path = sounds_theme .. "hit-gargoyle-3.ogg" }
}

-----------
-- music --
-----------

local music_theme = engine.paths.music_theme

music = {}

-- menu --
music.intro = { path = music_theme .. "intro", loops = false, volume = 1.0 }
music.loop  = { path = music_theme .. "loop", loops = true, volume = 1.0 }



