------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Assets             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local world_size = engine.platform.constants.scene.world_size

--------------
-- textures --
--------------

local textures_theme = engine.paths.textures_theme

textures = {}

-- foreground --
textures.foreground = { path = textures_theme .. "foreground", size = world_size }

-- background --
textures.background = { path = textures_theme .. "background", size = world_size }

-- wall --
textures.wall = { path = textures_theme .. "wall" }

-- hud --
textures.pause_menu = { path = textures_theme .. "hud/pause_menu", size = v2f( 320.0, 377.0 ) }

-- bonus --
textures.bonus_parrot     = { path = textures_theme .. "bonus/parrot", size = v2f( 128.0, 103.5 ), collides = true }
textures.bonus_cannon     = { path = textures_theme .. "bonus/cannon", size = v2f( 116.0, 128.0 ) }
textures.bonus_cannonball = { path = textures_theme .. "bonus/cannonball", size = v2f( 52.5, 128.0 ), collides = true }
textures.bonus_hooker     = { path = textures_theme .. "bonus/hooker", size = v2f( 86.0, 128.0 ), collides = true }
textures.bonus_fountain   = { path = textures_theme .. "bonus/kraken_fountain", size = v2f( 256.0, 190.0 ) }
textures.bonus_swinger    = { path = textures_theme .. "bonus/swinger", size = v2f( 128.0, 42.5 ), collides = true }

-- enemies --
textures.new_high_score  = { path = textures_theme .. "enemies/new_high_score", size = v2f( 320.0, 56.0 ) }
textures.foliage1        = { path = textures_theme .. "enemies/foliage1", size = v2f( 34.0, 44.0 ) }
textures.foliage2        = { path = textures_theme .. "enemies/foliage2", size = v2f( 34.0, 63.0 ) }
textures.mast            = { path = textures_theme .. "enemies/mast", size = v2f( 117.0, 53.0 ), collides = true }
textures.crows_nest      = { path = textures_theme .. "enemies/crows_nest", size = v2f( 61.0, 78.0 ), collides = true }
textures.cannon          = { path = textures_theme .. "enemies/cannon", size = v2f( 58.0, 117.0 ), collides = true }
textures.cannonball      = { path = textures_theme .. "enemies/cannonball", size = v2f( 22.0, 22.0 ), collides = true }
textures.parrot          = { path = textures_theme .. "enemies/parrot", size = v2f( 39.5, 64.0 ), collides = true }
textures.rope            = { path = textures_theme .. "enemies/rope", size = v2f( 256.0, 6.0 ) }
textures.hooker          = { path = textures_theme .. "enemies/hooker", size = v2f( 64.0, 64.0 ), collides = true }
textures.kraken          = { path = textures_theme .. "enemies/kraken", size = v2f( 93.5, 128.0 ) }
textures.kraken_injured  = { path = textures_theme .. "enemies/kraken_injured", size = v2f( 93.5, 128.0 ) }
textures.tentacle_top    = { path = textures_theme .. "enemies/tentacle_top", size = v2f( 87.5, 152.5 ), collides = true }
textures.tentacle_bottom = { path = textures_theme .. "enemies/tentacle_bottom", size = v2f( 178.0, 106.0 ) }
textures.swinger         = { path = textures_theme .. "enemies/swinger", size = v2f( 64.0, 64.0 ), collides = true }
textures.icon_hook       = { path = textures_theme .. "enemies/icon_hook", size = v2f( 64.0, 64.0 ) }
textures.icon_sword      = { path = textures_theme .. "enemies/icon_sword", size = v2f( 64.0, 64.0 ) }
textures.icon_eye        = { path = textures_theme .. "enemies/icon_eye", size = v2f( 64.0, 64.0 ) }
textures.new_high_score_locale  = { path = textures_theme .. "enemies/new_high_score_locale", size = v2f( 320.0, 59.0 ) }

-- particles --
textures.particles_droplet    = { path = textures_theme .. "particles/droplet", size = v2f( 32.0, 32.0 ) }
textures.particles_smoke      = { path = textures_theme .. "particles/smoke", size = v2f( 64.0, 56.0 ) }
textures.particles_cannonball = { path = textures_theme .. "particles/cannonball", size = v2f( 11.0, 22.0 ) } 
textures.particles_feather    = { path = textures_theme .. "particles/feather", size = v2f( 14.0, 32.0 ) } 
textures.particles_fish_steak = { path = textures_theme .. "particles/fish_steak", size = v2f( 20.0, 32.0 ) } 
textures.particles_flesh      = { path = textures_theme .. "particles/flesh", size = v2f( 26.0, 32.0 ) } 
textures.particles_crossbones = { path = textures_theme .. "particles/crossbones", size = v2f( 32.0, 32.0 ) }

textures.email_dx = { path = textures_theme .. "email_dx", size = v2f(320.0, 320.0) }

------------
-- meshes --
------------

local meshes_path = engine.paths.meshes

meshes = {}

-- backgrounds
meshes.bg_back          = { path = meshes_path .. "bg_back" }
meshes.bg_back_swirls   = { path = meshes_path .. "bg_back_swirls" }
meshes.bg_middle        = { path = meshes_path .. "bg_middle" }
meshes.bg_middle_swirls = { path = meshes_path .. "bg_middle_swirls" }
meshes.bg_front         = { path = meshes_path .. "bg_front" }
meshes.bg_front_swirls  = { path = meshes_path .. "bg_front_swirls" }

------------
-- sounds --
------------

local sounds_theme = engine.paths.sounds_theme

sounds = {}

-- ninja --
sounds.bonus_trigger_parrot  = { path = sounds_theme .. "bonus_trigger_parrot.ogg", priority = 100.0 }
sounds.bonus_loop_parrot     = { path = sounds_theme .. "bonus_loop_parrot.ogg", priority = 100.0 }
sounds.bonus_trigger_swinger = { path = sounds_theme .. "bonus_trigger_swinger.ogg", priority = 100.0 }
sounds.bonus_loop_swinger    = { path = sounds_theme .. "bonus_loop_swinger.ogg", priority = 100.0 }
sounds.bonus_trigger_cannon  = { path = sounds_theme .. "bonus_trigger_cannon.ogg", priority = 100.0 }
sounds.bonus_trigger_kraken = { path = sounds_theme .. "bonus_trigger_kraken.ogg", priority = 100.0 }
sounds.bonus_loop_kraken    = { path = sounds_theme .. "bonus_loop_kraken.ogg", priority = 100.0 }
sounds.bonus_hooker = 
{
	{ path = sounds_theme .. "bonus_hooker_1.ogg", priority = 100.0 },
	{ path = sounds_theme .. "bonus_hooker_2.ogg", priority = 100.0 },
	{ path = sounds_theme .. "bonus_hooker_3.ogg", priority = 100.0 }
}

-- enemies --
sounds.hit_cannonball =
{
	{ path = sounds_theme .. "hit_cannonball_1.ogg" },
	{ path = sounds_theme .. "hit_cannonball_2.ogg" },
	{ path = sounds_theme .. "hit_cannonball_3.ogg" }
}
sounds.cannon_fire =
{
	{ path = sounds_theme .. "cannon_fire_1.ogg" },
	{ path = sounds_theme .. "cannon_fire_2.ogg" },
	{ path = sounds_theme .. "cannon_fire_3.ogg" }
}
sounds.hit_hooker =
{
	{ path = sounds_theme .. "hit_hooker_1.ogg" },
	{ path = sounds_theme .. "hit_hooker_2.ogg" },
	{ path = sounds_theme .. "hit_hooker_3.ogg" }
}
sounds.swinger_enter =
{
	{ path = sounds_theme .. "swinger_enter_1.ogg" },
	{ path = sounds_theme .. "swinger_enter_2.ogg" },
	{ path = sounds_theme .. "swinger_enter_3.ogg" }
}
sounds.hit_swinger =
{
	{ path = sounds_theme .. "hit_swinger_1.ogg" },
	{ path = sounds_theme .. "hit_swinger_2.ogg" },
	{ path = sounds_theme .. "hit_swinger_3.ogg" }
}
sounds.hit_parrot =
{
	{ path = sounds_theme .. "hit_parrot_1.ogg" },
	{ path = sounds_theme .. "hit_parrot_2.ogg" },
	{ path = sounds_theme .. "hit_parrot_3.ogg" }
}
sounds.hit_kraken =
{
	{ path = sounds_theme .. "hit_kraken_1.ogg" },
	{ path = sounds_theme .. "hit_kraken_2.ogg" },
	{ path = sounds_theme .. "hit_kraken_3.ogg" }
}
sounds.kraken_warning =
{
	{ path = sounds_theme .. "kraken_warning_1.ogg" },
	{ path = sounds_theme .. "kraken_warning_2.ogg" },
	{ path = sounds_theme .. "kraken_warning_3.ogg" }
}

-----------
-- music --
-----------

local music_theme = engine.paths.music_theme

music = {}

-- menu --
music.intro = { path = music_theme .. "intro", loops = false, volume = 1.0 }
music.loop  = { path = music_theme .. "loop", loops = true, volume = 1.0 }


