------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Assets             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local world_size = engine.platform.constants.scene.world_size

--------------
-- textures --
--------------

local textures_theme = engine.paths.textures_theme

textures = {}

-- foreground --
textures.foreground = { path = textures_theme .. "foreground", size = world_size }

-- background --
textures.background = { path = textures_theme .. "background", size = world_size }

-- wall --
textures.wall = { path = textures_theme .. "wall" }

-- hud --
textures.pause_menu = { path = textures_theme .. "hud/pause_menu", size = v2f( 320.0, 377.0 ) }

-- bonus --
textures.bonus_frog_still = { path = textures_theme .. "bonus/frog_still", size = v2f( 64.0, 56.5 ), collides = true }
textures.bonus_frog_blur  = { path = textures_theme .. "bonus/frog_blur", size = v2f( 64.0, 34.0 ), collides = true }
textures.bonus_dart       = { path = textures_theme .. "bonus/dart", size = v2f( 128.0, 64.0 ), collides = true }
textures.bonus_beetle     = { path = textures_theme .. "bonus/beetle", size = v2f( 128.0, 128.0 ), collides = true }
textures.bonus_bee        = { path = textures_theme .. "bonus/bee", size = v2f( 112.5, 83.0 ), collides = true }

-- enemies --
textures.new_high_score = { path = textures_theme .. "enemies/new_high_score", size = v2f( 320.0, 56.0 ) }
textures.bridge         = { path = textures_theme .. "enemies/bridge", size = v2f( 260.0, 51.1875 ) }
textures.frog           = { path = textures_theme .. "enemies/frog", size = v2f( 42.5, 37.5 ), collides = true }
textures.foliage1       = { path = textures_theme .. "enemies/foliage1", size = v2f( 1.0, 1.0 ) }
textures.foliage2       = { path = textures_theme .. "enemies/foliage2", size = v2f( 1.0, 1.0 ) }
textures.foliage3       = { path = textures_theme .. "enemies/foliage3", size = v2f( 1.0, 1.0 ) }
textures.witch          = { path = textures_theme .. "enemies/witch", size = v2f( 40.0, 40.0 ), collides = true }
textures.dart           = { path = textures_theme .. "enemies/dart", size = v2f( 48.0, 32.0 ), collides = true }
textures.monkey         = { path = textures_theme .. "enemies/monkey", size = v2f( 44.5, 44.5 ), collides = true }
textures.beehive        = { path = textures_theme .. "enemies/beehive", size = v2f( 25.5, 26.0 ), collides = true }
textures.beetle         = { path = textures_theme .. "enemies/beetle", size = v2f( 50.0, 50.0 ), collides = true }
textures.snake_head     = { path = textures_theme .. "enemies/snake_head", size = v2f( 32.0, 48.5 ), collides = true }
textures.snake_mid      = { path = textures_theme .. "enemies/snake_mid", size = v2f( 32.0, 43.0 ), collides = true }
textures.snake_tail     = { path = textures_theme .. "enemies/snake_tail", size = v2f( 32.0, 36.5 ), collides = true }
textures.branch         = { path = textures_theme .. "enemies/branch", size = v2f( 125.0, 72.0 ), collides = true }
textures.bee            = { path = textures_theme .. "enemies/bee", size = v2f( 32.0, 21.0 ), collides = true }
textures.flytrap        = { path = textures_theme .. "enemies/flytrap", size = v2f( 64.0, 61.0 ), collides = true }
textures.new_high_score_locale  = { path = textures_theme .. "enemies/new_high_score_locale", size = v2f( 320.0, 59.0 ) }

-- particles --
textures.particles_smoke   = { path = textures_theme .. "particles/smoke", size = v2f( 64.0, 56.0 ) }
textures.particles_trippy  = { path = textures_theme .. "particles/trippy", size = v2f( 16.0, 16.0 ) }
textures.particles_leaf    = { path = textures_theme .. "particles/leaf", size = v2f( 24.0, 32.0 ) }
textures.particles_wing_1  = { path = textures_theme .. "particles/wing_1", size = v2f( 26.0, 32.0 ) }
textures.particles_wing_2  = { path = textures_theme .. "particles/wing_2", size = v2f( 19.0, 32.0 ) }
textures.particles_feather = { path = textures_theme .. "particles/feather", size = v2f( 11.0, 32.0 ) } 
textures.particles_honey   = { path = textures_theme .. "particles/honey", size = v2f( 32.0, 32.0 ) }

-- effects --
textures.effects_trippy = { path = textures_theme .. "effects/trippy", size = v2f( 64.0, 64.0 ) }
textures.effects_honey  = { path = textures_theme .. "effects/honey", size = v2f( 64.0, 57.0 ) }

textures.email_dx = { path = textures_theme .. "email_dx", size = v2f(320.0, 320.0) }

------------
-- meshes --
------------

local meshes_path = engine.paths.meshes

meshes = {}

-- backgrounds
meshes.bg_back   = { path = meshes_path .. "bg_back" }
meshes.bg_middle = { path = meshes_path .. "bg_middle" }
meshes.bg_front  = { path = meshes_path .. "bg_front" }
meshes.foliage1  = { path = meshes_path .. "foliage1" }
meshes.foliage2  = { path = meshes_path .. "foliage2" }
meshes.foliage3  = { path = meshes_path .. "foliage3" }

------------
-- sounds --
------------

local sounds_theme = engine.paths.sounds_theme

sounds = {}

-- ninja --
sounds.bonus_trigger_beetle  = { path = sounds_theme .. "bonus_trigger_beetle.ogg", priority = 100.0 }
sounds.bonus_loop_beetle     = { path = sounds_theme .. "bonus_loop_beetle.ogg", priority = 100.0 }
sounds.bonus_trigger_frog    = { path = sounds_theme .. "bonus_trigger_frog.ogg", priority = 100.0 }
sounds.bonus_frog_swoosh     = { path = sounds_theme .. "bonus_frog_swoosh.ogg", priority = 100.0 }
sounds.bonus_trigger_bee     = { path = sounds_theme .. "bonus_trigger_bee.ogg", priority = 100.0 }
sounds.bonus_loop_bee        = { path = sounds_theme .. "bonus_loop_bee.ogg", priority = 100.0 }
sounds.bonus_trigger_beehive = { path = sounds_theme .. "bonus_trigger_beehive.ogg", priority = 100.0 }
sounds.bonus_loop_beehive    = { path = sounds_theme .. "bonus_loop_beehive.ogg", priority = 100.0 }
sounds.bonus_trigger_dart    = { path = sounds_theme .. "bonus_trigger_dart.ogg", priority = 100.0 }
sounds.bonus_loop_dart       = { path = sounds_theme .. "bonus_loop_dart.ogg", priority = 100.0 }

-- enemies --
sounds.hit_frog =
{
	{ path = sounds_theme .. "hit_frog_1.ogg" },
	{ path = sounds_theme .. "hit_frog_2.ogg" },
	{ path = sounds_theme .. "hit_frog_3.ogg" }
}
sounds.beetle_enter =
{
	{ path = sounds_theme .. "beetle_enter_1.ogg" },
	{ path = sounds_theme .. "beetle_enter_2.ogg" },
	{ path = sounds_theme .. "beetle_enter_3.ogg" }
}
sounds.hit_beetle =
{
	{ path = sounds_theme .. "hit_beetle_1.ogg" },
	{ path = sounds_theme .. "hit_beetle_2.ogg" },
	{ path = sounds_theme .. "hit_beetle_3.ogg" }
}
sounds.shoot_dart =
{
	{ path = sounds_theme .. "shoot_dart_1.ogg" },
	{ path = sounds_theme .. "shoot_dart_2.ogg" },
	{ path = sounds_theme .. "shoot_dart_3.ogg" }
}
sounds.snake_hiss =
{
	{ path = sounds_theme .. "snake_hiss_1.ogg" },
	{ path = sounds_theme .. "snake_hiss_2.ogg" },
	{ path = sounds_theme .. "snake_hiss_3.ogg" }
}
sounds.beehive_explode =
{
	{ path = sounds_theme .. "beehive_explode_1.ogg", priority = 10.0 },
	{ path = sounds_theme .. "beehive_explode_2.ogg", priority = 10.0 },
	{ path = sounds_theme .. "beehive_explode_3.ogg", priority = 10.0 }
}
sounds.monkey_enter =
{
	{ path = sounds_theme .. "monkey_enter_1.ogg" },
	{ path = sounds_theme .. "monkey_enter_2.ogg" },
	{ path = sounds_theme .. "monkey_enter_3.ogg" }
}
sounds.hit_bee =
{
	{ path = sounds_theme .. "hit_bee_1.ogg" },
	{ path = sounds_theme .. "hit_bee_2.ogg" },
	{ path = sounds_theme .. "hit_bee_3.ogg" }
}
sounds.hit_beehive =
{
	{ path = sounds_theme .. "hit_beehive_1.ogg" },
	{ path = sounds_theme .. "hit_beehive_2.ogg" },
	{ path = sounds_theme .. "hit_beehive_3.ogg" }
}
sounds.hit_dart =
{
	{ path = sounds_theme .. "hit_dart_1.ogg" },
	{ path = sounds_theme .. "hit_dart_2.ogg" },
	{ path = sounds_theme .. "hit_dart_3.ogg" }
}

-----------
-- music --
-----------

local music_theme = engine.paths.music_theme

music = {}

-- menu --
music.intro = { path = music_theme .. "intro", loops = false, volume = 1.0 }
music.loop  = { path = music_theme .. "loop", loops = true, volume = 1.0 }

