------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Assets             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local world_size = engine.platform.constants.scene.world_size

--------------
-- textures --
--------------

local textures_theme  = engine.paths.textures_theme

textures = {}

-- foreground --
textures.foreground = { path = textures_theme .. "foreground", size = world_size }

-- background --
textures.background = { path = textures_theme .. "background", size = world_size }

-- wall --
textures.wall = { path = textures_theme .. "wall" }

-- hud --
textures.pause_menu = { path = textures_theme .. "hud/pause_menu", size = v2f( 320.0, 377.0 ) }

-- bonus --
textures.bonus_star     = { path = textures_theme .. "bonus/star", size = v2f( 64.0, 64.0 ), collides = true }
textures.bonus_bird     = { path = textures_theme .. "bonus/bird", size = v2f( 125.0, 101.0 ), collides = true }
textures.bonus_squirrel = { path = textures_theme .. "bonus/squirrel", size = v2f( 85.0, 85.0 ), collides = true }
textures.bonus_bomb     = { path = textures_theme .. "bonus/bomb", size = v2f( 55.0, 160.0 ), collides = true }

-- enemies --
textures.laundry        = { path = textures_theme .. "enemies/laundry", size = v2f( 260.0, 64.0 ) }
textures.balcony        = { path = textures_theme .. "enemies/balcony", size = v2f( 55.0, 83.0 ), collides = true }
textures.overhang       = { path = textures_theme .. "enemies/overhang", size = v2f( 99.0, 60.0 ), collides = true }
textures.thrower        = { path = textures_theme .. "enemies/thrower", size = v2f( 40.0, 40.0 ), collides = true }
textures.climber        = { path = textures_theme .. "enemies/climber", size = v2f( 60.0, 60.0 ), collides = true }
textures.star           = { path = textures_theme .. "enemies/star", size = v2f( 30.0, 30.0 ), collides = true }
textures.squirrel       = { path = textures_theme .. "enemies/squirrel", size = v2f( 50.0, 25.0 ), collides = true }
textures.bird           = { path = textures_theme .. "enemies/bird", size = v2f( 45.0, 45.0 ), collides = true }
textures.bomb           = { path = textures_theme .. "enemies/bomb", size = v2f( 32.0, 48.0 ), collides = true }
textures.new_high_score = { path = textures_theme .. "enemies/new_high_score", size = v2f( 320.0, 56.0 ) }
textures.new_high_score_locale  = { path = textures_theme .. "enemies/new_high_score_locale", size = v2f( 320.0, 59.0 ) }

-- particles --
textures.particles_smoke   = { path = textures_theme .. "particles/smoke", size = v2f( 45.0, 39.0 ) }
textures.particles_fur     = { path = textures_theme .. "particles/fur", size = v2f( 25.0, 21.0 ) }
textures.particles_feather = { path = textures_theme .. "particles/feather", size = v2f( 20.0, 33.0 ) }
textures.particles_bomb    = { path = textures_theme .. "particles/bomb", size = v2f( 11.0, 11.0 ) }

textures.email_dx = { path = textures_theme .. "email_dx", size = v2f(320.0, 320.0) } -- Deluxe image
textures.email = { path = textures_theme .. "email", size = v2f(320.0, 320.0) } -- Classic image

------------
-- meshes --
------------

local meshes_path = engine.paths.meshes

meshes = {}

-- backgrounds
meshes.bg_back   = { path = meshes_path .. "bg_back" }
meshes.bg_middle = { path = meshes_path .. "bg_middle" }
meshes.bg_front  = { path = meshes_path .. "bg_front" }

------------
-- sounds --
------------

local sounds_theme = engine.paths.sounds_theme

sounds = {}

-- ninja --
sounds.bonus_trigger_bird     = { path = sounds_theme .. "bonus_trigger_bird.ogg", priority = 100.0 }
sounds.bonus_loop_bird        = { path = sounds_theme .. "bonus_loop_bird.ogg", priority = 100.0 }
sounds.bonus_trigger_star     = { path = sounds_theme .. "bonus_trigger_star.ogg", priority = 100.0 }
sounds.bonus_loop_star        = { path = sounds_theme .. "bonus_loop_star.ogg", priority = 100.0 }
sounds.bonus_trigger_squirrel = { path = sounds_theme .. "bonus_trigger_squirrel.ogg", priority = 100.0 }
sounds.bonus_loop_squirrel    = { path = sounds_theme .. "bonus_loop_squirrel.ogg", priority = 100.0 }


-- enemies --
sounds.hit_star =
{
	{ path = sounds_theme .. "hit_star_1.ogg" },
	{ path = sounds_theme .. "hit_star_2.ogg" },
	{ path = sounds_theme .. "hit_star_3.ogg" }
}
sounds.hit_squirrel =
{
	{ path = sounds_theme .. "hit_squirrel_1.ogg" },
	{ path = sounds_theme .. "hit_squirrel_2.ogg" },
	{ path = sounds_theme .. "hit_squirrel_3.ogg" }
}
sounds.hit_bomb =
{
	{ path = sounds_theme .. "hit_bomb_1.ogg" },
	{ path = sounds_theme .. "hit_bomb_2.ogg" },
	{ path = sounds_theme .. "hit_bomb_3.ogg" }
}
sounds.hit_bird =
{
	{ path = sounds_theme .. "hit_bird_1.ogg" },
	{ path = sounds_theme .. "hit_bird_2.ogg" },
	{ path = sounds_theme .. "hit_bird_3.ogg" }
}
sounds.fuse = { path = sounds_theme .. "fuse.ogg", loops = true }

-----------
-- music --
-----------

local music_theme = engine.paths.music_theme

music = {}

-- menu --
music.intro = { path = music_theme .. "intro", loops = false, volume = 1.0 }
music.loop  = { path = music_theme .. "loop", loops = true, volume = 1.0 }


