------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Background         --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine   = engine.instance
local chunks   = engine.chunks
local assets   = chunks.assets
local pool     = chunks.pool
local textures = chunks.textures
local constants = engine.platform.constants

-- LuaBackground --
LuaBackground = chunks.class.new
{

}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBackground )

-- The created method.
function LuaBackground:created( owner )
	local game = owner:getGame()
	
	-- Initialize owner.
	owner.location = pool.get_v3f( 0, 0, -100 )
	owner.hidden   = true
	
	-- Spawn the background image.
	do
		local actor = owner:spawn( SpriteActor() )
		
		textures.setTexture( actor, assets.textures.background )
	end
	
	-- Function to spawn a building.
	local spawnBuilding = function( mesh, color, location, lifetime )
		local actor = owner:spawn( ActorBase() )
		
		actor.mesh     = mesh.path
		actor.texture  = "builtin://white"
		actor.color    = color
		actor.location = location
		actor.static   = true
		actor.scale    = pool.get_v3f( 1.25, 1.25, 1.0 )
		
		do
			local s = SequenceAction()
			
			s:addAction( MoveOffsetAction( lifetime, InterpAction.Lerp, pool.get_v3f( 0.0, -480.0, 0.0 ) ) )
			s:addAction( DestroyAction( 0.0 ) )
			
			actor:addAction( s )
		end
	end
	
	-- buildings --
	spawnBuilding( assets.meshes.bg_back, pool.get_colorf( 0.97, 0.76, 0.32, 1.0 ), 
	               pool.get_v3f( 0.0, -52.0, 0.5 ), 250.0 )
	spawnBuilding( assets.meshes.bg_middle, pool.get_colorf( 0.87, 0.65, 0.23, 1.0 ), 
	               pool.get_v3f( 0.0, -92.0, 1.0 ), 150.0 )
	spawnBuilding( assets.meshes.bg_front, pool.get_colorf( 0.77, 0.55, 0.14, 1.0 ), 
	               pool.get_v3f( 0.0, -152.0, 1.5 ), 100.0 )
end

-- Stop walls from moving.
function LuaBackground:stop()
	local owner = self.owner
	local size  = owner:getNumOwned()
	
	for i = 0,size - 1 do
		local child = owner:getOwned( i )
		
		child:clearActions()
	end
end

--Restart the walls after a boost revive
function LuaBackground:restart()
	local owner = self.owner
	local size  = owner:getNumOwned()

	for i = 1,size - 1 do
		local child = owner:getOwned( i )
		
        local lifetime
        if (i==1) then 
            lifetime = 250
        elseif (i==2) then 
            lifetime = 150 
        elseif (i==3) then 
            lifetime = 100 
        end
        
		child:clearActions()
		local s = SequenceAction()
        s:addAction( MoveOffsetAction( lifetime, InterpAction.Lerp, pool.get_v3f( 0.0, -480.0, 0.0 ) ) )
		s:addAction( DestroyAction( 0.0 ) )
		child:addAction( s )
	end
end
