------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Background         --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine   = engine.instance
local chunks   = engine.chunks
local assets   = chunks.assets
local pool     = chunks.pool
local textures = chunks.textures
local constants = engine.platform.constants

-- LuaBackground --
LuaBackground = chunks.class.new
{

}
-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBackground )

-- The created method.
function LuaBackground:created( owner )
	local game = owner:getGame()

	-- Initialize owner.
	owner.location = pool.get_v3f( 0, 0, -100 )
	owner.hidden   = true

	-- Spawn the background image.
	do
		local actor = owner:spawn( SpriteActor() )

		textures.setTexture( actor, assets.textures.background )
	end
	
	-- Function to spawn a building.
	local spawnBuilding = function( mesh, color, location, lifetime )
		local actor = owner:spawn( ActorBase() )

		actor.mesh     = mesh.path
		actor.texture  = "builtin://white"
		actor.color    = color
		actor.location = location
		actor.static   = true
		actor.scale    = pool.get_v3f( 3.0, 3.0, 1.0 )

		do
			local s = SequenceAction()

			s:addAction( MoveOffsetAction( lifetime, InterpAction.Lerp, pool.get_v3f( 0.0, -480.0, 0.0 ) ) )
			s:addAction( DestroyAction( 0.0 ) )

			actor:addAction( s )
		end
	end

	local function byteColor(r, g, b, a)
		return colorf(r / 255, g / 255, b / 255, (a or 255) / 255)
	end

	
	local yoffset = -242.0
	if constants.WIDESCREEN then
		yoffset = -330
	end
	
	-- buildings --
	spawnBuilding( assets.meshes.bg_back, byteColor(183,156,196), v3f( 0.0, yoffset, 0.5 ), 270.0 )
	spawnBuilding( assets.meshes.bg_middle, byteColor(147,128,168), v3f( 0.0, yoffset, 1.0 ), 120.0 )
	spawnBuilding( assets.meshes.bg_front, byteColor(99,101,150), v3f( 0.0, yoffset, 1.5 ), 60.0 )
	
	
end

-- Stop walls from moving.
function LuaBackground:stop()
	local owner = self.owner
	local size  = owner:getNumOwned()

	for i = 0,size - 1 do
		local child = owner:getOwned( i )

		child:clearActions()
	end
end

--Restart the walls after a boost revive
function LuaBackground:restart()
	local owner = self.owner
	local size  = owner:getNumOwned()

	for i = 1,size - 1 do
		local child = owner:getOwned( i )
		
        local lifetime
        if (i==1) then 
            lifetime = 270
        elseif (i==2) then 
            lifetime = 120 
        elseif (i==3) then 
            lifetime = 60 
        end
        
		child:clearActions()
		local s = SequenceAction()
        s:addAction( MoveOffsetAction( lifetime, InterpAction.Lerp, pool.get_v3f( 0.0, -480.0, 0.0 ) ) )
		s:addAction( DestroyAction( 0.0 ) )
		child:addAction( s )
	end
end

