------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Balcony            --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaBalcony --
LuaBalcony = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	offset = -132.5
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBalcony )

-- The created method.
function LuaBalcony:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "balcony"
	owner.location = pool.get_v3f( self.offset, 0.0, 0.0 )
	
	-- Balcony.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.balcony )

		self.visuals.balcony = actor
	end
	
	self:setVisual( "balcony" )
	
	-- We're not killable.
	self.killable = false
end

-- The collided event.
function LuaBalcony:onCollided( other )
	-- Spawn particles.
	do
		particles.spawn_big( self.owner:getGame().lua.root, 
		                     assets.textures.particles_smoke, 
		                     self.visual:getWorldLocation() )
	end
	
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_overhang )
end

