------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Bonus Meter        --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine   = engine.instance
local chunks   = engine.chunks
local assets   = chunks.assets
local pool     = chunks.pool
local textures = chunks.textures

-- LuaBonusMeter --
LuaBonusMeter = chunks.class.new
{
	last_bonus_type = "",
	indicators      = nil
}
	
-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaBonusMeter )

-- The created method.
function LuaBonusMeter:created( owner )
	self.indicators = {}
end

-- Pause.
function LuaBonusMeter:pause()
	self.owner.delta_time_factor = 0.0
end

-- Unpause.
function LuaBonusMeter:unPause()
	self.owner.delta_time_factor = 1.0
end

-- Clear all indicators.
function LuaBonusMeter:clear()
	for i = 1,#self.indicators do
		local actor = self.indicators[ i ]
		local s     = SequenceAction()
		
		s:addAction( ScaleAction( 0.3, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
		s:addAction( DestroyAction( 0.0 ) )
		
		actor:clearActions()
		actor:addAction( s )
	end
	
	self.indicators = {}
end

-- Update the bonus meter.
function LuaBonusMeter:update( enemy, bonus_count )
	local bonus_type = enemy.label
	
	-- If the enemy type has changed, kill existing indicators.
	if self.last_bonus_type ~= bonus_type then
		self:clear()
	end
	
	-- If the enemy is a bonus enemy, update 
	-- the last bonus type.  Otherwise, clear
	-- our value and exit.
	if enemy.lua.bonus ~= nil then
		self.last_bonus_type = bonus_type
	else
		self.last_bonus_type = ""
		
		return
	end 
	
	-- Spawn a new indicator.
	do
		local root = nil
		
		-- Root.
		do
			root = self.owner:spawn( ActorBase() )
			
			root.location = pool.get_v3f( -35.0 + 35.0 * ( bonus_count - 1 ), 0.0, 1.0 )
			root.scale    = pool.get_v3f( 0.0, 0.0, 1.0 )
			
			-- Scale in.
			do
				local s = SequenceAction()
				
				s:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.7, 0.7, 1.0 ) ) )
				
				root:addAction( s )
			end
			
			-- Add it to the table.
			table.insert( self.indicators, root )
		end
		
		-- Glow.
		do
			local actor = root:spawn( SpriteActor() )
			
			actor.scale = pool.get_v3f( 1.5, 1.5, 1.0 )
			
			textures.setTexture( actor, assets.textures.bonus_meter_glow )
		end
		
		-- The bonus meter image.
		do
			local actor = root:spawn( SpriteActor() )
			
			actor.location = pool.get_v3f( 0.0, 0.0, 2.0 )
			
			if enemy.lua.bonusmeterOverride then
				enemy.lua:bonusmeterOverride( actor )
			else
				local visual = enemy.lua.visual
			
				actor.texture = getRealURI( visual.texture )
				actor.size    = visual.size
			end
			
			-- Add a reference to the visual on the root's lua table.
			root.lua = { visual = actor }
		end
	end
	
	-- Reset animation on all indicators.
	do
		for i = 1,#self.indicators do
			local actor     = self.indicators[ i ]
            local animation = actor.lua.visual.texture.animation
			
            if animation then
				animation:reset()
			end
		end
	end
end

-- Start bonus bob.
function LuaBonusMeter:bonus()
	for i = 1,#self.indicators do
		local actor = self.indicators[ i ]
		local l     = LoopAction()
		
		l:addAction( ScaleAction( 0.3, InterpAction.Smooth, pool.get_v3f( 0.3, 0.3, 1.0 ) ) )
		l:addAction( ScaleAction( 0.3, InterpAction.Smooth, pool.get_v3f( 0.7, 0.7, 1.0 ) ) )		
		
		actor:clearActions()
		actor:addAction( l )
	end
end

	