------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Catapult           --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local effects   = chunks.effects
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaCatapult --
LuaCatapult = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	offset = -115.0
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaCatapult )

-- The created method.
function LuaCatapult:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "catapult"
	owner.location = pool.get_v3f( self.offset, 0.0, 0.0 )
	
	-- Unfired.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.catapult )
		
		self.visuals.catapult = actor
	end
	
	self:setVisual( "catapult" )
	
	-- We're not killable.
	self.killable = false
end

-- The collided event.
function LuaCatapult:onCollided( other )
	local game  = self.owner:getGame()
	local ninja = game.lua.ninja
	
	-- Spawn particles.
	do
		particles.spawn_big( self.owner:getGame().lua.root, 
							 assets.textures.particles_smoke, 
							 self.visual:getWorldLocation() )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_overhang )
end

-- Setup easy cannon.
function LuaCatapult:easy( owner )
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local projectile = owner:spawn( chunks.boulder.spawn() )
			
			projectile.location = actor.location + pool.get_v3f( -17.0, 32.0, 0.0 )
			
			-- Actions
			do
				local s2    = SequenceAction()
				local s3    = SequenceAction()
				local split = SplitAction()
				local delay = utility.frandom( 2.1, 2.4 )
				
				s2:addAction( MoveOffsetAction( delay, InterpAction.Smooth, 
				                                pool.get_v3f( 350.0, 0.0, 0.0 ) ) )
				s3:addAction( MoveOffsetAction( delay * 0.3, InterpAction.Smooth,
				                                pool.get_v3f( 0.0, 60.0, 0.0 ) ) )
				s3:addAction( MoveOffsetAction( delay * 0.7, InterpAction.Smooth,
				                                pool.get_v3f( 0.0, -60.0, 0.0 ) ) )
				split:addAction( s2 )
				split:addAction( s3 )
				
				projectile:addAction( split )
			end
			
			-- Play sound.
			sound.play_random( assets.sounds.catapult )
		end
		
		s:addAction( SleepAction( 0.3 ) )
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end
end

-- Setup hard cannon.
function LuaCatapult:hard( owner )
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local projectile = owner:spawn( chunks.boulder.spawn() )
			
			projectile.location = actor.location + pool.get_v3f( -17.0, 32.0, 0.0 )
			
			-- Actions
			do
				local s2    = SequenceAction()
				local s3    = SequenceAction()
				local split = SplitAction()
				local delay = 1.6 
				
				s2:addAction( MoveOffsetAction( delay, InterpAction.Smooth, 
				                                pool.get_v3f( 350.0, 0.0, 0.0 ) ) )
				s3:addAction( MoveOffsetAction( delay * 0.3, InterpAction.Smooth,
				                                pool.get_v3f( 0.0, 60.0, 0.0 ) ) )
				s3:addAction( MoveOffsetAction( delay * 0.7, InterpAction.Smooth,
				                                pool.get_v3f( 0.0, -60.0, 0.0 ) ) )
				split:addAction( s2 )
				split:addAction( s3 )
				
				projectile:addAction( split )
			end
			
			-- Play sound.
			sound.play_random( assets.sounds.catapult )
		end
		
		s:addAction( SleepAction( 0.55 ) )
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end
end


