------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Constants          --
------------------------

-- module --
module(...,package.seeall)

local world_size_x = 320.0 + ( ( 768.0 - 320.0 * ( 1024.0 / 480.0 ) ) / 2.0 )
	
-- scene camera info
scene = 
{
	world_size      = v2f( world_size_x, 480.0 ),
	clipping_planes = v2f( 500.0, -500.0 )
}

-- window camera info
window = 
{
	world_size      = v2f( world_size_x, 480.0 ),
	clipping_planes = v2f( 500.0, -500.0 )
}

-- other
clear_color    = colorf( 1.0, 1.0, 1.0, 1.0 )
clear_color_on = false
clear_depth_on = true
delta_time_cap = 1.0 / 25.0
ad_height      = 50.0

-- game specific
wall_tile_size = v2f( 32.0 + ( world_size_x - 320.0 ) / 2.0, 50.0 )

-- Menu overrides.
function menu_overrides()
	local engine = engine.instance
	local chunks = engine.chunks
	local assets = chunks.assets 
	
	-- Classic.
	if engine.product == "classic" then
		-- Menus.
		do
			assets.textures.main.size     = v2f( world_size_x, 480.0 )
			assets.textures.options.size  = v2f( world_size_x, 480.0 )
			assets.textures.credits.size  = v2f( world_size_x, 480.0 )
			assets.textures.adbuyout.size = v2f( world_size_x, 480.0 )
			assets.textures.tutorial.size = v2f( world_size_x, 480.0 )
		end
	-- Deluxe.
	elseif engine.product == "deluxe" then
		-- Menus.
		do
			assets.textures.main_classic.size    = v2f( world_size_x, 480.0 )
			assets.textures.main_jungle.size     = v2f( world_size_x, 480.0 )
			assets.textures.main_pirates.size    = v2f( world_size_x, 480.0 )
			assets.textures.main_castle.size     = v2f( world_size_x, 480.0 )
			assets.textures.options_classic.size = v2f( world_size_x, 480.0 )
			assets.textures.options_jungle.size  = v2f( world_size_x, 480.0 )
			assets.textures.options_pirates.size = v2f( world_size_x, 480.0 )
			assets.textures.options_castle.size  = v2f( world_size_x, 480.0 )
			assets.textures.credits.size         = v2f( world_size_x, 480.0 )
			assets.textures.tutorial.size        = v2f( world_size_x, 480.0 )
		end
	end
end

-- Game overrides.
function game_overrides()
	local engine        = engine.instance
	local chunks        = engine.chunks
	local assets_shared = chunks.assets_shared
	local assets        = chunks.assets 
	local meshes_path   = engine.paths.meshes

	-- All themes.
	do
		assets.textures.foreground.size = v2f( world_size_x, 480.0 )
		assets.textures.pause_menu.size = v2f( world_size_x, 377.0 )
	end
	
	-- Classic theme.
	if engine.theme == "classic" then
		-- Override balcony.
		do
			assets.textures.balcony.size     = v2f( 87.0, 83.0 )
			chunks.balcony.LuaBalcony.offset = -148.9
		end
		
		-- Override overhang.
		do
			assets.textures.overhang.size      = v2f( 131.0, 60.0 )
			chunks.overhang.LuaOverhang.offset = -130.5
		end
	-- Jungle theme.
	elseif engine.theme == "jungle" then
		-- Override foliage.
		do
			assets.meshes.foliage1.path       = meshes_path .. "ipad/foliage1" 
			assets.meshes.foliage2.path       = meshes_path .. "ipad/foliage2"
			assets.meshes.foliage3.path       = meshes_path .. "ipad/foliage3"
			chunks.foliage.LuaFoliage.scale   = v3f( 0.5, 0.5, 1.0 )
			chunks.foliage.LuaFoliage.offsets =
			{
				-138.0,
				-137.0,
				-158.0
			}
		end
		
		-- Override branch.
		do
			assets.textures.branch.size    = v2f( 144.5, 71.5 )
			chunks.branch.LuaBranch.offset = -110.0
		end
	-- Pirates theme.
	elseif engine.theme == "pirates" then
		-- Override foliage.
		do
			assets.textures.foliage1.size     = v2f( 57.5, 44.0 ) 
			assets.textures.foliage2.size     = v2f( 56.0, 62.5 ) 
			chunks.foliage.LuaFoliage.offsets = 
			{
				-152.8,
				-153.0
			}
		end
		
		-- Mast.
		do
			assets.textures.mast.size  = v2f( 140.5, 53.0 )
			chunks.mast.LuaMast.offset = -111.5
		end
		
		-- Crows nest.
		do
			assets.textures.crows_nest.size       = v2f( 84.5, 77.5 )
			chunks.crows_nest.LuaCrowsNest.offset = -140.0
		end
		
		-- Cannon.
		do
			assets.textures.cannon.size    = v2f( 82.0, 117.0 )
			chunks.cannon.LuaCannon.offset = -149.0
		end
		
		-- Kraken.
		do
			assets.textures.tentacle_bottom.size = v2f( 202.0, 105.5 )
			chunks.kraken.LuaKraken.offset       = v2f( -50.1, 111.8 )
		end
	-- Castle theme.
	elseif engine.theme == "castle" then
		-- Drawbridge.
		do
			chunks.drawbridge.LuaDrawbridge.offset = -113.5
			assets.textures.drawbridge.size        = v2f( 135.5, 71.5 )
		end
		
		-- Archer.
		do
			assets.textures.archer.size = v2f( 78.0, 85.0 )
			chunks.archer.LuaArcher.offset = -142.0
		end
		
		-- Catapult.
		do
			chunks.catapult.LuaCatapult.offset = -136.0			
		end
	end
end