------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Constants          --
------------------------

-- module --
module(...,package.seeall)

local screen_size = DeviceInfo.get():query( "display.size" )

-- scene camera info
scene = 
{
	world_size      = v2f( 320.0, 480.0 ),
	clipping_planes = v2f( 500.0, -500.0 )
}

-- window camera info
window = 
{
	world_size      = v2f( 320.0, 480.0 ),
	clipping_planes = v2f( 500.0, -500.0 )
}

-- other
clear_color    = colorf( 1.0, 1.0, 1.0, 1.0 )
clear_color_on = false
clear_depth_on = true
delta_time_cap = 1.0 / 25.0
ad_height      = 50.0

-- game specific
wall_tile_size = v2f( 32.0, 50.0 )

local on_back = function( callback )
    return function( self, game, events )
        for event in events:all() do
            local e = AndroidEvent.fast_cast( event )

            if e and e.type == AndroidEvent.AndroidEvent_BackButton then
                callback( game )
            end
        end
    end
end

local function recurse( parent, callback )
    local size = parent:getNumOwned()

    if callback( parent ) then
        return true
    end

    for i = 0, size - 1 do
        if recurse( parent:getOwned( i ), callback ) then
            return true
        end
    end

    return false
end

-- Horrible hacky loading screen for android.
function loading( engine, the_callback )
    local game      = engine.owner:getRoot():spawn( WindowGame() )
    local constants = engine.platform.constants 

    -- Window camera.
    do
        local camera = game:getWindow():spawn( OrthographicCamera() )
        
        camera.world_size      = constants.window.world_size
        camera.clipping_planes = constants.window.clipping_planes
        camera.clear_color_on  = false
        camera.clear_depth_on  = false
        camera.scene           = game:getWindow()

        game:getWindow().camera = camera
    end

    local the_texture = "?://texture/shared/hud/loading"
    local the_size    = v2f( 189.0, 161.0 )

    -- Fade out.
    local fade_out = function( owner, callback )
        local actor = owner:spawn( SpriteActor() )
        
        callback       = callback or function( inner ) end
        actor.texture  = "builtin://white"
        actor.location = v3f( 0.0, 0.0, 100.0 )
        actor.size     = constants.scene.world_size
        actor.color    = colorf( 1.0, 1.0, 1.0, 0.0 )
        
        -- Actions.
        do
            local s = SequenceAction()
            
            s:addAction( AlphaAction( 0.5, InterpAction.Smooth, 1.0 ) )
            s:addAction( LuaAction( 0.0, callback ) )
            
            actor:addAction( s )
        end

        local visual = actor:spawn( SpriteActor() )

        visual.location = v3f( 0, 0, 1 )
        visual.texture  = the_texture
        visual.size     = the_size
        
         --The word "Loading"
        local loading_text = visual:spawn(Text())
        loading_text.location = v3f( 0.0, 58.0, 1.0 )
		local text = lstring( "loading" )
		local format = {"ninja-text", 28.0, colorf(1.0, 0.0, 0.0, 1.0), 100.0, TextAlign.Center, -1.0, 4.0}

		local alignment = TextAlign.Center
		loading_text.style = format[1]
		loading_text.options = {height = format[2], max_width = format[4] or 0.0, align = alignment, localization_scaling_limit = 0.1} 	 	
	
		loading_text.text = text
    end

    local fade_in = function( owner, callback )
    	local actor = owner:spawn( SpriteActor() )
    	
    	callback       = callback or function( inner ) end
    	actor.texture  = "builtin://white"
    	actor.location = v3f( 0.0, 0.0, 100.0 )
    	actor.size     = constants.scene.world_size
    	actor.color    = colorf( 1.0, 1.0, 1.0, 1.0 )
    	
    	-- Actions.
    	do
    		local s = SequenceAction()
    		
    		s:addAction( AlphaAction( 0.5, InterpAction.Smooth, 0.0 ) )
    		s:addAction( LuaAction( 0.0, callback ) )
    		s:addAction( DestroyAction( 0.0 ) )
    		
    		actor:addAction( s )
    	end

        local visual = actor:spawn( SpriteActor() )

        visual.location = v3f( 0, 100, 1 )
        visual.texture  = the_texture
        visual.size     = the_size
        
        --The word "Loading"
        local loading_text = visual:spawn(Text())
        loading_text.location = v3f( 0.0, 58.0, 1.0 )
		local text = lstring( "loading" )
		local format = {"ninja-text", 28.0, colorf(1.0, 0.0, 0.0, 1.0), 100.0, TextAlign.Center, -1.0, 4.0}

		local alignment = TextAlign.Center
		loading_text.style = format[1]
		loading_text.options = {height = format[2], max_width = format[4] or 0.0, align = alignment, localization_scaling_limit = 0.1} 	 	
	
		loading_text.text = text

			
    end

    local final = function() game:destroy() end
    local start = function()
        the_callback()
        fade_in( game:getWindow(), final )
    end

    fade_out( game:getWindow(), start )
end

function menu_overrides()
    local engine = engine.instance
    local chunks = engine.chunks

    -- This is so horrible. :/
    chunks.game.LuaMenuGame.onInputEvents = on_back(
        function( game )
            
            local callback = function( inner )
                if inner.lua and inner.lua.onBack then
                    inner.lua:onBack()

                    return true
                end

                if inner.lua and inner.lua.dismiss then
                    inner.lua:dismiss()

                    return true
                end

                return false
            end

            -- If this wasn't handled by any other
            -- menu window, close the game.
            if not recurse( game, callback ) then
                chunks.events.onShouldExit()
            end
        end
    )
end

function game_overrides()
    local engine = engine.instance
    local chunks = engine.chunks

    -- Moar haxor.
    local existing = chunks.game.LuaMainGame.onInputEvents
    local proxy    = on_back(
        function( game )
            local callback = function( inner )
                if inner.lua and inner.lua.dismiss then
                    inner.lua:dismiss()

                    return true
                end

                return false
            end

            -- If this wasn't handled by any other
            -- menu window, close the game.
            if not recurse( game, callback ) then
                game.lua:onPause()
                
                return true
            end

            return false
        end
    )

    chunks.game.LuaMainGame.onInputEvents = function( self, game, events )
        if not proxy( self, game, events ) then
            existing( self, game, events )
        end
    end
end
