------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- LuaCreditsMenu     --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local constants = engine.platform.constants
local chunks    = engine.chunks
local assets    = chunks.assets
local assets_shared    = chunks.assets_shared
local pool      = chunks.pool
local textures  = chunks.textures
local fade      = chunks.fade
local events    = chunks.events
local utility 	= chunks.utility

-- LuaCreditsMenu --
LuaCreditsMenu = chunks.class.inherit( chunks.window.LuaWindow )()
	
-- The spawner.
spawn = chunks.class.spawner( Window, LuaCreditsMenu )

-- The created method.
function LuaCreditsMenu:created( owner )
	local game     = owner:getGame()
	local x_offset = 5.0
	local y_offset = constants.ad_height + 10.0
	
	-- Background.
	do
		textures.setTexture( owner, assets.textures.credits )
	end
	
	-- Back.
	do
		local on_release = function( inner ) self:onBack() end
		local actor      = self:addButton( on_release )

		actor.anchor = make_anchor( Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f( x_offset, y_offset, 1.0 ) )
		
		textures.setTexture( actor, assets_shared.textures.back_button )
		
		if engine.default_language_enabled then
			local overlay_logo = utility.createWindowWithTexture( self.owner, assets.textures.credits_logo )
			overlay_logo.location = pool.get_v3f( -15.0, 210.0, 1.0 ) 
		else
			utility.createText( owner, lstring("credits"),
				{"ninja-text-large", 35.0, pool.get_colorf(1, 0, 0, 1), 0, TextAlign.Center, -20, 220} )
		end

		local format = {"ninja-text", 35.0, pool.get_colorf(0.0, 1.0, 0.0, 1.0), 47.0, TextAlign.Center, 0.0, 10.0}
		local fx = text_fx or {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}
		
		local credit_text_size = 18.0
		local height_offset = 12.0 + 148

        local label = nil
        local section_break = 8
        local name_break = 4

		utility.createText(self.owner, 
			lstring("backflip_studios"),
			{"ninja-text", 30.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 250.0, TextAlign.Center, -20.0, 185.0}, 
			fx)

		label = utility.createText(self.owner, 
			lstring("executive_director"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("julian_farrior"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - section_break

		label = utility.createText(self.owner, 
			lstring("technical_director"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("dale_thoms"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - section_break

		label = utility.createText(self.owner, 
			lstring("lead_game_designer"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("tom_blind"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - section_break

		label = utility.createText(self.owner, 
			lstring("programming"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("kevin_depue"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("jon_golden"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("ambrose_krapacs"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("curtis_bennett"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - name_break

        label = utility.createText(self.owner, 
            lstring("tye_hooley"),
            {"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
            fx)
        height_offset = height_offset - label.size.y - section_break	

		label = utility.createText(self.owner, 
			lstring("art"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
            height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("ryan_drag"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
            height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("annie_fodge"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
            height_offset = height_offset - label.size.y - name_break

		label = utility.createText(self.owner, 
			lstring("tommy_lhou"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
	    height_offset = height_offset - label.size.y - name_break

	    label = utility.createText(self.owner, 
			lstring("brian_vinton"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - section_break

		label = utility.createText(self.owner, 
			lstring("music_sfx"),
			{"ninja-text-small", credit_text_size - 4, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
        height_offset = height_offset - label.size.y - name_break

		utility.createText(self.owner, 
			lstring("aubrey_hodges"),
			{"ninja-text-small", credit_text_size, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 300.0, TextAlign.Center, -20.0, height_offset}, 
			fx)
			
	end
	
	-- Fade in.
	do
		fade.fade_in( owner )
	end
	
	-- Analytics.
	events.on_credits_menu()
end

function LuaCreditsMenu:onBack()
    local owner    = self.owner
    local game     = owner:getGame()
    local callback = function( o )
        game:getWindow():spawn( chunks.options.spawn() )
        owner:destroy()
    end
			
    fade.fade_out( owner, callback )
end


