------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Death Menu         --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine   = engine.instance
local chunks   = engine.chunks
local assets   = chunks.assets
local pool     = chunks.pool
local textures = chunks.textures
local sound    = chunks.sound
local events   = chunks.events
local utility  = chunks.utility

-- LuaDeathMenu --
LuaDeathMenu = chunks.class.inherit( chunks.window.LuaWindow )
{
    m_play_death_sound = true,
    m_social_handle,
    transition_started = false

}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaDeathMenu )

-- The created event.
function LuaDeathMenu:show()
    local owner = self.owner
    local game  = owner:getGame()

    --In case connection is regained after a missed preload opportunity at the main menu
    if not GameService.get():zeb() then
        chunks.incent_ads.preloadAd("postroll")
        chunks.incent_ads.preloadAd("preroll")
    end
    
    -- Make the ticker disappear.
    game.lua.hud.lua.m_score_bar.hidden_recursive = true

    -- Initialize owner.
    owner.location = pool.get_v3f( 0.0, 55.0, 10.0 )
    textures.setTexture( owner, assets.textures.death_menu )

    if engine.default_language_enabled then
        local overlay_height = utility.createWindowWithTexture( owner, assets.textures.death_menu_height_locale )
        overlay_height.location = pool.get_v3f( -37.0, 90.0, 1.0 )

        local overlay_best = utility.createWindowWithTexture( owner, assets.textures.death_menu_best_locale )
        overlay_best.location = pool.get_v3f( 80.0, 60.0, 1.0 )

    else
        local height_text = utility.createText( owner, lstring("height"),
            {"ninja-text-large", 54.0, pool.get_colorf(1, 0, 0, 1), 150, TextAlign.Center, -35, 98} )

        local best_text = utility.createText( owner, lstring("best"),
            {"ninja-text", 35.0, pool.get_colorf(1, 1, 1, 1), 50, TextAlign.Center, 79, 70, 0.1, 0, true} )
    end

    local score      = game.lua.height
    local high_score = game.lua.best_height

    -- Score text.
    do
        local text = owner:spawn( chunks.ticker.spawn() )

        text.location = pool.get_v3f( -40.0, 44.0, 1.0 )
        text.scale    = pool.get_v3f( 1.2, 1.2, 1.2 )
        text.color    = pool.get_colorf( 1.0, 1.0, 1.0, 1.0 )

        -- If you got a new high score, show it in red.
        if score >= high_score then
            high_score = score
            text.color = pool.get_colorf( 1.0, 0.0, 0.0, 1.0 )
        end

        text.value = score
    end

    -- High score text.
    do
        local text = owner:spawn( chunks.ticker.spawn() )

        text.location = pool.get_v3f( 78.0, 34.0, 1.0 )
        text.scale    = pool.get_v3f( 0.6, 0.6, 0.6 )
        text.color    = pool.get_colorf( 1.0, 0.0, 0.0, 1.0 )
        text.value    = high_score
    end

    -- Menu button.
    do
        local on_release = function( inner )
            self:transitionOut( function( o ) game.lua:onQuit() end )
        end

        local button = nil

        if engine.default_language_enabled then
            button = self:addButton( on_release )
            textures.setTexture( button, assets.textures.menu_button )
        else
            local text = lstring( "menu" )
            local format = {"ninja-text", 20.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 42.0, TextAlign.Center, -1.0, 3.0}
            button = utility.createButtonWithTextAndTexture(owner, text, format, assets.textures.menu_button_locale )
            button.on_release = on_release
        end

        button.anchor = make_anchor( Anchor.Anchor_TR, Anchor.Anchor_TR, pool.get_v3f( -5.0, -10.0, 1.0 ) )

    end

    local y_offset = -15.0
    local spacer   = 50.0
    local x_adjust = 0.0

    -- Leaderboards button.
    local gc = GameCenterService.get()

    if gc then
        local on_release = function( inner )
            if gc:isLoggedIn() then
                events.on_leaderboard_theme()
            else
                --Whoops, user is not logged in.
                if not self.m_social_handle then
                    local callback = function(handle, status, response)
                        self.m_social_handle = nil

                        if status == 0 then
                            events.on_leaderboard_theme()
                        end
                            chunks.utility.showDialog(lstring("gc_relogin_title"),
                                                      lstring("gc_relogin_text"),
                                                      lstring("ok"))
                    end

                    self.m_social_handle = gc:login(callback)
                end
            end
        end

        local button = self:addButton( on_release )

        button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C,
                                pool.get_v3f( -1.0 * spacer / 2.0 - spacer, y_offset, 1.0 ) )


        local info     = DeviceInfo.get()
        local version = info:query( "platform.version.major" )
        if(version < 7)then
            textures.setTexture( button, assets.textures.leaderboards_button )
        else
            print("Texture is ", assets.textures.leaderboards_button_new.path)
            textures.setTexture( button, assets.textures.leaderboards_button_new )
        end
    else
        x_adjust = -25.0
    end

    -- Facebook button.
    do
        local on_release = function( inner )
            if not self.m_social_handle then
                local fb = FacebookService.get()
                if fb then
                    if fb:isLoggedIn() then
                        events.on_facebook(score)
                    else
                        --Login first
                        local callback = function(handle, status, response)
                            self.m_social_handle = nil
                            if status == 0 then
                                events.on_facebook(score)
                            end
                        end

                        self.m_social_handle = fb:login(true, callback)
                    end
                else
                    --FACEBOOK UNAVAILABLE...
                    print("FACEBOOK UNAVAILABLE")

                end
            end
        end

        local button = self:addButton( on_release )

        button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C,
                                     pool.get_v3f((-1.0 * spacer / 2.0) + x_adjust, y_offset, 1.0 ) )

        textures.setTexture( button, assets.textures.facebook_button )
    end

    -- Twitter button.
    if engine.platform.device ~= "android" then
        local on_release = function( inner )
            events.on_twitter(score)
        end

        local button = self:addButton( on_release )

        button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C,
                                     pool.get_v3f( (spacer / 2.0) + x_adjust, y_offset, 1.0 ) )

        textures.setTexture( button, assets.textures.twitter_button )
    end
    

    -- Email button.
    do
        local on_release = function( inner )
            events.on_email(score)
        end

        local button = self:addButton( on_release )

        button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C,
                                     pool.get_v3f( (spacer / 2.0 + spacer) + x_adjust, y_offset, 1.0 ) )

        textures.setTexture( button, assets.textures.email_button )
    end

    -- Replay button.
    do
        local on_release = function( owner )
            events.showGameplayInterstitial(
                function()
                    self:transitionOut( function() game.lua:onReplay() end )
                end
            )
        end

        local button = nil
        if engine.default_language_enabled then
            button = self:addButton( on_release )
            textures.setTexture( button, assets.textures.replay_button )
        else
            local text = lstring( "again" )
            local format = {"ninja-text-large", 70.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 140.0, TextAlign.Center, -2.0, 8.0}
            button = utility.createButtonWithTextAndTexture(owner, text, format, assets.textures.replay_button_locale )
            button.on_release = on_release
        end

        button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C,
                                pool.get_v3f( -27.0, -78.0, 1.0 ) )

        -- Setup a bobbing action.
        do
            local l = LoopAction()

            l:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 1.1, 1.1, 1.0 ) ) )
            l:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )

            button:addAction( l )
        end
    end

    -- The item store button.
    if not GameService.get():zeb() then
        do
            local on_release = function( inner )
                self:openItemStore()
            end
    
            local button = nil
            if engine.default_language_enabled then
                button = self:addButton( on_release )
                textures.setTexture( button, assets.textures.item_store )
            else
                local shields_text = lstring( "shields" )
                local format = {"ninja-text", 35.0, pool.get_colorf(0.86, 0.98, 1.0, 1.0), 105.0, TextAlign.Center, 0.0, 6.0}
                local fx = {drop_shadow = true, shadow_color = colorf( 0.0, 0.27, 0.48, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 100000}
                button = utility.createButtonWithTextAndTexture(self.owner, shields_text, format, assets.textures.item_store_locale, fx )
                button.on_release = on_release
            end
    
            button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -155.0, 1.0 ) )
    
        end
    end

    -- The "try ninjump deluxe" button.
    if engine.product == "classic" then
        local on_release = function( inner )
            events.showNinJumpDeluxe()
        end
        local button = self:addButton( on_release )
        button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -225.0, 1.0 ) )

        if not engine.default_language_enabled then
            textures.setTexture( button, assets.textures.try_deluxe_locale )
            utility.createText( button, lstring( "try" ), {"ninja-text", 30.0, pool.get_colorf(1, 0, 0, 1), 50.0, TextAlign.Center, 0, 26} )
        else
            textures.setTexture( button, assets.textures.try_deluxe )
        end
    end

    -- Play death sound.
    if self.m_play_death_sound == true then
        sound.play( assets.sounds.die )
    end

    -- Transition in.
    self:transitionIn()
end

-- Transition in.
function LuaDeathMenu:transitionIn()
    local game = self.owner:getGame()
    local s    = SequenceAction()
    local s2   = SplitAction()

    s:addAction( ScaleAction( 0.0, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
    s:addAction( RotateOffsetAction( 0.0, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, -180.0 ) ) )
    s:addAction( AlphaAction( 0.0, InterpAction.Smooth, 0.0 ) )
    s2:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
    s2:addAction( RotateOffsetAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 180.0 ) ) )
    s2:addAction( AlphaAction( 0.4, InterpAction.Smooth, 1.0 ) )
    s:addAction( s2 )

    self.owner:addAction( s )
end

-- Transition out.
function LuaDeathMenu:transitionOut( callback )
    local s  = SequenceAction()
    local s2 = SplitAction()

    s2:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
    s2:addAction( RotateOffsetAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, -180.0 ) ) )
    s2:addAction( AlphaAction( 0.4, InterpAction.Smooth, 0.0 ) )
    s:addAction( s2 )
    s:addAction( LuaAction( 0.0, callback ) )
    s:addAction( DestroyAction( 0.0 ) )

    if(not self.owner:isDestroyed()) then   
        if(self.owner ~= nil) then   
            if(not self.transition_started) then
                self.owner:addAction( s )
                self.transition_started = true
            end
        end
    end

end

function LuaDeathMenu:openItemStore()
    if chunks.iap.purchasingAvailable() then
       self:transitionOut( function( o ) self.owner:getOwner().lua:openItemStore() end )
       if(ask_use) then
            item_store.lua:setImmediateUseFlag()
       end
    else
        chunks.iap.displayPurchasingUnavailable()
    end
end

function LuaDeathMenu:destroy()
    if self.m_social_handle then
        self.m_social_handle:cancel()
    end
end
