-- module --
module(...,package.seeall)

local engine = engine.instance
local chunks = engine.chunks
local swrve = chunks.swrve

local BANNER_AD_TYPES = { 
    gameplay = "gameplay",
    main_menu = "main_menu"
}

local ui_handle
local ad_handle

-- I know, these are global...  :(
AT_MAIN_MENU = false
IN_GAME = false


function onGameReady()
    --Do Nothing... whee!  
    --The Bridge Controller did absolutely nothing, suspect it was preemptive.
end

function onShouldExit()
    --Do Nothing... whee!  
    --The Bridge Controller did absolutely nothing, suspect it was preemptive.
    print("JUST WALK AWAY")
end

function showNotificationsConfirmationPrompt()
    local user_data = UserData.get()
    local num_completed_rounds = user_data.num_completed_rounds
    print("showNotificationsConfirmationPrompt runs=", num_completed_rounds)
    
    if(num_completed_rounds >= 2)then
        print("Showing notif prompt")
        local service = GameService.get()
        if service then
            service:registerForNotifications()
        end
    end
    
end

function showRatingsDialog(height_reached)
    local user_data = UserData.get()
    local num_completed_rounds = user_data.num_completed_rounds
    local complete_limit = 20
    
    local high_score = user_data:getHighScore(engine.theme)

    --only show the ratings dialog if user scored at least 75% of their top score for this theme
    if num_completed_rounds == 0 or 
       num_completed_rounds < complete_limit or
       (height_reached < (0.75 * high_score)) then
        user_data.num_completed_rounds = num_completed_rounds + 1
        return
    end

    -- Reset completed round count
    user_data.num_completed_rounds = 0

    local ui = PlatformUIService.get();
    if ui then
        local title = lstring("rating_title")
        local message = lstring("rating_body")
        local no_text = lstring("rating_no_text")
        local yes_text = lstring("rating_yes_text")
        ui:showRatingsDialog(title, message, yes_text, no_text, function() end)
    end
end

function on_leaderboard_ui(_leaderboard_name)
    local gc = GameCenterService.get()
    if gc then
        if (gc:isLoggedIn()) then
            gc:displayLeaderboardsUI(_leaderboard_name or "", function() end)
        else
            --Whoops, user is not logged in.
            if(ui_handle == nil) then
                local callback = function(handle, status, response)
                    ui_handle = nil

                    if status == 0 then
                        events.on_leaderboard_theme()
                    end
                        chunks.utility.showDialog(lstring("gc_relogin_title"),
                                                      lstring("gc_relogin_text"),
                                                      lstring("ok"))
                end
                ui_handle = gc:login(callback)
            end
        end
    end
end

function on_leaderboard_theme()
	on_leaderboard_ui(engine.theme)
end

function postScoreToLeaderboard(_score)
    print("Lua call postScoreToLeaderboard");
    local gc = GameCenterService.get()
    if gc then
        local leaderboard_name = engine.theme
        gc:postScore(_score, leaderboard_name, function() end)
    end
end

function resumeGameplayAd()
    local ads = AdsService.get()
    if ads then
        ads:refreshBanners(BANNER_AD_TYPES.gameplay)
    end
end

-- The user goes to the main menu.
function on_main_menu_appeared()
    showBannerAds(BANNER_AD_TYPES.main_menu)
    AT_MAIN_MENU = true
end

-- The main menu begins fading out
function on_main_menu_disappeared()
    hideAllAds()
end

function on_shield_store_appeared()
    hideAllAds()
end

function on_shield_store_disappeared()
    if AT_MAIN_MENU == true then
        showBannerAds(BANNER_AD_TYPES.main_menu)
    elseif IN_GAME == true then
        showBannerAds(BANNER_AD_TYPES.gameplay)
    end
end

-- The user goes to the tutorial menu.
function on_tutorial_menu()
    sendAnalyticEvent("OPENED_TUTORIAL")
end

-- The user goes to the credits menu.
function on_credits_menu()
    sendAnalyticEvent("OPENED_CREDITS")
end

-- The user started a game.
function on_start_game()
    -- show gameplay ads
    showBannerAds(BANNER_AD_TYPES.gameplay)

    sendAnalyticEvent("STARTED_GAME_" .. engine.theme)
    IN_GAME = true
end

-- The user quit a game.
function on_quit_game()
    sendAnalyticEvent("QUIT_GAME_" .. engine.theme)
    IN_GAME = false
end

-- Brings up facebook.
function on_facebook(score)
    local fb = FacebookService.get()
    if fb then
        local details = chunks.social.getSocialDetails()

        if not details then
            print("Events.on_facebook() - Could not get details! Ignoring Facebook Post Request")
            return
        end

        local caption = lstring("post_to_facebook")
        caption:arg(1, string.format("%d", score))
        caption:arg(2, lstring(engine.theme))
        caption:arg(3, details.product_name)
        caption:arg(4, details.platform)

        local description = lstring.format("")

        local fb_parms = FacebookStoryParameters()
        fb_parms.name = details.product_name
        fb_parms.caption = caption
        fb_parms.description = description
        fb_parms.link = details.url
        fb_parms.picture_url = details.picture


        fb:publishFeedStory(fb_parms, true, true, function() end)
    end
end

-- Brings up twitter.
function on_twitter(score)
    local twitter = TwitterService.get()
    if twitter then
        if twitter:canTweet() then
            local details = chunks.social.getSocialDetails()

            if not details then
                print("Events.on_twitter() - Could not get details! Ignoring Tweet Request")
                return
            end
            
            local tweet = lstring("post_to_twitter")
            tweet:arg(1, string.format("%d", score))
            tweet:arg(2, lstring(engine.theme))
            tweet:arg(3, details.product_name)
            tweet:arg(4, details.url)
            
            twitter:tweet(tweet, function() end)
        else
            local ui = PlatformUIService.get()
            if ui then
                local title = lstring("tw_login_title")
                local body = lstring("tw_login_body")
                local okay = lstring("ok")

                ui:showDialog(title, body, okay, function() end)
            end
        end
    end
end

-- Brings up email.
function on_email(score)
    local ui = PlatformUIService.get()
    if ui then        
        local details = chunks.social.getSocialDetails()

        if not details then
            print("Events.on_email() - Could not get details! Ignoring Email Request")
            return
        end

        local subject = lstring("email_subject")
        subject:arg(1, details.product_name)
        subject:arg(2, string.format("%d", score))

        local body = lstring("email_body")
        body:arg(1, string.format("%d", score))
        body:arg(2, lstring(engine.theme))
        body:arg(3, details.product_name)
        body:arg(4, details.platform)
        body:arg(5, details.url)

        -- Set image path
        local img_path 
        if NJ_CLASSIC then
            img_path = chunks.assets.textures.email.path
        elseif NJ_DELUXE or NJ_DELUXE_FREE then
            img_path = chunks.assets.textures.email_dx.path
        end

        if img_path then
            local gs = GameService.get()
            local params = gs:constructEmailParams(img_path, subject, body, true)

            local callback = function(handle, status, response)
                print("***** Callback Fired, status= ", status)
                ui_handle = nil
                if (status == MailComposeResult.MailComposeResult_NoImplementation) then
                    -- Show, need to configure mail dialog
                    local title = lstring("no_email_title")
                    local message = lstring("no_email_body")
                    local ok = lstring("ok")
                    print("***** MailComposeResult_NoImplementation")
                    ui:showDialog(title, message, ok, function() end)
                elseif (status == 5) then --This SHOULD NOT happen, but there seems to be a platform bug that is returning the value
                    --of 5, which is not mapped to any MailComposeResult enum
                    --TODO either get a fix from platform or check against a new status == 5 enum
       
                    local title = lstring("no_email_title")
                    local message = lstring("no_email_body")
                    local ok = lstring("ok")
                    print("***** null")
                    ui:showDialog(title, message, ok, function() end)
                end
            end
            
            print("***** showComposeMailDialog")
            ui_handle = ui:showComposeMailDialog(params, callback)
        end
    end
end

-- The game over event.
function on_game_over(_score)
    sendAnalyticEvent("FINISHED_GAME_" .. engine.theme)
    resumeGameplayAd()
end	

function showNinJumpDeluxe()
    local ui = PlatformUIService.get()
    if ui then
		local platform = chunks.social.getPlatform()
        local url = chunks.social.getDeluxeURL()
		
		if platform == "android" then
			ui:launchExternalUrl(url)
		else	
		    ui:showWebView(url, "", WebViewOrientation["WebViewOrientation_Any"], function() end)
		end
    end
end

function showGameplayInterstitial(_callback)
    if NJ_DELUXE and not doesNJDXFirstLaunchedDuringPromotionExist() then
        _callback()
        return
    end

	local interstitial_setting = ""
	
	if GameService.get():zeb() then
		interstitial_setting = "on"
	else
		interstitial_setting = swrve.getResourceAttributeString("ad_info", "interstitials")
	end
    
    local ads = AdsService.get()
    
    if (interstitial_setting == "on") then
        print("Showing standard interstitital")
        if ads then
            ad_handle = ads:showInterstitial("gameplay_interstitial", 2.0, _callback )
        else
            print("NO ADS SERVICE!")
            _callback()
        end
    elseif (interstitial_setting == "alt") then
        print("Showing alt-id interstitital")
        if ads then
            ad_handle = ads:showInterstitial("gameplay_interstitial_alternate", 2.0, _callback )
        else
            print("NO ADS SERVICE!")
            _callback()
        end
    elseif (interstitial_setting == "off") then
        print("Ads disabled by swrve, not showing interstitital")
        _callback()
    else
        print("Unrecognized value for interstitial_setting, skipping ad")
        _callback()
    end
end

function show_privacy_policy()
    local ui = PlatformUIService.get()
    if ui then
        local url = "http://www.backflipstudios.com/help/privacy-policy/"
        ui:showWebView(url, "", WebViewOrientation["WebViewOrientation_Any"], function() end)
    end
end

function showBannerAds(ad_type)
    
	if GameService.get():zeb() then
		local ads = AdsService.get()
	    if ads then
	        ads:hideAllBanners()
	        ads:showBanners(ad_type)
	    end
	else
		local banner_setting = swrve.getResourceAttributeString("ad_info", "banners")
	    if(banner_setting == "on") then
	        local ads = AdsService.get()
	        if ads then
	            ads:hideAllBanners()
	            ads:showBanners(ad_type)
	        end
	    elseif(banner_setting == "off") then
	        print("Banner ads disabled by swrve")
	    end
	end
end

function hideAllAds()
    local ads = AdsService.get()
    if ads then
        ads:hideAllBanners()
    end
end

function sendAnalyticEvent(event_name, parameters)
    --NOTE: LOOKS LIKE CURRENT EVENTS DON'T ACTUALLY ADD ANY PARAMETERS HERE.  NEED TO CHECK.
    --UPDATE: I let Nemco know and he's going to bring it up to marketing.
    local event = AnalyticsEvent(event_name)
    local parameters = parameters or {}
    for param, value in pairs(parameters) do
        event:add(param, value)
    end
    event:send()
end
