------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Fade               --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local textures  = chunks.textures
local utility   = chunks.utility;

-- Fade in, then call a callback.
function fade_in( owner, callback )
	local game  = owner:getGame()
	local actor = owner:spawn( SpriteActor() )
	
	callback       = callback or function( inner ) end
	actor.texture  = "builtin://white"
	actor.location = pool.get_v3f( 0.0, 0.0, 100.0 )
	actor.size     = constants.scene.world_size
	actor.color    = pool.get_colorf( 1.0, 1.0, 1.0, 1.0 )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		s:addAction( AlphaAction( 0.5, InterpAction.Smooth, 0.0 ) )
		s:addAction( LuaAction( 0.0, callback ) )
		s:addAction( DestroyAction( 0.0 ) )
		
		actor:addAction( s )
	end
	
	return actor
end

-- Fade out, then call a callback.
function fade_out( owner, callback )
	local game  = owner:getGame()
	local actor = owner:spawn( SpriteActor() )
	
	callback       = callback or function( inner ) end
	actor.texture  = "builtin://white"
	actor.location = pool.get_v3f( 0.0, 0.0, 100.0 )
	actor.size     = constants.scene.world_size
	actor.color    = pool.get_colorf( 1.0, 1.0, 1.0, 0.0 )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		s:addAction( AlphaAction( 0.5, InterpAction.Smooth, 1.0 ) )
		s:addAction( LuaAction( 0.0, callback ) )
		
		actor:addAction( s )
	end
	
	return actor
end	

-- Fade in loading.
function fade_in_loading( owner, callback )
	local actor   = fade_in( owner, callback )
	local loading = actor:spawn( SpriteActor() )
	
	loading.location = pool.get_v3f( 0.0, 0.0, 0.5 )
	
	textures.setTexture( loading, assets.textures.loading )

    local label_holder = loading:spawn( SpriteActor() )
    local offset = 0    
    local height = 43.0
    
    if engine.locale_language == "ja" or engine.locale_language == "ko" or string.sub(engine.locale_language,1,string.len("zh")) == "zh" then
        height = 48.0
        offset = -4
    end
    
    if engine.locale_language == "it" or engine.locale_language == "pt" or engine.locale_language == "fr" then
        offset = -3
    end

    label_holder.location = pool.get_v3f( 0.0, offset + 61.0, 1.0 )

	local label = utility.createText( label_holder, lstring( "loading" ), {"sans-text-large", 43.0, pool.get_colorf(1, 1, 1, 1), 170.0, TextAlign.Center, 0, 0} )
end

-- Fade out loading.
function fade_out_loading( owner, callback )
	local actor   = fade_out( owner, callback )
	local loading = actor:spawn( SpriteActor() )
	
	loading.location = pool.get_v3f( 0.0, 0.0, 0.5 )
	
	textures.setTexture( loading, assets.textures.loading )

    local label_holder = loading:spawn( SpriteActor() )
    local offset = 0    
    local height = 43.0
    
    if engine.locale_language == "ja" or engine.locale_language == "ko" or string.sub(engine.locale_language,1,string.len("zh")) == "zh" then
        height = 48.0
        offset = -4
    end
    
    if engine.locale_language == "it" or engine.locale_language == "pt" or engine.locale_language == "fr" then
        offset = -3
    end

    label_holder.location = pool.get_v3f( 0.0, offset + 61.0, 1.0 )

	local label = utility.createText( label_holder, lstring( "loading" ), {"sans-text-large", 43.0, pool.get_colorf(1, 1, 1, 1), 170.0, TextAlign.Center, 0, 0} )
end

