------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Flytrap            --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaFlytrap --
LuaFlytrap = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaFlytrap )

-- The created method.
function LuaFlytrap:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "flytrap"
	owner.location = pool.get_v3f( -100.0, 0.0, 0.0 )
	
	-- Balcony.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.flytrap )

		self.visuals.flytrap = actor
	end
	
	self:setVisual( "flytrap" )
	
	-- We're not killable.
	self.killable = false
end

-- The collided event.
function LuaFlytrap:onCollided( other )
	-- Spawn particles.
	do
		particles.spawn_big( self.owner:getGame().lua.root, 
		                     assets.textures.particles_leaf, 
		                     self.visual:getWorldLocation() )
	end
	
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_enemy )
end

