------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Foliage            --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures

-- LuaFoliage --
LuaFoliage = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	scale   = v3f( 0.5, 0.5, 1.0 ),
	offsets =
	{
		-128.0,
		-118.5,
		-140.0
	}
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaFoliage )

-- The created method.
function LuaFoliage:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )

	local index = math.random( 1, #self.offsets )
	
	-- Initialize owner.
	owner.label    = "foliage"
	owner.location = pool.get_v3f( self.offsets[ index ], 0.0, 0.0 )
	
	-- Randomly choose foliage.
	do
		local actor = owner:spawn( ActorBase() )
		
		actor.texture = assets.textures[ "foliage" .. tostring( index ) ].path
		actor.mesh    = assets.meshes[ "foliage" .. tostring( index ) ].path
		actor.scale   = self.scale
		
		self.visuals.foliage = actor
	end
	
	self:setVisual( "foliage" )
end



