------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- LuaMenuGame        --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local pool      = chunks.pool
local assets    = chunks.assets
local music     = chunks.music

-- LuaMenuGame --
LuaMenuGame = chunks.class.new()

-- The spawner.
spawn = chunks.class.spawner( BasicGame, LuaMenuGame )

-- The created method.
function LuaMenuGame:created( game )
	-- Spawn root.
	self.root = game:getScene():spawn( ActorBase() )
	
	-- Scene camera.
	do
		local camera = game:getScene():spawn( OrthographicCamera() )
		
		camera.world_size      = constants.scene.world_size
		camera.clipping_planes = constants.scene.clipping_planes
		camera.clear_color     = constants.clear_color
        camera.clear_color_on  = constants.clear_color_on
        camera.clear_depth_on  = constants.clear_depth_on
        camera.scene           = game:getScene()
	end
	
	-- Window camera.
	do
		local camera = game:getWindow():spawn( OrthographicCamera() )
		
		camera.world_size      = constants.window.world_size
		camera.clipping_planes = constants.window.clipping_planes
		camera.clear_color_on  = false
        camera.clear_depth_on  = false
        camera.scene           = game:getWindow()

        game:getWindow().camera = camera
	end
	
	-- Spawn main menu.
	do
		game:getWindow():spawn( chunks.main.spawn() )
	end
	
	-- Start music.
    engine:playMenuMusic()
end

-- The replay event.
function LuaMenuGame:onPlay( theme )
	local game = self.owner
	
	engine:spawnMainGame( theme )
	game:destroy()
	Audio.get():restoreState()
end
