------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Hud                --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine   = engine.instance
local chunks   = engine.chunks
local constants = engine.platform.constants
local assets   = chunks.assets
local pool     = chunks.pool
local textures = chunks.textures
local utility = chunks.utility
local swrve = chunks.swrve

-- LuaHud --
LuaHud = chunks.class.inherit( chunks.window.LuaWindow )
{
    m_score_bar = nil,
	ticker = nil,
	meter  = nil,
	shield_meter = nil,
	m_spawning_ground = nil,
	m_shield_button = nil,
	m_shield_meter = nil,
	m_flying_shield = nil,
	m_start_boost_offered = nil,
	m_end_boost_offered = nil,
	m_pause_button = nil,
	m_game = nil,
	m_async_handle = nil,
}
	
-- The spawner.
spawn = chunks.class.spawner( Window, LuaHud )

-- The created method.
function LuaHud:created( owner )
	local game = owner:getGame()
	local ninja = game.lua:getNinja()
	self.m_game = game
	
	-- Initialize owner.
	owner.location = pool.get_v3f( 0.0, 0.0, 100.0 )
	owner.size     = constants.scene.world_size
	owner.hidden   = true
	
	--Blank Window from which to spawn the store, for input issues
	local spawning_ground = owner:spawn( Window() )
	spawning_ground.size = pool.get_v2f(0, 0)
	self.m_spawning_ground = spawning_ground

    -- Score bar.
	do
        local ads_bought_out = chunks.iap:adBuyoutPurchased()
		local score_bar      = owner:spawn( Window() )	
        local yoffset        = -assets.textures.score_bar.size.y / 2

        if not ads_bought_out then
            yoffset = yoffset - 50
        end

        score_bar.anchor = make_anchor( Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f( 0, yoffset, 9 ) )	

		textures.setTexture( score_bar, assets.textures.score_bar )

		self.m_score_bar = score_bar
    end

	-- The ticker.
	do
		local ticker = self.m_score_bar:spawn( chunks.ticker.spawn() )	
		ticker.force_color = true
		ticker.location = pool.get_v3f( 0.0, 0.0, 0.001 )
		ticker.scale    = pool.get_v3f( 0.75, 0.75, 1.0 )		
		self.ticker = ticker
	end
    
	-- The pause button.
	do
		-- Bring up the pause menu.
		local on_release = function( inner )
			game.lua:onPause()
		end	
	
		local button = self:addButton( on_release )

        button.scale = pool.get_v3f( 0.01, 0.01, 1.0 )
		button.anchor = make_anchor( Anchor.Anchor_BR, Anchor.Anchor_BR, pool.get_v3f( 0.0, 0.0, 0.0 ) )	

		textures.setTexture( button, assets.textures.pause_button )
        
		self.m_pause_button = button
	end
	
	local shield_button = nil

	-- The shield button - uses a shield received via IAP.
	do
			
		-- Crack a shield on your ninja head.
		local on_release = function( inner ) end			
		local button     = self:addButton( on_release )		
		
		button.scale = pool.get_v3f( 0.01, 0.01, 1.0 )
        button.anchor = make_anchor( Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f( 10.0, -8.0, 1.0 ) )
        
        textures.setTexture( button, assets.textures.shield_button )
        
		self.m_shield_button = button
		
        button.lua         = {}
		button.lua.pulsate = function(inner)
							local pulse = FiniteLoopAction(3)
							local pulse_up = SplitAction() 
							pulse_up:addAction( ScaleAction( 0.3, InterpAction.Smooth, pool.get_v3f( 1.3, 1.3, 1.0 ) ) )
							pulse_up:addAction( MoveOffsetAction( 0.3, InterpAction.Smooth, pool.get_v3f( 0.0, 4.0, 0.0 ) ) )
							local pulse_down = SplitAction() 
							pulse_down:addAction( ScaleAction( 0.3, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
							pulse_down:addAction( MoveOffsetAction( 0.3, InterpAction.Smooth, pool.get_v3f( 0.0, -4.0, 0.0 ) ) )
							pulse:addAction(pulse_up)
							pulse:addAction(pulse_down)

							-- visual_button.location = pool.get_v3f(0.0, 0.0, 0.0)
							-- visual_button.scale = pool.get_v3f(1.0, 1.0, 1.0)
							-- visual_button:clearActions()
							-- visual_button:addAction( pulse )	

							button.location = pool.get_v3f(0.0, 0.0, 0.0)
							button.scale = pool.get_v3f(1.0, 1.0, 1.0)
							button:clearActions()
							button:addAction( pulse )	
						end
						
		--Shield count in ticker form
		local shield_ticker = button:spawn( chunks.ticker.spawn() )		
		shield_ticker.location = pool.get_v3f( 11.0, 0.0, 1.0 )
		shield_ticker.scale    = pool.get_v3f( 0.7, 0.7, 1.0 )
		shield_ticker.value    = UserData.get():getShields()
		shield_ticker.lua.delay = 0
		
 		shield_ticker.lua.update = function(inner)
			local shields = UserData.get():getShields()
			if(shields > 9999) then shields = 9999 end
			shield_ticker.value = shields
		end
		
		self.m_shield_meter = shield_ticker
		-- Crack a shield on your ninja head.
		button.on_release = function( inner )
			button:clearActions()
			button.scale = pool.get_v3f(1.0, 1.0, 1.0)
			button.location = pool.get_v3f(0.0, 0.0, 0.0)

			if (not ninja.lua:isDead()) then
				local shields = UserData.get():getShields()
				if(shields > 0) then 
					self:activateShield()
					
				elseif not GameService.get():zeb() then
                    game.lua:forcePause()
                    if chunks.iap.purchasingAvailable() then
                        self:openItemStore(true)
                    else
                        local unpause_game = function()
                            game.lua:onUnpause()
                            game.lua:onPause()
                        end
                        chunks.iap.displayPurchasingUnavailable(unpause_game)
                    end
				end
			end
		end
		shield_button = button
	end
	
	self.m_start_boost_offered = false
	self.m_end_boost_offered = false
	
	local device_name = DeviceInfo.get():query("hw.description"):sub(1, 5)
	local ipod4 = (device_name == "iPod4")
	
	if (not GameService.get():zeb() and not ipod4) then
	
	    local createIncentAdsButton = function()
	    
	        local bullet_time = swrve.getResourceAttributeFloat("start_boost_info", "duration")
	        local chance_to_show_ad = swrve.getResourceAttributeFloat("start_boost_info", "frequency")
	        local cancel_button
	        local slowdown_factor = 0.333
	        local max_height_to_activate_boost = 100
	        local roll = math.random()
	        
	        if(roll < chance_to_show_ad) then
	            print("Hit the roll")
                local height = game.lua.height
            
                if(height < max_height_to_activate_boost) then
                
                    self.m_start_boost_offered = true
                    
                    local big_layout = swrve.getResourceAttributeBool("start_boost_info", "big_layout") 
                    local big_panel
                    local cancel_button_style = swrve.getResourceAttributeString("start_boost_info", "cancel_button_style")
                    local cancel_button_pos = swrve.getResourceAttributeString("start_boost_info", "cancel_button_pos")
                    
                    if (big_layout) then
                        --big layout overrides and forces cancel_button_style to be 'close_x'
                        cancel_button_style = "close_x"
                    end
                    
                    if (big_layout) then
                        big_panel = owner:spawn( Window() )	
                        big_panel.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0, 0, 1.0 ) )	
		                textures.setTexture( big_panel, assets.textures.full_ia_panel )

		                big_panel.scale = pool.get_v3f(0.0001, 0.0001, 1.0)
		                
		                local confirmation_text_style = swrve.getResourceAttributeString("start_boost_info", "confirm_prompt_copy")
                        local body
                        if(confirmation_text_style == "simple") then
                            body = lstring("preroll_video_confirmation_simple")
                        elseif(confirmation_text_style == "exact") then
                            body = lstring("preroll_video_confirmation_exact")
                            local min_boost_distance = swrve.getResourceAttributeFloat("start_boost_info", "height_min")
                            body:arg(1, min_boost_distance)
                        elseif(confirmation_text_style == "inspirational") then
                            body = lstring("preroll_video_confirmation_inspirational")
                        else
                            --default to "simple"
                            body = lstring("preroll_video_confirmation_simple")
                            error("Confirmation (preroll) text not found")
                        end
                        
                        if string.sub(engine.locale_language,1,string.len("zh")) == "zh" then
                            utility.createText( big_panel, body, {"ninja-text", 20.0, pool.get_colorf(1, 1, 1, 1), 70, TextAlign.Center, 0, 30, 1, 1}, nil, 180 )
		                else
		                    utility.createText( big_panel, body, {"ninja-text", 40.0, pool.get_colorf(1, 1, 1, 1), 70, TextAlign.Center, 0, 30, 1, 1}, nil, 180 )
		                end
                    end
                    
	                local button     = self:addButton( on_release )	
	                
	                if(big_layout)then	
                        button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -50.0, 2.0 ) )
                    else
                        button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -20.0, 2.0 ) )
                    end
                    
                    textures.setTexture( button, assets.textures.boost_button )
                    button.scale = pool.get_v3f(0.0001, 0.0001, 1.0)
                
                    local icon_style = swrve.getResourceAttributeString("start_boost_info", "button") .. "_icon"
                    local icon = utility.createWindowWithTexture(button, assets.textures[icon_style])
                    icon.location = pool.get_v3f(60, 0, 1.0)
                
                    local shield_will_be_awarded = swrve.getResourceAttributeBool("start_boost_info", "give_shield")
                    if(shield_will_be_awarded)then
                        local shield = utility.createWindowWithTexture(button, assets.textures.boost_shield_icon)
                        shield.location = pool.get_v3f(-60, 0, 1.0)
                    end
                
                    local rocket = utility.createWindowWithTexture(button, assets.textures.boost_rocket_icon)
                    rocket.location = pool.get_v3f(-60, 0, 2.0)
                    
                    if(big_layout) then
                        big_panel:addAction( SleepAction( 1.0 ) )
                        big_panel:addAction( ScaleAction( 0.05, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
	    	            big_panel:addAction( ScaleAction( 0.07, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
	    	            big_panel:addAction( SleepAction( math.floor(bullet_time) * slowdown_factor ) )
	    	            big_panel:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.0001, 0.0001, 1.0 ) ) )
		                big_panel:addAction( DestroyAction(0.0) )
                    end
                
                    button:addAction( SleepAction( 1.0 ) )
                    button:addAction( LuaAction(0.0, function() game.delta_time_factor = slowdown_factor 
                                                        game.lua:silenceFootsteps() end ) ) --BULLET TIME
                    button:addAction( ScaleAction( 0.05, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
	    	        button:addAction( ScaleAction( 0.07, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
		       
		        
		            local s = SequenceAction()
                    s:addAction( SleepAction( 0.56 * slowdown_factor ) )
                    s:addAction( ScaleAction( 0.15 * slowdown_factor, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
	    	        s:addAction( ScaleAction( 0.07 * slowdown_factor, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
	    	        s:addAction( ScaleAction( 0.15 * slowdown_factor, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
	    	        s:addAction( ScaleAction( 0.07 * slowdown_factor, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
	    	        local l = FiniteLoopAction(math.floor(bullet_time))
	    	        l:addAction(s)
                    button:addAction(l)
                
		            button:addAction( LuaAction(0.0, function() game.delta_time_factor = 1.0 
		                                                game.lua:unsilenceFootsteps() end ) ) --END BULLET TIME
    		        button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
		            button:addAction( DestroyAction(0.0) )
		            button:addAction( LuaAction(0.0, function() self:showHudButtons() end ) )

				    local cancel_enabled = swrve.getResourceAttributeBool("start_boost_info", "allow_cancel")
				    
                    local button_copy = swrve.getResourceAttributeString("start_boost_info", "copy")
                    print("start boost copy ", button_copy)
                    utility.createText( button, lstring(button_copy), {"ninja-text", 30.0, pool.get_colorf(1, 1, 1, 1), 70, TextAlign.Center, 0, 3, 1, 1} )
       
                    button.on_release = function( inner )
                        if(game.paused) then
                            return
                        end
                    
                        local show_ad_callback = function(handle, code, btnidx, already_approved)
                        
                            if(engine.platform.device == "android") then
                                game.lua:ceaseIgnoringPauseEvents()
                            end
                                
                            game.delta_time_factor = 1.0
                            button.enabled = false
                            
                            local yes_button_selected = false
                            if(not already_approved) then
                                local yes_button_on_right = swrve.getResourceAttributeBool("start_boost_info", "confirm_prompt_watch_right")
                                if(engine.platform.device == "android") then
                                    yes_button_on_right = not yes_button_on_right
                                end
                        
                                if(yes_button_on_right)then
                                    if(btnidx:getInt() == 1)then
                                        yes_button_selected = true
                                    end
                                else
                                    if(btnidx:getInt() == 0)then
                                        yes_button_selected = true
                                    end
                                end
                            end
                            
                            if already_approved or yes_button_selected or big_layout then
                            
                                UserData.get():setConfirmedPrerollVideo(true)
                                chunks.sound.play( assets.sounds.click )
                                game.lua:silenceFootsteps()
                
                                if (not ninja.lua:isDead() ) then
                
                                    game.delta_time_factor = 0.0
                                    button:clearActions()
                        
                                    if(engine.platform.device == "android") then
                                        game.lua:ignoreNextPauseEvent()
                                    end
                        
                                    local ad_shown_callback = function()
                                        game.delta_time_factor = 1.0
                    
                                        local activate_shield = swrve.getResourceAttributeBool("start_boost_info", "give_shield")
                                        if(activate_shield) then
			                                self:activateIncentivizedShield()
			                            end
			                            self:activateIncentivizedBoost(function() self:showHudButtons() end)
			                            
			                
		        	                    button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
		                    
            		                    button:addAction( LuaAction(1.4, function()        
            		                        --just a little delay to give the fuse video time to clear out of the view      
    		                                self:showHudButtons()
	        	                            game.lua:getNinja().lua:startBoostUsed()
    		                                chunks.analytics.reportVideoWatched(true)	   
    		                                chunks.incent_ads.preloadAd("preroll")                
		                                end  ) )
		                    
		                                button:addAction( DestroyAction(0.0) )
		                                
		                                if(cancel_enabled) then
		                                    cancel_button:clearActions()
                                            cancel_button:addAction( DestroyAction(0.0) )
                                        end
                                        
                                        if(big_layout) then
                                            big_panel:clearActions()
                                            big_panel:addAction( DestroyAction(0.0) )
                                        end
                                        
                                        if(engine.platform.device == "android") then
			                                game.lua:ignoreNextPauseEvent()
			                            end
		                            end
		        
		                            local no_ad_callback = function()
                                        game.delta_time_factor = 1.0
			        
        			                    if(engine.platform.device == "android") then
    			                            game.lua:ceaseIgnoringPauseEvents()
			                            end
			                
			                            button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
    		                            button:addAction( DestroyAction(0.0) )
	    	                            button:addAction( LuaAction(0.0, function() self:showHudButtons() end  ) )
		                                
		                                if(cancel_enabled) then
		                                    cancel_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
    		                                cancel_button:addAction( DestroyAction(0.0) )
	    	                                cancel_button:addAction( LuaAction(0.0, function() self:showHudButtons() end  ) )
	    	                            end
	    	                            
	    	                            if(big_layout) then
                                            big_panel:clearActions()
                                            big_panel:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                                            big_panel:addAction( DestroyAction(0.0) )
                                        end
	    	                            
    		                            game.lua:unsilenceFootsteps()
	    	                        end
		            
		                            local result = chunks.incent_ads.showIncentVideo("preroll", ad_shown_callback, no_ad_callback)
	        	                    chunks.analytics.reportVideoStarted(true)	
        		                end
		                    else--user said "no thanks" to the fuse ad prompt
		                        
                                    
		                        button:clearActions()
		                        button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
		                        button:addAction( DestroyAction(0.0) )
		                        
		                        if(cancel_enabled) then
		                            cancel_button:clearActions()
		                            cancel_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.0001, 0.0001, 1.0 ) ) )
		                            cancel_button:addAction( DestroyAction(0.0) )
		                        end
		                        
		                        if(big_layout) then
                                    big_panel:clearActions()
                                    big_panel:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.0001, 0.0001, 1.0 ) ) )
                                    big_panel:addAction( DestroyAction(0.0) )
                                end
                                        
		                        self:showHudButtons()
		                    end
		                end--end show ad callback
		                
		                --disable the cancel button to prevent monkey bug shenanigans
		                if(cancel_enabled) then
		                    cancel_button.enabled = false
		                end
		                
		                local show_confirmation = swrve.getResourceAttributeBool("start_boost_info", "show_confirm_prompt")
		                print("SHOW CONFIRMATION SETTING", show_confirmation)
		                if((not show_confirmation) or UserData.get():getConfirmedPrerollVideo() or big_layout)then
		                    if(engine.platform.device == "android") then
                                game.lua:ignoreNextPauseEvent()
                            end
		                    show_ad_callback(nil, nil, nil, true)
		                else
		                    local ui = PlatformUIService.get()
                            if ui ~= nil then
                                local confirmation_text_style = swrve.getResourceAttributeString("start_boost_info", "confirm_prompt_copy")
    				            local body
    				            if(confirmation_text_style == "simple") then
    				                body = lstring("preroll_video_confirmation_simple")
    				            elseif(confirmation_text_style == "exact") then
    				                body = lstring("preroll_video_confirmation_exact")
    				                local min_boost_distance = swrve.getResourceAttributeFloat("start_boost_info", "height_min")
    				                body:arg(1, min_boost_distance)
    				            elseif(confirmation_text_style == "inspirational") then
    				                body = lstring("preroll_video_confirmation_inspirational")
    				            else
                                    --default to "save"
                                    body = lstring("preroll_video_confirmation_simple")
                                    error("Confirmation (preroll) text not found")
    				            end
    				            
                                local details = chunks.social.getSocialDetails()
                                local title = lstring("watch_video")
                                local okay = lstring("watch")
                                local not_okay = lstring("cancel")
                                
                                local yes_button_on_right = swrve.getResourceAttributeBool("start_boost_info", "confirm_prompt_watch_right")
                                if(engine.platform.device == "android") then
                                    yes_button_on_right = not yes_button_on_right
                                end
                                
                                local buttons
                                if(yes_button_on_right)then
                                    buttons = {not_okay, okay}
                                else
                                    buttons = {okay, not_okay}
                                end
                                
                                game.delta_time_factor = 0.0
                                if(engine.platform.device == "android") then
                                    game.lua:ignoreNextPauseEvent()
                                end
                                self.m_async_handle = ui:showPrompt(title, body, buttons, show_ad_callback)
                            end
                        end
		            end
		            
		            if(cancel_enabled) then
                        --now create the cancel button
                        cancel_button     = self:addButton( on_release )	
                        
                        local setup_close_x_button = function()
                            if(big_layout) then
                                if (cancel_button_pos == "top_right") then
                                    cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 94.0, 100.0, 2.0 ) ) 
                                    textures.setTexture( cancel_button, assets.textures.small_exit_button )
                                elseif (cancel_button_pos == "top_left") then
                                    cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( -94.0, 100.0, 2.0 ) ) 
                                    textures.setTexture( cancel_button, assets.textures.small_exit_button )
                                else --in case of bad data from swrve, fall back to top right
                                    print("Cancel button position not recognized")
                                    cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 94.0, 100.0, 2.0 ) ) 
                                    textures.setTexture( cancel_button, assets.textures.small_exit_button )
                                end
                            else
                                if (cancel_button_pos == "top_right") then
                                    cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 80.0, 5.0, 5.0 ) ) 
                                    textures.setTexture( cancel_button, assets.textures.small_exit_button )
                                elseif (cancel_button_pos == "top_left") then
                                    cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( -80.0, 5.0, 5.0 ) ) 
                                    textures.setTexture( cancel_button, assets.textures.small_exit_button )
                                else --in case of bad data from swrve, fall back to top right
                                    print("Cancel button position not recognized")
                                    cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 80.0, 5.0, 5.0 ) ) 
                                    textures.setTexture( cancel_button, assets.textures.small_exit_button )
                                end
                            end
                        end
                        
                        if (cancel_button_style == "close_x") then
                            setup_close_x_button()
                        elseif 	(cancel_button_style == "full") then
                            cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -70.0, 2.0 ) ) 
                            textures.setTexture( cancel_button, assets.textures.replay_button_locale )
                        else --in case of bad data from swrve, fall back to x-style button
                            print("Cancel button style not recognized")
                            setup_close_x_button()
                        end
                        
                        if (cancel_button_style == "full") then
                            local cancel_button_copy = swrve.getResourceAttributeString("start_boost_info", "cancel_copy")
                            utility.createText( cancel_button, lstring(cancel_button_copy), {"ninja-text", 30.0, pool.get_colorf(1, 1, 1, 1), 70, TextAlign.Center, 0, 3, 1, 1} )
                        end
                        
                        cancel_button.size = cancel_button.size / 1.5
                        cancel_button.scale = pool.get_v3f(0.01, 0.01, 1.0)
                    
                        cancel_button:addAction( SleepAction( 1.0 ) )
                        cancel_button:addAction( ScaleAction( 0.05, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
                        cancel_button:addAction( ScaleAction( 0.07, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
                    
                        cancel_button:addAction( SleepAction( 1.0 * slowdown_factor * math.floor(bullet_time)) )
                    
                        cancel_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                        cancel_button:addAction( DestroyAction(0.0) )
                    
                        cancel_button.on_release = function( inner )
                            if(game.paused) then
                                return
                            end

                            game.delta_time_factor = 1.0
                            button.enabled = false
                            
                            button:clearActions()
                            button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                            button:addAction( DestroyAction(0.0) )
                            self:showHudButtons()
                        
                            cancel_button:clearActions()
                            cancel_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                            cancel_button:addAction( DestroyAction(0.0) )
                            
                            if(big_layout) then
                                big_panel:clearActions()
                                big_panel:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.0001, 0.0001, 1.0 ) ) )
                                big_panel:addAction( DestroyAction(0.0) )
                            end
                        end
                    end
                    
		        end
		    else
		        print("Missed the roll")
		        self:showHudButtons()
		    end
        end
    
        local preroll_trigger = swrve.getResourceAttributeString("feature_info", "preroll_trigger")
        print("Current best height ", game.lua.best_height)
        if (preroll_trigger == "incent_ad") then
            local preroll_feature = swrve.getResourceAttributeString("feature_info", "preroll_feature")
            print("Preroll feature is ", preroll_feature)
            if ((preroll_feature == "start_boost") and (game.lua.best_height > 0)) then
                --do not offer a boost the first time the user plays a level
                local ad_available = chunks.incent_ads.videoAvailable("preroll")
                if(ad_available) then
                    print("ad is available")
                    createIncentAdsButton()
                else
                    print("ad is NOT available")
                    self:showHudButtons()
                end
            else
	            self:showHudButtons()
	        end
	    else
	        print("preroll trigger does not exist")
	        self:showHudButtons()
	    end
	else
	    self:showHudButtons()
	end
	
	-- Bonus indicator.
	do
		local meter = shield_button:spawn( chunks.bonusmeter.spawn() )
		meter.location = pool.get_v3f( 8.0, 33.0, 0.0 )
		self.meter = meter	
	end 
end

function LuaHud:hideHudButtons()
    self.m_shield_button:addAction( ScaleAction( 0.3, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
    self.m_pause_button:addAction( ScaleAction( 0.3, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
end

function LuaHud:showHudButtons()
    self.m_shield_button:addAction( ScaleAction( 0.3, InterpAction.Smooth, pool.get_v3f( 1.3, 1.3, 1.0 ) ) )
    self.m_shield_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
    
    self.m_pause_button:addAction( ScaleAction( 0.3, InterpAction.Smooth, pool.get_v3f( 1.3, 1.3, 1.0 ) ) )
    self.m_pause_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
end

function LuaHud:ninjaDeath(game_over_callback, show_delay)

	local device_name = DeviceInfo.get():query("hw.description"):sub(1, 5)
	local ipod4 = (device_name == "iPod4")
	
	if not (not GameService.get():zeb() and not ipod4) then
        game_over_callback()
        return --no fuse stuff for zeb builds or ipod 4, which cant handle the memory load
    end
    
    local createIncentAdsButton = function()
	    
	    local chance_to_show_ad = swrve.getResourceAttributeFloat("fall_recovery_info", "frequency")
	    local display_time = swrve.getResourceAttributeFloat("fall_recovery_info", "duration")
	    local cancel_button
	    local roll = math.random()
	    local game = self.owner:getGame()
	    
	    if((roll < chance_to_show_ad) and (not self.m_end_boost_offered)) then
	        print("Hit the (post) roll")
            self:hideHudButtons()

            self.m_end_boost_offered = true
            
            local big_layout = swrve.getResourceAttributeBool("fall_recovery_info", "big_layout") 
            local big_panel
            local cancel_button_style = swrve.getResourceAttributeString("fall_recovery_info", "cancel_button_style")
            local cancel_button_pos = swrve.getResourceAttributeString("fall_recovery_info", "cancel_button_pos")
            
            if (big_layout) then
                --big layout overrides and forces cancel_button_style to be 'close_x'
                cancel_button_style = "close_x"
            end
            
            if (big_layout) then
                big_panel = self.owner:spawn( Window() )	
                big_panel.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0, 0, 1.0 ) )	
                textures.setTexture( big_panel, assets.textures.full_ia_panel )

                big_panel.scale = pool.get_v3f(0.0001, 0.0001, 1.0)
                
                local confirmation_text_style = swrve.getResourceAttributeString("fall_recovery_info", "confirm_prompt_copy")
                local body
                if(confirmation_text_style == "resume") then
                    body = lstring("postroll_video_confirmation_resume")
                elseif(confirmation_text_style == "save") then
                    body = lstring("postroll_video_confirmation_save")
                else
                    --default to "save"
                    body = lstring("postroll_video_confirmation_resume")
                    print("Confirmation text not found")
                end
                
                if (string.sub(engine.locale_language,1,string.len("zh")) == "zh") or(engine.locale_language == "ja") then
                    utility.createText( big_panel, body, {"ninja-text", 20.0, pool.get_colorf(1, 1, 1, 1), 70, TextAlign.Center, 0, 30, 1, 1}, nil, 180 )
                else
                    utility.createText( big_panel, body, {"ninja-text", 40.0, pool.get_colorf(1, 1, 1, 1), 70, TextAlign.Center, 0, 30, 1, 1}, nil, 180 )
                end
            end

            --on_release is nil here. is that okay?
            local button     = self:addButton( on_release )		

            if(big_layout)then	
                button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -50.0, 2.0 ) )
            else
                button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -20.0, 2.0 ) )
            end

            textures.setTexture( button, assets.textures.boost_button )
            button.scale = pool.get_v3f(0.0001, 0.0001, 1.0)
            
            local icon_style = swrve.getResourceAttributeString("fall_recovery_info", "button") .. "_icon"
            local icon = utility.createWindowWithTexture(button, assets.textures[icon_style])
            icon.location = pool.get_v3f(66, 0, 1.0)
            
            local shield_will_be_awarded = swrve.getResourceAttributeBool("fall_recovery_info", "give_shield")
            if(shield_will_be_awarded)then
                local shield = utility.createWindowWithTexture(button, assets.textures.boost_shield_icon)
                shield.location = pool.get_v3f(-68, 0, 1.0)
            end
            
            local rocket = utility.createWindowWithTexture(button, assets.textures.boost_rocket_icon)
            rocket.location = pool.get_v3f(-68, 0, 2.0)
            
            if(big_layout) then
                big_panel:addAction( SleepAction( show_delay ) )
                big_panel:addAction( ScaleAction( 0.15, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
                big_panel:addAction( ScaleAction( 0.07, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
                big_panel:addAction( SleepAction( math.floor(display_time) ) )
                big_panel:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.0001, 0.0001, 1.0 ) ) )
                big_panel:addAction( DestroyAction(0.0) )
            end
            
            button.scale = pool.get_v3f(0.001, 0.001, 1.0)
            button:addAction( SleepAction( show_delay ) )

            button:addAction( ScaleAction( 0.15, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
            button:addAction( ScaleAction( 0.07, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )

            local s = SequenceAction()
            s:addAction( SleepAction( 0.56 ) )
            s:addAction( ScaleAction( 0.15, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
            s:addAction( ScaleAction( 0.07, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
            s:addAction( ScaleAction( 0.15, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
            s:addAction( ScaleAction( 0.07, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
            local l = FiniteLoopAction(math.floor(display_time))
            l:addAction(s)
            button:addAction(l) 
            
            button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.001, 0.001, 1.0 ) ) )
            button:addAction( LuaAction(0.0, function() game_over_callback() end ) )
            button:addAction( DestroyAction(0.0) )
   
            local cancel_enabled = swrve.getResourceAttributeBool("fall_recovery_info", "allow_cancel")
            local button_copy = swrve.getResourceAttributeString("fall_recovery_info", "copy")

            if engine.locale_language == "de" or engine.locale_language == "it" or engine.locale_language == "ja" then
                utility.createText( button, lstring(button_copy), {"ninja-text", 44, pool.get_colorf(1, 1, 1, 1), 100, TextAlign.Center, 0, 3, 1, 1} )
            else
                utility.createText( button, lstring(button_copy), {"ninja-text", 30, pool.get_colorf(1, 1, 1, 1), 70, TextAlign.Center, 0, 3, 1, 1} )
            end
            
            local cancel_button
    
            button.on_release = function( inner )
            
                local show_ad_callback = function(handle, code, btnidx, already_approved)
    
                    game.delta_time_factor = 1.0
                    button.enabled = false
                    if(engine.platform.device == "android") then
                        game.lua:ceaseIgnoringPauseEvents()
                    end
                                
                    local yes_button_selected = false
                    
                    if(not already_approved) then
                        local yes_button_on_right = swrve.getResourceAttributeBool("fall_recovery_info", "confirm_prompt_watch_right")
                        if(engine.platform.device == "android") then
                            yes_button_on_right = not yes_button_on_right
                        end
                        
                        if(yes_button_on_right)then
                            if(btnidx:getInt() == 1)then
                                yes_button_selected = true
                            end
                        else
                            if(btnidx:getInt() == 0)then
                                yes_button_selected = true
                            end
                        end
                    end
                        
                    if already_approved or yes_button_selected or big_layout then
                        
                        UserData.get():setConfirmedPostrollVideo(true)  
                        chunks.sound.play( assets.sounds.click )
                        button:clearActions()
                        
                        if(cancel_enabled) then
                            cancel_button:clearActions()
                        end
                        
                        if(engine.platform.device == "android") then
                            game.lua:ignoreNextPauseEvent()
                        end
                    
                        local ad_shown_callback = function()

                            local activate_shield = swrve.getResourceAttributeBool("fall_recovery_info", "give_shield")
                            if(activate_shield) then
                                self:activateIncentivizedShield()
                            end
                            self:activateRevivalBoost(function() self:showHudButtons() end)
                        
                            
                        
                            button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                        
                            button:addAction( LuaAction(1.4, function()        
                                --just a little delay to give the fuse video time to clear out of the view      
                                self:showHudButtons()
                                game.lua:getNinja().lua:reviveUsed()  
                                chunks.analytics.reportVideoWatched(false)	    
                                chunks.incent_ads.preloadAd("postroll")                    
                            end  ) )
                            button:addAction( DestroyAction(0.0) )
                            
                            if(cancel_enabled) then
                                cancel_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                                cancel_button:addAction( SleepAction( 1.4 ) )
                                cancel_button:addAction( DestroyAction(0.0) )
                            end
                            
                            if(big_layout) then
                                big_panel:clearActions()
                                big_panel:addAction( DestroyAction(0.0) )
                            end


                            if(engine.platform.device == "android") then
                                game.lua:ignoreNextPauseEvent()
                            end
                        end
            
                        local no_ad_callback = function()

                            if(engine.platform.device == "android") then
                                game.lua:ceaseIgnoringPauseEvents()
                            end
                        
                            button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                            button:addAction( DestroyAction(0.0) )
                            button:addAction( LuaAction(0.0, function() game_over_callback() end  ) )

                            if(cancel_enabled) then
                                cancel_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
    		                    cancel_button:addAction( DestroyAction(0.0) )
                            end
                            
                            if(big_layout) then
                                big_panel:clearActions()
                                big_panel:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                                big_panel:addAction( DestroyAction(0.0) )
                            end

                        end
            
                        local result = chunks.incent_ads.showIncentVideo("postroll", ad_shown_callback, no_ad_callback)
                        chunks.analytics.reportVideoStarted(false)	
                    
                    else--user said "no thanks" to the fuse ad prompt
                        button:clearActions()
                        self:showHudButtons()
                        button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                        button:addAction( DestroyAction(0.0) )
                        button:addAction( LuaAction(0.0, function() game_over_callback() end  ) )
                        
                        if(cancel_enabled) then
                            cancel_button:clearActions()
		                    cancel_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
		                    cancel_button:addAction( DestroyAction(0.0) )
		                end
		                
                        if(big_layout) then
                            big_panel:clearActions()
                            big_panel:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                            big_panel:addAction( DestroyAction(0.0) )
                        end

                    end
                
                end--of show_ad_callback

                if(cancel_enabled) then
                    --disable the cancel button to prevent monkey bug shenanigans
                    cancel_button.enabled = false
                end
                
                local show_confirmation = swrve.getResourceAttributeBool("fall_recovery_info", "show_confirm_prompt")
                
                if((not show_confirmation) or UserData.get():getConfirmedPostrollVideo() or big_layout)then
                    if(engine.platform.device == "android") then
                        game.lua:ignoreNextPauseEvent()
                    end

                    show_ad_callback(nil, nil, nil, true)
                else
                    local ui = PlatformUIService.get()
                    if ui ~= nil then
                
                        local confirmation_text_style = swrve.getResourceAttributeString("fall_recovery_info", "confirm_prompt_copy")
                        local body
                        if(confirmation_text_style == "resume") then
                            body = lstring("postroll_video_confirmation_resume")
                        elseif(confirmation_text_style == "save") then
                            body = lstring("postroll_video_confirmation_save")
                        else
                            --default to "save"
                            body = lstring("postroll_video_confirmation_resume")
                            print("Confirmation text not found")
                        end
    				            
                        local details = chunks.social.getSocialDetails()
                        local title = lstring("watch_video")
                        local okay = lstring("watch")
                        local not_okay = lstring("cancel")
                        
                        local yes_button_on_right = swrve.getResourceAttributeBool("fall_recovery_info", "confirm_prompt_watch_right")
                        if(engine.platform.device == "android") then
                            yes_button_on_right = not yes_button_on_right
                        end
                        
                        local buttons
                        if(yes_button_on_right)then
                            buttons = {not_okay, okay}
                        else
                            buttons = {okay, not_okay}
                        end
                    
                        game.delta_time_factor = 0.0
                        if(engine.platform.device == "android") then
                            game.lua:ignoreNextPauseEvent()
                        end
                        self.m_async_handle = ui:showPrompt(title, body, buttons, show_ad_callback)
                    end
                end
            end
            
            if(cancel_enabled) then
                --now create the cancel button
                cancel_button     = self:addButton( on_release )		
                
                local setup_close_x_button = function()
                    if(big_layout) then
                        if (cancel_button_pos == "top_right") then
                            cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 94.0, 100.0, 2.0 ) ) 
                            textures.setTexture( cancel_button, assets.textures.small_exit_button )
                        elseif (cancel_button_pos == "top_left") then
                            cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( -94.0, 100.0, 2.0 ) ) 
                            textures.setTexture( cancel_button, assets.textures.small_exit_button )
                        else --in case of bad data from swrve, fall back to top right
                            print("Cancel button position not recognized")
                            cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 94.0, 100.0, 2.0 ) ) 
                            textures.setTexture( cancel_button, assets.textures.small_exit_button )
                        end
                    else
                        if (cancel_button_pos == "top_right") then
                            cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 80.0, 5.0, 5.0 ) ) 
                            textures.setTexture( cancel_button, assets.textures.small_exit_button )
                        elseif (cancel_button_pos == "top_left") then
                            cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( -80.0, 5.0, 5.0 ) ) 
                            textures.setTexture( cancel_button, assets.textures.small_exit_button )
                        else --in case of bad data from swrve, fall back to top right
                            print("Cancel button position not recognized")
                            cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 80.0, 5.0, 5.0 ) ) 
                            textures.setTexture( cancel_button, assets.textures.small_exit_button )
                        end
                    end
                end
            
                if (cancel_button_style == "close_x") then
                    setup_close_x_button()
                elseif 	(cancel_button_style == "full") then
                    cancel_button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -70.0, 2.0 ) ) 
                    textures.setTexture( cancel_button, assets.textures.replay_button_locale )
                else --in case of bad data from swrve, fall back to x-style button
                    print("Cancel button style not recognized")
                    setup_close_x_button()
                end

                if 	(cancel_button_style == "full") then
                    local cancel_button_copy = swrve.getResourceAttributeString("fall_recovery_info", "cancel_copy")
                    utility.createText( cancel_button, lstring(cancel_button_copy), {"ninja-text", 30.0, pool.get_colorf(1, 1, 1, 1), 70, TextAlign.Center, 0, 3, 1, 1} )
                end
                
                cancel_button.size = cancel_button.size / 1.5
                cancel_button.scale = pool.get_v3f(0.01, 0.01, 1.0)
            
                cancel_button:addAction( SleepAction( show_delay ) )
                cancel_button:addAction( ScaleAction( 0.15, InterpAction.Smooth, pool.get_v3f( 1.2, 1.2, 1.0 ) ) )
                cancel_button:addAction( ScaleAction( 0.07, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
            
                cancel_button:addAction( SleepAction( 1.0 * math.floor(display_time)) )
            
                cancel_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                cancel_button:addAction( DestroyAction(0.0) )
            
                cancel_button.on_release = function( inner )
                    if(game.paused) then
                        return
                    end

                    button.enabled = false
                    cancel_button.enabled = false
                     
                    button:clearActions()
                    button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                    button:addAction( DestroyAction(0.0) )
                
                    cancel_button:clearActions()
                    cancel_button:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.01, 0.01, 1.0 ) ) )
                    cancel_button:addAction( DestroyAction(0.0) )
                
                    if(big_layout) then
                        big_panel:clearActions()
                        big_panel:addAction( ScaleAction( 0.2, InterpAction.Smooth, pool.get_v3f( 0.0001, 0.0001, 1.0 ) ) )
                        big_panel:addAction( DestroyAction(0.0) )
                    end

                    game_over_callback()
                end
            end
		else
		    print("Missed the (post) roll")
		    game_over_callback()
		end
    end
    
    local postroll_trigger = swrve.getResourceAttributeString("feature_info", "postroll_trigger")
    
    if ((postroll_trigger == "incent_ad") and (self.m_game.lua.best_height > 0)) then
    
        local ad_available = chunks.incent_ads.videoAvailable("postroll")
        local postroll_feature = swrve.getResourceAttributeString("feature_info", "postroll_feature")
        if((postroll_feature == "fall_recovery") and (ad_available)) then
            createIncentAdsButton()
        else
            game_over_callback()
        end
    else
        print("postroll trigger feature does not exist (or this is the first run)")
        game_over_callback()
    end
end

function LuaHud:openItemStore(ask_use)
   local item_store = self.m_spawning_ground:spawn( chunks.shield_store.spawn() )
   if(ask_use) then
		item_store.lua:setImmediateUseFlag()
   end
end

function LuaHud:getSpawningGround()
	return self.m_spawning_ground
end

function LuaHud:isMainMenu()
	return false
end

function LuaHud:activateRevivalBoost(callback)
    local game = self.owner:getGame()
    game.lua:getNinja().lua:incentiveRevival(callback)
end

function LuaHud:activateIncentivizedBoost(callback)
    local min_boost_distance = swrve.getResourceAttributeFloat("start_boost_info", "height_min")
    local max_boost_distance = swrve.getResourceAttributeFloat("start_boost_info", "height_max")
    local boost_distance
    if(max_boost_distance > min_boost_distance) then
        boost_distance = min_boost_distance + (math.random() * (max_boost_distance - min_boost_distance))
    else
        boost_distance = min_boost_distance
    end
    print("resolved boost distance is ", boost_distance)
    
    local game = self.owner:getGame()
    game.lua:getNinja().lua:incentiveBonusStart(boost_distance, callback)
end

function LuaHud:activateIncentivizedShield()
	local game = self.owner:getGame()
	local shield = game.lua:getNinja().lua:getShield()
    shield.lua:equip()
end

function LuaHud:activateShield(from_immediate_use_dialog)
	local game = self.owner:getGame()
	local shield = game.lua:getNinja().lua:getShield()
	if( UserData.get():getShields() > 0) then
		if shield.lua:canEquip() then
			local former_shields = UserData.get():getShields()
			UserData.get():setShields(former_shields - 1)
			self.m_shield_meter.lua.update()
			shield.lua:silentEquip()
			chunks.sound.play( assets.sounds.shieldpack ) 
			if (from_immediate_use_dialog) then
				shield.lua:silentUnequip()
				shield.lua:equip()
			else
				self:createFlyingShield()
			end
		end
	end
end

function LuaHud:pulsateShieldButton()
	self.m_shield_button.lua.pulsate()
end

function LuaHud:incrementShieldCounter()
	local shield_ticker = self.m_shield_meter
	
	local shields = UserData.get():getShields()
	if(shields > 9999) then shields = 9999 end
	
	if(shield_ticker.lua.delay == 0) then
		if(shield_ticker.value < shields) then
			local difference = (shields - shield_ticker.value)
			if (difference > 100) then
				shield_ticker.value = shield_ticker.value + 100
			elseif (difference > 10) then
				shield_ticker.value = shield_ticker.value + 10
			elseif (difference >= 1) then
				shield_ticker.value = shield_ticker.value + 1
			end
			shield_ticker.lua.delay = 2
		else
			shield_ticker.value = shields
			shield_ticker.lua.delay = 2
		end
	else
		shield_ticker.lua.delay = shield_ticker.lua.delay - 1
	end
end

function LuaHud:createFlyingShield()
	flying_shield = self.owner:spawn( Window() )
	textures.setTexture( flying_shield, assets.textures.shieldpack )
	flying_shield.scale = pool.get_v3f( 0.4, 0.4, 1.0 )
	
	if (engine.platform.device == "ipad") then
		flying_shield.location = pool.get_v3f( -160, -227, 1.0 )
	else
		flying_shield.location = pool.get_v3f( -139, -227, 1.0 )
	end
	self.m_flying_shield = flying_shield
end

function LuaHud:tick(game, dt)

	self:incrementShieldCounter()
	
	if(self.m_flying_shield) then
		--move the flying shield quickly to the ninja.  cant do this with an action
		--since the position of the ninja is variable	
		local ninja_position = self.owner:getGame().lua:getNinja().location
		local shield_position = self.m_flying_shield.location
		local target_vector = ninja_position - shield_position
		local normal_target, length = utility.normalize(target_vector)	
		if(length > 20) then
			self.m_flying_shield.location = self.m_flying_shield.location + (normal_target * 1000 * dt)
		else
			self.owner:getGame().lua:getNinja().lua:getShield().lua:silentUnequip()
			self.owner:getGame().lua:getNinja().lua:getShield().lua:equip()
			self.m_flying_shield:addAction(DestroyAction(0.0))
			self.m_flying_shield = nil
		end	
	end
	
end

function LuaHud:getTicker()
	return self.ticker
end

