------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Knight             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local effects   = chunks.effects
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaKnight --
LuaKnight = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaKnight )

-- The created method.
function LuaKnight:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "knight"
	owner.location = pool.get_v3f( -106.0, 0.0, 0.0 )
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.knight )

		self.visuals.knight = actor
	end
	
	self:setVisual( "knight" ) 
	
	-- We're not killable.
	self.killable = false
end

-- The collided event.
function LuaKnight:onCollided( other )
	-- Spawn particles.
	do
		particles.spawn_normal( self.owner:getGame().lua.root, 
		                        assets.textures.particles_spark, 
		                        self.visual:getWorldLocation() )
	end
	
	-- Spawn slice.
	do
		effects.spawn_flash( self.owner:getGame().lua.root, 
		                     assets.textures.effects_slice, 
		                     self.visual:getWorldLocation() )
	end
	
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_enemy )
end

-- Setup easy sequence.
function LuaKnight:easy()
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		
		s:addAction( MoveOffsetAction( utility.frandom( 2.0, 2.5 ), InterpAction.Lerp, 
		                               pool.get_v3f( 0.0, -150.0, 0.0 ) ) )
		
		actor:addAction( s )
	end	
end




