------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Kraken             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound
local constants = engine.platform.constants

-- LuaKraken --
LuaKraken = chunks.class.inherit( chunks.enemy.LuaEnemy )
{
	offset = v2f( -50.1 + 12.0, 111.8 ),
	head   = nil
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaKraken )

-- The created method.
function LuaKraken:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "kraken"
end

-- Bonus meter override.
function LuaKraken:bonusmeterOverride( actor )
	actor.scale = pool.get_v3f( 0.7, 0.7, 1.0 )

	textures.setTexture( actor, assets.textures.icon_eye )
end	

-- The collided event.
function LuaKraken:onCollided( other, is_bonus )
	local game       = self.owner:getGame()
	local ninja      = game.lua.ninja
	local spawn_meat = function( texture )
		local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
		local system  = spawner( self.owner:getGame().lua.root, 
		                         texture, 
		                         self.visual:getWorldLocation() )
		
		system:clearScaleRanges()   
		
		if is_bonus then
			system.particles_per_spawn_range = pool.get_v2i( 5, 9 )
		else
			system.particles_per_spawn_range = pool.get_v2i( 2, 4 )
		end
		
		system.lifetime              = utility.frandom( 0.05, 0.05 ) 
		system.spawn_frequency_range = pool.get_v2f( 0.1, 0.1 )
		
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) )
		system:addColorRange( pool.get_colorf( 1.0, 1.0, 1.0, 0.0 ) )
	end
	
	-- Change head textures.
	do
		textures.setTexture( self.head, assets.textures.kraken_injured )
	end
	
	-- Spawn fish steak.
	spawn_meat( assets.textures.particles_fish_steak )
	
	-- Spawn flesh.
	spawn_meat( assets.textures.particles_flesh )
	
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_kraken )
end

-- Setup easy sequence.
function LuaKraken:easy( tentacle_bottom_owner, tentacle_top_owner )
	local owner          = self.owner
	local game           = owner:getGame()
	local ninja          = game.lua.ninja
	local world_size     = constants.scene.world_size
	local peek_delay     = 1.5
	local attack_delay   = 3.0
	local tentacle_drift = -110.0
	
	-- The head piece: move it in first as a warning.
	do
		local actor = owner:spawn( SpriteActor() )
		local size  = assets.textures.kraken.size
		
		textures.setTexture( actor, assets.textures.kraken )
		
		-- Its initial position should be offscreen.
		actor.location = pool.get_v3f( math.abs( ninja.lua.sides.left.offset.x ) + size.x * 0.5 + 25.0, 100.0, 0.0 )
		
		-- Remember the head actor for collisions.
		self.head = actor
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( MoveOffsetAction( peek_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( -1.0 * size.x, 0.0, 0.0 ) ) )
			s:addAction( MoveOffsetAction( peek_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( size.x, 0.0, 0.0 ) ) )		
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( -1.0 * size.x, 0.0, 0.0 ) ) )
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( size.x, 0.0, 0.0 ) ) )	                               
			
			actor:addAction( s )
		end
	end
	
	local tentacle_bottom = nil
	
	-- The bottom tentacle.
	do
		local actor = tentacle_bottom_owner:spawn( SpriteActor() )
		local size  = assets.textures.tentacle_bottom.size
		
		textures.setTexture( actor, assets.textures.tentacle_bottom )
		
		-- Its initial position should be offscreen.
		actor.location = pool.get_v3f( world_size.x * 0.5 + size.x * 0.5 + 5.0, -160.0, 0.0 )
		
		tentacle_bottom = actor
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( SleepAction( peek_delay ) )
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( -1.0 * size.x, 0.0, 0.0 ) ) )
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( size.x, tentacle_drift, 0.0 ) ) )			                 
			
			actor:addAction( s )
		end
	end
	
	-- The top tentacle.
	do
		local actor = tentacle_top_owner:spawn( CollidableActor() )
		local size  = tentacle_bottom.size
		
		textures.setTexture( actor, assets.textures.tentacle_top )
		
		-- Its initial position should be offscreen.
		actor.location = tentacle_bottom.location + pool.get_v3f( self.offset.x, self.offset.y, 0.0 )
		
		self.visuals.kraken = actor
		self:setVisual( "kraken" )
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( SleepAction( peek_delay ) )
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( -1.0 * size.x, 0.0, 0.0 ) ) )
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( size.x, tentacle_drift, 0.0 ) ) )			                 
			
			actor:addAction( s )
		end
	end
	
	-- Play warning sound.
	sound.play_random( assets.sounds.kraken_warning )
end

-- Setup hard sequence.
function LuaKraken:hard( tentacle_bottom_owner, tentacle_top_owner )
	local owner          = self.owner
	local game           = owner:getGame()
	local ninja          = game.lua.ninja
	local world_size     = constants.scene.world_size
	local peek_delay     = 1.0
	local attack_delay   = 1.25
	local tentacle_drift = -110.0
	
	-- The head piece: move it in first as a warning.
	do
		local actor = owner:spawn( SpriteActor() )
		local size  = assets.textures.kraken.size
		
		textures.setTexture( actor, assets.textures.kraken )
		
		-- Its initial position should be offscreen.
		actor.location = pool.get_v3f( math.abs( ninja.lua.sides.left.offset.x ) + size.x * 0.5 + 25.0, 100.0, 0.0 )
		
		-- Remember the head actor for collisions.
		self.head = actor
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( MoveOffsetAction( peek_delay, InterpAction.Smooth,
			                               pool.get_v3f( -1.0 * size.x, 0.0, 0.0 ) ) )
			s:addAction( SleepAction( attack_delay / 2.0 ) )                               
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( size.x, 0.0, 0.0 ) ) )	                               
			
			actor:addAction( s )
		end
	end
	
	local tentacle_bottom = nil

	-- The bottom tentacle.
	do
		local actor = tentacle_bottom_owner:spawn( SpriteActor() )
		local size  = assets.textures.tentacle_bottom.size
		
		textures.setTexture( actor, assets.textures.tentacle_bottom )
		
		-- Its initial position should be offscreen.
		actor.location = pool.get_v3f( world_size.x * 0.5 + size.x * 0.5 + 5.0, -160.0, 0.0 )
		
		tentacle_bottom = actor
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( SleepAction( peek_delay ) )
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( -1.0 * size.x, 0.0, 0.0 ) ) )
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( size.x, tentacle_drift, 0.0 ) ) )			                 
			
			actor:addAction( s )
		end
	end
	
	-- The top tentacle.
	do
		local actor = tentacle_top_owner:spawn( CollidableActor() )
		local size  = tentacle_bottom.size
		
		textures.setTexture( actor, assets.textures.tentacle_top )
		
		-- Its initial position should be offscreen.
		actor.location = tentacle_bottom.location + pool.get_v3f( self.offset.x, self.offset.y, 0.0 )
		
		self.visuals.kraken = actor
		self:setVisual( "kraken" )
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( SleepAction( peek_delay ) )
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( -1.0 * size.x, 0.0, 0.0 ) ) )
			s:addAction( MoveOffsetAction( attack_delay / 2.0, InterpAction.Smooth,
			                               pool.get_v3f( size.x, tentacle_drift, 0.0 ) ) )			                 
			
			actor:addAction( s )
		end
	end
	
	-- Play warning sound.
	sound.play_random( assets.sounds.kraken_warning )
end

-- The bonus setup method.
function LuaKraken:bonus( ninja )
	local game          = ninja:getGame()
	local offset        = ninja.lua.sides.left.offset
	local dir           = ( ninja:getWorldLocation().x < 0.0 ) and -1 or 1
	local initial_delay = 0.8
	local bob_delay     = 0.6
	local rotation      = v3f( 0.0, ( dir == -1 ) and 0.0 or 180.0, 0.0 )
	local num_bobs      = 8
	local lifetime      = initial_delay + bob_delay * num_bobs
	local fountain      = nil
	
	-- Set visual.
	textures.setTexture( ninja.lua.visual, assets.textures.ninja_falling )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 2.2 )
	
	-- Initial rotation.
	ninja.lua:setRotation( rotation, rotation )
	
	-- Fountain.
	do
		local actor = game:getScene():spawn( SpriteActor() )
		local size  = assets.textures.bonus_fountain.size
		
		textures.setTexture( actor, assets.textures.bonus_fountain )
		
		-- Initial location.
		actor.location = pool.get_v3f( 0.0, -250.0 - size.x / 2.0, offset.z - 1.0 )
		if constants.WIDESCREEN then
			actor.location = actor.location + v3f(0, -44, 0)
		end
	
		fountain = actor
		
		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( MoveOffsetAction( initial_delay, InterpAction.Smooth,
			                               pool.get_v3f( 0.0, size.x * 0.9, 0.0 ) ) )
			
			-- Bob.
			for i = 1,num_bobs - 1 do
				s:addAction( MoveOffsetAction( bob_delay * 0.5, InterpAction.Smooth,
											   pool.get_v3f( 0.0, -40.0, 0.0 ) ) )
				s:addAction( MoveOffsetAction( bob_delay * 0.5, InterpAction.Smooth,
											   pool.get_v3f( 0.0, 40.0, 0.0 ) ) )                               
			end
		
			s:addAction( MoveOffsetAction( bob_delay, InterpAction.Smooth,
			                               pool.get_v3f( 0.0, size.x * -1.2, 0.0 ) ) )                               
			s:addAction( DestroyAction( 0.0 ) )                               
			
			actor:addAction( s )
		end	
	end
	
	-- Actions.
	do
		local s = SequenceAction()
		
		-- Move to the center.
		if constants.WIDESCREEN then
			s:addAction( MoveAction( initial_delay * 0.5, InterpAction.Smooth,
		                         pool.get_v3f( 0.0, -204.0, offset.z ) ) )
		else
			s:addAction( MoveAction( initial_delay * 0.5, InterpAction.Smooth,
		                         pool.get_v3f( 0.0, -160.0, offset.z ) ) )
		end
		s:addAction( MoveOffsetAction( initial_delay * 0.5, InterpAction.Smooth,
		                         pool.get_v3f( 0.0, 120.0, 0.0 ) ) )                         
		
		-- Bob.
		for i = 1,num_bobs - 1 do
			s:addAction( MoveOffsetAction( bob_delay * 0.5, InterpAction.Smooth,
			                               pool.get_v3f( 0.0, -60.0, 0.0 ) ) )
			s:addAction( MoveOffsetAction( bob_delay * 0.5, InterpAction.Smooth,
			                               pool.get_v3f( 0.0, 60.0, 0.0 ) ) )                               
		end
		
		s:addAction( MoveAction( bob_delay, InterpAction.Smooth,
		                         pool.get_v3f( math.abs( offset.x ) * dir, offset.y, offset.z ) ) )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )
		
		ninja:addAction( s )
	end
	
	-- Particles.
	do
		local system = fountain:spawn( ParticleSystem() )

		-- Set explosion parameters.
		system.location                  = pool.get_v3f( 0.0, 40.0, 0.5 )
		system.texture                   = assets.textures.particles_droplet.path
		system.particle_size             = assets.textures.particles_droplet.size
		system.explosion                 = true
		system.lifetime                  = lifetime
		system.lifetime_range            = pool.get_v2f( 0.9, 1.0 )
		system.particles_per_spawn_range = pool.get_v2i( 6, 8 )
		system.spawn_frequency_range     = pool.get_v2f( 0.03, 0.04 )
		system.speed_range               = pool.get_v2f( 180.0, 240.0 )
		system.initial_rotation_range    = pool.get_v2f( 0.0, 360.0 )
		system.rotation_speed_range      = pool.get_v2f( 120.0, 150.0 )
        system.acceleration_range_x   = pool.get_v2f( -30.0, 0.0 )
        system.acceleration_range_y   = pool.get_v2f( 0.0, -10.0 )
		system.follow_system             = true
		
		-- scale transitions
		system:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
		system:addScaleRange( pool.get_v2f( 0.4, 0.6 ) )
		system:addScaleRange( pool.get_v2f( 0.6, 0.8 ) )
		system:addScaleRange( pool.get_v2f( 0.4, 0.6 ) )
		system:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_kraken )
	sound.play( assets.sounds.bonus_loop_kraken )
	
	return lifetime
end

-- Update the ninja's trailer.
function LuaKraken:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 1.0, 0.0, 0.0, 1.0 ) ) )
			
	trailer:addAction( s )	
end