------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- LuaMainMenu        --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local constants = engine.platform.constants
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local textures  = chunks.textures
local fade      = chunks.fade
local events    = chunks.events
local utility   = chunks.utility
local swrve     = chunks.swrve
local sound  = chunks.sound
-- LuaMainMenu --
LuaMainMenu = chunks.class.inherit( chunks.window.LuaWindow ){
    m_spawning_ground = nil,
    m_iap_handle = nil,
    m_active_popup = nil
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaMainMenu )

-- The created method.
function LuaMainMenu:created( owner )
    engine:doStartupTasks()
    --this call is interspersed throughout the code at critical locations where it is possible that audio states will get screwed up by ad interference
    Audio.get():restoreState()
    
    local game     = owner:getGame()
    local x_offset = 5.0
    local y_offset = constants.ad_height + 10.0
    local button_offset = y_offset

    if not GameService.get():zeb() then
        chunks.incent_ads.preloadAd("postroll")
        chunks.incent_ads.preloadAd("preroll")
    end
    
    -- Background.
    do
        textures.setTexture( owner, assets.textures.main )
    end

    --Blank Window from which to spawn the store, for input issues
    local spawning_ground = owner:spawn( Window() )
    spawning_ground.size = pool.get_v2f(0, 0)
    self.m_spawning_ground = spawning_ground

    -- Generates a function to play the specified theme.
    local play_game = function( theme )
        return function( outer )
            local userdata = UserData.get()
            local callback = function( inner )
                -- If this is the first time they've
                -- played, show the tutorial menu.
                if userdata.first_time_played then
                    self.m_active_popup = game:getWindow():spawn( chunks.tutorial.spawn() )

                    self.m_active_popup.lua:play( theme )
                    owner:destroy()
                    userdata.first_time_played = false
                    userdata:flush()
                else
                    game.lua:onPlay( theme )
                end
            end

            -- Change fade accordingly.
            if userdata.first_time_played then
                fade.fade_out( owner, callback )
            else
                fade.fade_out_loading( owner, callback )
            end

            events.on_main_menu_disappeared()
        end
    end

    -- Privacy Policy.
    do
        local privacy_policy_btn
        local on_release = function( inner )
            events.show_privacy_policy()
        end

        if engine.default_language_enabled then
            privacy_policy_btn  = self:addButton( on_release )
            textures.setTexture( privacy_policy_btn, assets.textures.privacy_button )

            local text = privacy_policy_btn:spawn(Window())
            textures.setTexture(text, assets.textures.privacy_text)
            text.receive_input = false
        else
            local text = lstring( "privacy_policy" )
            local format
            
            if(string.sub(engine.locale_language,1,string.len("zh")) == "zh") then
                format = {"ninja-text", 24.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 100.0, TextAlign.Center, -1.0, 6.0}
            else
                format = {"ninja-text", 12.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 100.0, TextAlign.Center, -1.0, 6.0}
            end
                
            privacy_policy_btn = utility.createButtonWithTextAndTexture(self.owner, text, format, assets.textures.options_screen_blank_button, nil, 100 )
            privacy_policy_btn.on_release = function() on_release(); sound.play( assets.sounds.click ) end
        end
        privacy_policy_btn.anchor = make_anchor(Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f( x_offset + 5, button_offset, 1.0))
        button_offset = button_offset + 38.0
    end

    -- Shields Button
    if not GameService.get():zeb() then
        do
            local on_release = function( inner )
                if(UserData.get():getFreeShieldsIssued()) then
                    self:openItemStore()
                else
                    -- Bail if there is no iAP support
                    if not chunks.iap.purchasingAvailable() then
                        chunks.iap.displayPurchasingUnavailable()
                        return
                    end
    
                    self:openShieldInfo()
                end
            end
    
            local button = nil
            if engine.default_language_enabled then
                button = self:addButton( on_release )
                textures.setTexture( button, assets.textures.item_store )
            else
                local shields_text = lstring( "shields" )
                local format = {"ninja-text", 40.0, pool.get_colorf(0.86, 0.98, 1.0, 1.0), 105.0, TextAlign.Center, 0.0, 6.0}
                local fx = {drop_shadow = true, shadow_color = colorf( 0.0, 0.27, 0.48, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 100000}
                button = utility.createButtonWithTextAndTexture(self.owner, shields_text, format, assets.textures.item_store_locale, fx )
                button.on_release = function() on_release(); sound.play( assets.sounds.click ) end
    
            end
    
            button.anchor = make_anchor(Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f( x_offset, button_offset, 1.0))
            button_offset = button_offset + 48.0
        end
    end
    
    if not chunks.iap.adBuyoutPurchased() and not GameService.get():zeb() then
        local on_release = function( inner )
            -- Bail if there is no iAP support
            if not chunks.iap.purchasingAvailable() then
                chunks.iap.displayPurchasingUnavailable()
                return
            end

            local products_consumed_callback = function()
                local callback = function( o )
                    game:getWindow():spawn( chunks.main.spawn() )
                    owner:destroy()
                end

                fade.fade_out( owner, callback )
            end
            self.m_iap_handle = chunks.iap.purchaseProduct("ad_buyout", function() end, products_consumed_callback)
        end

        local actor = nil
        if engine.default_language_enabled then
            actor  = self:addButton( on_release )
            textures.setTexture(actor, assets.textures.adbuyout_button)
        else
            local remove_ads = lstring( "remove_ads" )
            local format = {"ninja-text", 40.0, pool.get_colorf(0.0, 0.588, 1.0, 1.0), 105.0, TextAlign.Center, -1.0, 6.0}

            if engine.locale_language == "de" or engine.locale_language == "it" then
                format[2] = 16.0
                actor = utility.createButtonWithTextAndTexture(self.owner, remove_ads, format, assets.textures.remove_ads_button_locale )
            else
                actor = utility.createButtonWithTextAndTexture(self.owner, remove_ads, format, assets.textures.options_button_locale )
            end
            actor.on_release = function() on_release(); sound.play( assets.sounds.click ) end
        end

        actor.anchor = make_anchor(Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f(x_offset + 3.0, button_offset, 1.0))
        button_offset = button_offset + 42.0

        if engine.locale_language == "de" or engine.locale_language == "it" then
            button_offset = button_offset + 10
        end
    end

    -- Options Button
    do
        local on_release = function( inner )
            local callback = function( o )
                game:getWindow():spawn( chunks.options.spawn() )
                owner:destroy()
            end

            fade.fade_out( owner, callback )

            events.on_main_menu_disappeared()
        end

        local actor = nil
        if engine.default_language_enabled then
            actor  = self:addButton( on_release )
            textures.setTexture( actor, assets.textures.options_button )
        else
            local options_text = lstring( "options" )
            local format = {"ninja-text", 40.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 105.0, TextAlign.Center, -1.0, 6.0}
            actor = utility.createButtonWithTextAndTexture(self.owner, options_text, format, assets.textures.options_button_locale )
            actor.on_release = function() on_release(); sound.play( assets.sounds.click ) end
        end

        actor.anchor = make_anchor(Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f(x_offset + 3.0, button_offset, 1.0))
        button_offset = button_offset + 46.0
    end

    -- Play Button
    do
        local actor = nil
        if engine.default_language_enabled then
            actor  = self:addButton( play_game( "classic") )
            textures.setTexture( actor, assets.textures.play_button )
        else
            local options_text = lstring( "play" )
            local format = {"ninja-text", 40.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 75.0, TextAlign.Center, -1.0, 8.0}
            actor = utility.createButtonWithTextAndTexture(self.owner, options_text, format, assets.textures.play_button_locale )
            actor.on_release = function() play_game( "classic")(); sound.play( assets.sounds.click ) end
        end

        actor.anchor = make_anchor(Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f(x_offset + 16.0, button_offset, 1.0))

        -- Setup a bobbing action.
        do
            local l = LoopAction()

            l:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 1.1, 1.1, 1.0 ) ) )
            l:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )

            actor:addAction( l )
        end
    end

    -- Leaderboards.
    local gc = GameCenterService.get()
    if gc then
        local on_release = function( inner )
            events.on_leaderboard_ui("classic")
        end
        local actor  = self:addButton( on_release )
        actor.anchor = make_anchor( Anchor.Anchor_BR, Anchor.Anchor_BR, pool.get_v3f( -x_offset - 20, y_offset, 1.0 ) )

        local info     = DeviceInfo.get()
        local version = info:query( "platform.version.major" )
        if(version < 7)then
            textures.setTexture( actor, assets.textures.leaderboards_button )
        else
            textures.setTexture( actor, assets.textures.leaderboards_button_new )
        end
    end

    -- Fade in.
    do
        fade.fade_in( owner )
    end

    events.on_main_menu_appeared()
end

function LuaMainMenu:openItemStore()
    if chunks.iap.purchasingAvailable() then   
        self.m_active_popup = self.m_spawning_ground:spawn( chunks.shield_store.spawn() )
    else
        chunks.iap.displayPurchasingUnavailable()
    end
end

function LuaMainMenu:openShieldInfo()
    self.m_active_popup = self.m_spawning_ground:spawn( chunks.shield_info.spawn() )
    self.m_active_popup.lua:setSpawnedFromMainMenu()
end

function LuaMainMenu:isMainMenu()
    return true
end

function LuaMainMenu:onBack()
    if self.m_active_popup and self.m_active_popup.lua then
        if self.m_active_popup.lua.dismiss then
            self.m_active_popup.lua:dismiss()
            self.m_active_popup = nil
            return;
        end
    end
    
    local ui = PlatformUIService.get()
    if ui ~= nil then

        local quit_callback = function(handle, code, btnidx) 
            if(btnidx:getInt() == 1)then
                GameService.get():quitGame() 
            end
        end

        local details = chunks.social.getSocialDetails()
        local title = lstring("quit_dialog_title_android")
        local body = lstring("quit_dialog_message_android")
        local okay = lstring("quit_dialog_quit_android")
        local not_okay = lstring("quit_dialog_cancel_android")
        local buttons = {not_okay, okay}

        self.m_async_handle = ui:showPrompt(title, body, buttons, quit_callback)
    end
end

function LuaMainMenu:getSpawningGround()
    return self.m_spawning_ground
end



