------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- LuaMainMenu        --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local constants = engine.platform.constants
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local textures  = chunks.textures
local fade      = chunks.fade
local events    = chunks.events
local utility   = chunks.utility
local swrve     = chunks.swrve
local sound  = chunks.sound

-- LuaMainMenu --
LuaMainMenu = chunks.class.inherit( chunks.window.LuaWindow )
{
    m_spawning_ground = nil,
    m_iap_handle = nil,
    m_active_popup = nil
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaMainMenu )

-- The created method.
function LuaMainMenu:created( owner )
    engine:doStartupTasks()
    --this call is interspersed throughout the code at critical locations where it is possible that audio states will get screwed up by ad interference
    Audio.get():restoreState()
    
    local game     = owner:getGame()
    local x_offset = 10.0
    local y_offset = 10.0
    local button_y = 90.0
    
    -- Background.
    do
        textures.setTexture( owner, assets.textures[ "main_" .. engine.theme ] )
    end

    if not GameService.get():zeb() then
        chunks.incent_ads.preloadAd("postroll")
        chunks.incent_ads.preloadAd("preroll")
    end
    
    --Blank Window from which to spawn the store, for input issues
    local spawning_ground = owner:spawn( Window() )
    spawning_ground.size = pool.get_v2f(0, 0)
    self.m_spawning_ground = spawning_ground

        -- Privacy Policy.
    do
        --shawn.harris - remove PP button for languages other then english so we can do a quick submission
        --and not have to worry about fixing button spacing
        --if engine.default_language_enabled then
        do
            local privacy_button
            local on_release = function( inner )
                events.show_privacy_policy()
            end

            if engine.default_language_enabled then
                privacy_button  = self:addButton( on_release )
                textures.setTexture( privacy_button, assets.textures.privacy_button )

                local text = privacy_button:spawn(Window())
                textures.setTexture(text, assets.textures.privacy_text)
                text.receive_input = false
            else
                local text = lstring( "privacy_policy" ) 
                local format
                if(string.sub(engine.locale_language,1,string.len("zh")) == "zh") then
                    format = {"ninja-text", 24.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 100.0, TextAlign.Center, -1.0, 6.0}
                else
                    format = {"ninja-text", 12.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 100.0, TextAlign.Center, -1.0, 6.0}
                end
                
                
                privacy_button = utility.createButtonWithTextAndTexture(self.owner, text, format, assets.textures.options_screen_blank_button, nil, 100 )
                privacy_button.on_release = function() on_release(); sound.play( assets.sounds.click ) end
            end

            privacy_button.anchor = make_anchor(Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f(x_offset + 4.0, button_y, 1.0))
            button_y = button_y + 44.0
        end
    end
    -- Options.
    do
        local on_release = function( inner )
            local callback = function( o )
                game:getWindow():spawn( chunks.options.spawn() )
                owner:destroy()
            end

            fade.fade_out( owner, callback )

            events.on_main_menu_disappeared()
        end

        local actor = nil
        local button_offset = 0
        if engine.default_language_enabled then
            actor  = self:addButton( on_release )
            textures.setTexture( actor, assets.textures.options_button )
        else
            local options_text = lstring( "options" )

            local text_height = 24.0
            local text_y_offset = 6.0

            if engine.locale_language == "ja" or engine.locale_language == "ko" or string.sub(engine.locale_language,1,string.len("zh")) == "zh" then
                text_height = 28.0
                text_y_offset = 4.0
            end

            local format = {"ninja-text", text_height, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 105.0, TextAlign.Center, -1.0, text_y_offset}
            actor = utility.createButtonWithTextAndTexture(self.owner, options_text, format, assets.textures.options_button_locale )
            actor.on_release = function() on_release(); sound.play( assets.sounds.click ) end

            button_y = button_y - button_offset
        end

        actor.anchor = make_anchor(Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f(x_offset + 4.0, button_y, 1.0))
        button_y = button_y + 44.0

        button_y = button_y + button_offset
    end

    -- Deluxe and Free Label
    local free_actor = nil;
    local deluxe_actor = owner:spawn( Window() )
    textures.setTexture(deluxe_actor, assets.textures.deluxe_label)


    if NJ_DELUXE_FREE then  --NOTE: This is defined by the project's preprocessor macros
        if constants.WIDESCREEN then
            deluxe_actor.location = pool.get_v3f( -80.0, 214.0, 1.0 )
        else
            deluxe_actor.location = pool.get_v3f( -80.0, 174.0, 1.0 )
        end
       if engine.platform.device ~= "android" then
           if engine.default_language_enabled then
                free_actor = owner:spawn( Window() )
                textures.setTexture(free_actor, assets.textures.free_label)
                if constants.WIDESCREEN then
                    free_actor.location = pool.get_v3f( 15.0, 218.0, 1.0 )
                else
                    free_actor.location = pool.get_v3f( 15.0, 178.0, 1.0 )
                end
           else
                local free_text = lstring( "free" )
                local format
                if constants.WIDESCREEN then
                    format = {"ninja-text", 38.0, pool.get_colorf(0.0, 0.0, 0.0, 1.0), 105.0, TextAlign.Center, 31.0, 210.0}
                else
                    format = {"ninja-text", 38.0, pool.get_colorf(0.0, 0.0, 0.0, 1.0), 105.0, TextAlign.Center, 31.0, 180.0}
                end
                local fx = {drop_shadow = true, shadow_color = colorf( 1, 1, 1, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 0}
                utility.createText( owner, free_text, format, fx)
           end
       end
    else
        if constants.WIDESCREEN then
            deluxe_actor.location = pool.get_v3f( 10.0, 212.0, 1.0 )
        else
            deluxe_actor.location = pool.get_v3f( 10.0, 172.0, 1.0 )
        end
    end

    -- Ad Buyout Button, only show if this is NJDX free and ads are not disabled
    if NJ_DELUXE_FREE and not chunks.iap.adBuyoutPurchased() and not GameService.get():zeb() then
        local on_release = function( inner )
            -- Bail if there is no iAP support
            if not chunks.iap.purchasingAvailable() then
                chunks.iap.displayPurchasingUnavailable()
                return
            end

            local products_consumed_callback = function()
                local callback = function( o )
                    game:getWindow():spawn( chunks.main.spawn() )
                    owner:destroy()
                end

                fade.fade_out( owner, callback )
            end
            self.m_iap_handle = chunks.iap.purchaseProduct("ad_buyout", function() end, products_consumed_callback)
        end

        local actor = nil
        if engine.default_language_enabled then
            actor  = self:addButton( on_release )
            textures.setTexture(actor, assets.textures.adbuyout_button)
        else
            --local remove_ads = lstring( "remove_ads" )
            --local format = {"ninja-text", 40.0, pool.get_colorf(0.0, 0.588, 1.0, 1.0), 105.0, TextAlign.Center, 0.0, 10.0}
            --actor = utility.createButtonWithTextAndTexture(self.owner, remove_ads, format, assets.textures.options_button_locale )


            local remove_ads = lstring( "remove_ads" )
            local format = {"ninja-text", 38.0, pool.get_colorf(0.0, 0.588, 1.0, 1.0), 105.0, TextAlign.Center, -1.0, 4.0}

            actor = utility.createButtonWithTextAndTexture(self.owner, remove_ads, format, assets.textures.options_button_locale )

            actor.on_release = function() on_release(); sound.play( assets.sounds.click ) end
        end
        actor.anchor = make_anchor(Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f(x_offset + 4.0, button_y, 1.0))
        button_y = button_y + 44.0

    end

    -- Shields
    if not GameService.get():zeb() then
        do
            local on_release = function( inner )
                if(UserData.get():getFreeShieldsIssued()) then
                    self:openItemStore()
                else
                    -- Bail if there is no iAP support
                    if not chunks.iap.purchasingAvailable() then
                        chunks.iap.displayPurchasingUnavailable()
                        return
                    end
                    
                    self:openShieldInfo()
                end
            end
    
            local button = nil
    
            if engine.default_language_enabled then
                button = self:addButton( on_release )
                textures.setTexture( button, assets.textures.item_store )
            else
                local shields_text = lstring( "shields" )
                local format = {"ninja-text", 38.0, pool.get_colorf(0.86, 0.98, 1.0, 1.0), 105.0, TextAlign.Center, 0.0, 4.0}
                local fx = {drop_shadow = true, shadow_color = colorf( 0.0, 0.27, 0.48, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 100000}
                button = utility.createButtonWithTextAndTexture(self.owner, shields_text, format, assets.textures.item_store_locale, fx )
                button.on_release = function() on_release(); sound.play( assets.sounds.click ) end
            end
    
            button.anchor = make_anchor(Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f(x_offset, button_y, 1.0))
    
        end
    end

    -- Generates play button bob actions.
    local bob = function( actor, index )
        local s     = SequenceAction()
        local l     = LoopAction()
        local delay = 0.35
        local pause = 2.0

        -- Initial bob sleep.
        if index > 0 then
            s:addAction( SleepAction( delay * index ) )
        end

        l:addAction( ScaleAction( delay, InterpAction.Smooth,
                                  pool.get_v3f( 0.8, 0.8, 1.0 ) ) )
        l:addAction( ScaleAction( delay, InterpAction.Smooth,
                                  pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
        l:addAction( SleepAction( pause ) )
        s:addAction( l )

        actor:addAction( s )
    end

    -- Generates a function to play the specified theme.
    local play_game = function( theme )
        return function( outer )
            local userdata = UserData.get()
            local callback = function( inner )
                -- If this is the first time they've
                -- played, show the tutorial menu.
                if userdata.first_time_played then
                    local tutorial = game:getWindow():spawn( chunks.tutorial.spawn() )

                    tutorial.lua:play( theme )
                    owner:destroy()
                    userdata.first_time_played = false
                    userdata:flush()
                else
                    game.lua:onPlay( theme )
                end
            end

            -- Change fade accordingly.
            if userdata.first_time_played then
                fade.fade_out( owner, callback )
            else
                fade.fade_out_loading( owner, callback )
            end

            events.on_main_menu_disappeared()
        end
    end
    
    local extra_y_offset = 35

    -- Classic play.
    
    -- NOTE: in phase 3 we want the classic option available for all android builds.
    -- For now classic will only be enabled by default in zeb builds
    
    -- if chunks.iap:adBuyoutPurchased() or engine.platform.device == "android" then
    
    if chunks.iap:adBuyoutPurchased() or engine.platform.device == "android" then
        local actor = nil

        if engine.default_language_enabled then
            actor  = self:addButton( play_game( "classic" ) )
            textures.setTexture( actor, assets.textures.classic_button )
        else
            local classic_text = lstring( "classic" )
            local format = {"ninja-text", 38.0, pool.get_colorf(0.99, 0.65, 0.0, 1.0), 105.0, TextAlign.Center, -1.0, 2.0}
            local fx = {drop_shadow = true, shadow_color = colorf( 0.2, 0.2, 0.2, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 100000}
            actor = utility.createButtonWithTextAndTexture(self.owner, classic_text, format, assets.textures.classic_button_locale, fx )
            actor.on_release = function() play_game( "classic" )(); sound.play( assets.sounds.click ) end
        end

        actor.anchor = make_anchor( Anchor.Anchor_BR, Anchor.Anchor_BR,
                               pool.get_v3f( -1.0 * x_offset, y_offset + extra_y_offset, 1.0 ) )
        -- Actions.
        bob( actor, 3 )
    end
    
    -- Debug sound fixing button
    --[[
    do
        local actor = nil

        actor  = self:addButton( function() Audio.get():restoreState() end )
        textures.setTexture( actor, assets.textures.jungle_button )
        

        actor.anchor = make_anchor( Anchor.Anchor_BR, Anchor.Anchor_BR,
                               pool.get_v3f( -1.0 * x_offset,
                                             y_offset + assets.textures.pirates_button.size.y * 5 + extra_y_offset,
                                             1.0 ) )

    end
    ]]
    
    -- Jungle play.
    do
        local actor = nil

        if engine.default_language_enabled then
            actor  = self:addButton( play_game( "jungle" ) )
            textures.setTexture( actor, assets.textures.jungle_button )
        else
            local jungle_text = lstring( "jungle" )
            local format = {"ninja-text", 38.0, pool.get_colorf(0.11, 1.0, 0.0, 1.0), 105.0, TextAlign.Center, 0.0, 2.0}
            local fx = {drop_shadow = true, shadow_color = colorf( 0.2, 0.2, 0.2, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 100000}
            actor = utility.createButtonWithTextAndTexture(self.owner, jungle_text, format, assets.textures.jungle_button_locale, fx )
            actor.on_release = function() play_game( "jungle" )(); sound.play( assets.sounds.click ) end
        end

        actor.anchor = make_anchor( Anchor.Anchor_BR, Anchor.Anchor_BR,
                               pool.get_v3f( -1.0 * x_offset,
                                             y_offset + assets.textures.pirates_button.size.y * 1.25 + extra_y_offset,
                                             1.0 ) )
        -- Actions.
        bob( actor, 2 )
    end

    -- Pirates play.
    do
        local actor  = nil

        if engine.default_language_enabled then
            actor = self:addButton( play_game( "pirates" ) )
            textures.setTexture( actor, assets.textures.pirates_button )
        else
            local pirates_text = lstring( "pirates" )
            local format = {"ninja-text", 38.0, pool.get_colorf(0.0, 0.588, 1.0, 1.0), 105.0, TextAlign.Center, 0.0, 2.0}
            local fx = {drop_shadow = true, shadow_color = colorf( 0.2, 0.2, 0.2, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 100000}
            actor = utility.createButtonWithTextAndTexture(self.owner, pirates_text, format, assets.textures.pirates_button_locale, fx )
            actor.on_release = function() play_game( "pirates" )(); sound.play( assets.sounds.click ) end
        end

        actor.anchor = make_anchor( Anchor.Anchor_BR, Anchor.Anchor_BR,
                               pool.get_v3f( -1.0 * x_offset,
                                             y_offset + assets.textures.jungle_button.size.y * 2.5 + extra_y_offset,
                                             1.0 ) )
        -- Actions.
        bob( actor, 1 )
    end

    -- Castle play.
    do
        local actor  = nil

        if engine.default_language_enabled then
            actor = self:addButton( play_game( "castle" ) )
            textures.setTexture( actor, assets.textures.castle_button )
        else
            local castle_text = lstring( "castle" )
            local format = {"ninja-text", 38.0, pool.get_colorf(0.705, 0.282, 1.0, 1.0), 105.0, TextAlign.Center, 0.0, 2.0}
            local fx = {drop_shadow = true, shadow_color = colorf( 0.2, 0.2, 0.2, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 100000}
            actor = utility.createButtonWithTextAndTexture(self.owner, castle_text, format, assets.textures.castle_button_locale, fx )
            actor.on_release = function() play_game( "castle" )(); sound.play( assets.sounds.click ) end
        end

        actor.anchor = make_anchor( Anchor.Anchor_BR, Anchor.Anchor_BR,
                               pool.get_v3f( -1.0 * x_offset,
                                             y_offset + assets.textures.castle_button.size.y * 3.75 + extra_y_offset,
                                             1.0 ) )
        -- Actions.
        bob( actor, 0 )
    end

    -- Leaderboards.
    local gc = GameCenterService.get()
    if gc then
        local on_release = function( inner )
            events.on_leaderboard_ui("")
            sound.play( assets.sounds.click )
        end
        local actor  = self:addButton( on_release )
        actor.anchor = make_anchor( Anchor.Anchor_TR, Anchor.Anchor_TR, pool.get_v3f( -x_offset, -y_offset, 1.0 ) )

        local info     = DeviceInfo.get()
        local version = info:query( "platform.version.major" )
        if(version < 7)then
            textures.setTexture( actor, assets.textures.leaderboards_button )
        else
            print("Texture is ", assets.textures.leaderboards_button_new.path)
            textures.setTexture( actor, assets.textures.leaderboards_button_new )
        end
    end

    -- Fade in.
    do
        fade.fade_in( owner )
    end

    events.on_main_menu_appeared()
end

function LuaMainMenu:openItemStore()
    if chunks.iap.purchasingAvailable() then   
        self.m_active_popup = self.m_spawning_ground:spawn( chunks.shield_store.spawn() )
    else
        chunks.iap.displayPurchasingUnavailable()
    end
end

function LuaMainMenu:openShieldInfo()
    self.m_active_popup = self.m_spawning_ground:spawn( chunks.shield_info.spawn() )
    self.m_active_popup.lua:setSpawnedFromMainMenu()
end

function LuaMainMenu:isMainMenu()
    return true
end

function LuaMainMenu:getSpawningGround()
    return self.m_spawning_ground
end

function LuaMainMenu:onBack()
    
    if self.m_active_popup and self.m_active_popup.lua then
        if self.m_active_popup.lua.dismiss then
            self.m_active_popup.lua:dismiss()
            self.m_active_popup = nil
            return;
        end
    end

    local ui = PlatformUIService.get()
    if ui ~= nil then

        local quit_callback = function(handle, code, btnidx) 
            if(btnidx:getInt() == 1)then
                GameService.get():quitGame() 
            end
        end

        local details = chunks.social.getSocialDetails()
        local title = lstring("quit_dialog_title_android")
        local body = lstring("quit_dialog_message_android")
        local okay = lstring("quit_dialog_quit_android")
        local not_okay = lstring("quit_dialog_cancel_android")
        local buttons = {not_okay, okay}

        self.m_async_handle = ui:showPrompt(title, body, buttons, quit_callback)
    end
end

function destroy()
    if self.m_iap_handle then
        self.m_iap_handle:cancel()
    end
end


