------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Monkey             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local effects   = chunks.effects
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound
local constants = engine.platform.constants


-- LuaMonkey --
LuaMonkey = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaMonkey )

-- The created method.
function LuaMonkey:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "monkey"
	owner.location = pool.get_v3f( -110.0, 0.0, 0.0 )
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.monkey )

		self.visuals.monkey = actor
	end
	
	self:setVisual( "monkey" )
	
	-- We're not killable.
	self.killable = false
end

-- The collided event.
function LuaMonkey:onCollided( other )
	local game  = self.owner:getGame()
	local ninja = game.lua.ninja
	
	-- Spawn particles.
	do
		particles.spawn_normal( self.owner:getGame().lua.root, 
								assets.textures.particles_leaf, 
								self.visual:getWorldLocation() )
	end
	
	-- Spawn slice.
	do
		effects.spawn_flash( self.owner:getGame().lua.root, 
							 assets.textures.effects_slice, 
							 self.visual:getWorldLocation() )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_enemy )
end

-- Setup easy thrower.
function LuaMonkey:easy()
	local actor = self.owner
	
	-- Play sound.
	sound.play_random( assets.sounds.monkey_enter )
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local projectile = actor:spawn( chunks.beehive.spawn() )
			
			-- Actions
			do
				local s2      = SequenceAction()
				local split   = SplitAction()
				local s3      = SequenceAction()
				local delay   = utility.frandom( 1.0, 1.2 )
				local target  = utility.random_normal( 10.0, 20.0 ) * utility.frandom( 160.0, 170.0 )
				local explode = function( inner )
					inner.lua:explode()
				end
				
				
				
				s2:addAction( MoveOffsetAction( delay, InterpAction.Lerp, 
				                                pool.get_v3f( target.x, target.y, 0.0 ) ) )
				s2:addAction( LuaAction( 0.0, explode ) )
				
				projectile:addAction( s2 )
			end
			
			-- Play sound.
			sound.play_random( assets.sounds.throw )
		end
		
		s:addAction( SleepAction( 0.35 ) )
		if constants.WIDESCREEN then
			s:addAction( SleepAction( 0.35 ) )
		end
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end
end


