--| module |--
module(...,package.seeall)
-- includes --
local engine    = engine.instance
local swrve     = engine.chunks.swrve

--This file handles the NEW analytics events, see spec at https://sites.google.com/a/backflipstudios.com/ninjump/incentivized-ads-update/analytics-events?pli=1
--Not to be confused with the old analytics reporting, which is handled through events.lua

function reportEndRun(distance, used_start_boost, start_boost_was_offered, used_revive, revive_was_offered)

    local cohort = engine.chunks.swrve.getResourceAttributeString("cohort_info", "cohort_name")
    local test = engine.chunks.swrve.getResourceAttributeString("cohort_info", "test_name")
    
    local level = (engine.theme):upper()
    local start_boost
    if(used_start_boost)then
        start_boost = "START_BOOST"
    elseif(not start_boost_was_offered)then
        start_boost = "NOT_AVAILABLE"
    else
        start_boost = "REFUSED"
    end
    
    local end_boost
    if(used_revive)then
        end_boost = "FALL_RECOVER"
    elseif(not revive_was_offered)then
        end_boost = "NOT_AVAILABLE"
    else
        end_boost = "REFUSED"
    end
    
    local starting_shields = UserData.get():getShieldCapacity() + 1
    local shields_remaining = UserData.get():getShields()
    local shields_used = starting_shields - shields_remaining
    
    local parameters = {    ["COHORT_NAME"] = cohort,
                            ["TEST_NAME"] = test,
                            ["LEVEL"] = level,
                            ["PRE_INCENT_VIDEO"] = start_boost,
                            ["POST_INCENT_VIDEO"] = end_boost,
                            ["DISTANCE_K"] = bucketedString(distance, 250),
                            ["SHIELDS_OWNED"] = tostring(starting_shields),
                            ["SHIELDS_USED"] = tostring(shields_used),
                            ["SHIELDS_REMAINING"] = tostring(shields_remaining) }
                            
    sendAnalyticsEvent("FINISH_RUN", parameters)
end

function reportShieldPurchase(previous_count, purchased_count)

    local cohort = engine.chunks.swrve.getResourceAttributeString("cohort_info", "cohort_name")
    local test = engine.chunks.swrve.getResourceAttributeString("cohort_info", "test_name")
    local level = (engine.theme):upper()
    
    if(UserData.get():getHighScore( engine.theme ) == 0) then
        level = "NONE"
    end
    
    local parameters = {    ["COHORT_NAME"] = cohort,
                            ["TEST_NAME"] = test,
                            ["LEVEL"] = level,
                            ["SHIELDS_OWNED"] = tostring(previous_count),
                            ["SHIELDS_BOUGHT"] = tostring(purchased_count) }
                            
    sendAnalyticsEvent("BUY_SHIELDS", parameters)
end

function reportVideoWatched(preroll)

    local cohort = engine.chunks.swrve.getResourceAttributeString("cohort_info", "cohort_name")
    local test = engine.chunks.swrve.getResourceAttributeString("cohort_info", "test_name")
    local level = (engine.theme):upper()
    local boost
    if (preroll) then
        boost = "START_BOOST"
    else
        boost = "FALL_RECOVER"
    end
    
    local parameters = {    ["COHORT_NAME"] = cohort,
                            ["TEST_NAME"] = test,
                            ["LEVEL"] = level,
                            ["INCENT_VIDEO"] = boost }
                            
    sendAnalyticsEvent("WATCH_INCENT_VIDEO", parameters)
end

--[[
function reportSessionStarted()
    local cohort = engine.chunks.swrve.getResourceAttributeString("cohort_info", "cohort_name")
    local test = engine.chunks.swrve.getResourceAttributeString("cohort_info", "test_name")
    
    local info     = DeviceInfo.get()
	local connection_code = info:query( "net.connection_type" )
	
	
    local connection
    if(connection_code == "lan")then
        connection = "WIFI"
    elseif(connection_code == "wwan")then
        connection = "CELL"
    elseif(connection_code == "none")then
        connection = "NONE"
    end
    print("Detected ", connection, " connection")
    
    
    local parameters = {    ["COHORT_NAME"] = cohort,
                            ["TEST_NAME"] = test,
                            ["CONNECTION_TYPE"] = connection }
                            
    sendAnalyticsEvent("SESSION_START", parameters)
end
]]

function reportVideoStarted(preroll)

    local cohort = engine.chunks.swrve.getResourceAttributeString("cohort_info", "cohort_name")
    local test = engine.chunks.swrve.getResourceAttributeString("cohort_info", "test_name")
    local level = (engine.theme):upper()
    local boost
    if (preroll) then
        boost = "START_BOOST"
    else
        boost = "FALL_RECOVER"
    end
    
    local parameters = {    ["COHORT_NAME"] = cohort,
                            ["TEST_NAME"] = test,
                            ["LEVEL"] = level,
                            ["INCENT_VIDEO"] = boost }
                            
    sendAnalyticsEvent("TRIGGER_INCENT_VIDEO", parameters)
end

function bucketedString(value, bucket_size)

    local bucketed = bucket_size * roundToNearest(value / bucket_size)
    return tostring(bucketed)
end

function roundToNearest(val)
    local floor = math.floor(val)
    if ((val - floor) < 0.5) then
        return floor
    else
        return floor + 1
    end
end

function sendAnalyticsEvent(event_name, parameters)

    print("sending Event, NAME=", event_name)
    
    local event = AnalyticsEvent(event_name)
    local parameters = parameters or {}
    for param, value in pairs(parameters) do
        print("   ", param, " - ", value)
        event:add(param, value)
    end
    event:send()
    
end