------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Particles          --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine  = engine.instance
local chunks  = engine.chunks
local utility = chunks.utility
local assets  = chunks.assets
local pool    = chunks.pool

-- Spawn a normal generic particle system.
function spawn_normal( owner, texture, location )
	local explosion = owner:spawn( ParticleSystem() )
	
	explosion.texture  = texture.path
	explosion.location = location
	
	-- Set explosion parameters.
	explosion.particle_size             = texture.size
	explosion.explosion                 = true
	explosion.lifetime                  = utility.frandom( 0.1, 0.15 )
	explosion.lifetime_range            = pool.get_v2f( 0.4, 0.6 )
	explosion.particles_per_spawn_range = pool.get_v2i( 4, 6 )
	explosion.spawn_frequency_range     = pool.get_v2f( 0.01, 0.05 )
	explosion.speed_range               = pool.get_v2f( 200.0, 250.0 )
	explosion.initial_rotation_range    = pool.get_v2f( 0.0, 360.0 )
	explosion.rotation_speed_range      = pool.get_v2f( 120.0, 150.0 )
	
	-- scale transitions
	explosion:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
	explosion:addScaleRange( pool.get_v2f( 0.8, 1.0 ) )
	explosion:addScaleRange( pool.get_v2f( 1.0, 1.2 ) )
	explosion:addScaleRange( pool.get_v2f( 0.8, 1.0 ) )
	explosion:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
	
	return explosion
end

-- Spawn a big generic particle system.
function spawn_big( owner, texture, location )
	local explosion = owner:spawn( ParticleSystem() )
	
	explosion.texture  = texture.path
	explosion.location = location
	
	-- Set explosion parameters.
	explosion.particle_size             = texture.size
	explosion.explosion                 = true
	explosion.lifetime                  = utility.frandom( 0.15, 0.2 )
	explosion.lifetime_range            = pool.get_v2f( 0.7, 0.8 )
	explosion.particles_per_spawn_range = pool.get_v2i( 15, 20 )
	explosion.spawn_frequency_range     = pool.get_v2f( 0.03, 0.04 )
	explosion.speed_range               = pool.get_v2f( 250.0, 300.0 )
	explosion.initial_rotation_range    = pool.get_v2f( 0.0, 360.0 )
	explosion.rotation_speed_range      = pool.get_v2f( 120.0, 150.0 )
    explosion.acceleration_range_x      = pool.get_v2f( -30.0, 0.0 )
    explosion.acceleration_range_y      = pool.get_v2f( 0.0, -10.0 )
	
	-- scale transitions
	explosion:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
	explosion:addScaleRange( pool.get_v2f( 0.8, 1.0 ) )
	explosion:addScaleRange( pool.get_v2f( 1.0, 1.2 ) )
	explosion:addScaleRange( pool.get_v2f( 0.8, 1.0 ) )
	explosion:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
	
	return explosion
end


-- Spawn an uber big generic particle system.
function spawn_uber( owner, texture, location )
	local explosion = owner:spawn( ParticleSystem() )
	
	explosion.texture  = texture.path
	explosion.location = location
	
	-- Set explosion parameters.
	explosion.particle_size             = texture.size
	explosion.explosion                 = true
	explosion.lifetime                  = utility.frandom( 0.25, 0.3 )
	explosion.lifetime_range            = pool.get_v2f( 0.8, 0.9 )
	explosion.particles_per_spawn_range = pool.get_v2i( 25, 30 )
	explosion.spawn_frequency_range     = pool.get_v2f( 0.03, 0.04 )
	explosion.speed_range               = pool.get_v2f( 250.0, 300.0 )
	explosion.initial_rotation_range    = pool.get_v2f( 0.0, 360.0 )
	explosion.rotation_speed_range      = pool.get_v2f( 120.0, 150.0 )
    explosion.acceleration_range_x      = pool.get_v2f( -30.0, 0.0 )
    explosion.acceleration_range_y      = pool.get_v2f( 0.0, -10.0 )
	
	-- scale transitions
	explosion:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
	explosion:addScaleRange( pool.get_v2f( 0.8, 1.0 ) )
	explosion:addScaleRange( pool.get_v2f( 1.0, 1.2 ) )
	explosion:addScaleRange( pool.get_v2f( 0.8, 1.0 ) )
	explosion:addScaleRange( pool.get_v2f( 0.0, 0.0 ) )
	
	return explosion
end






		