------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Pause Menu         --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local textures  = chunks.textures
local constants = engine.platform.constants
local utility  = chunks.utility

-- LuaPauseMenu --
LuaPauseMenu = chunks.class.inherit( chunks.window.LuaWindow )()

-- The spawner.
spawn = chunks.class.spawner( Window, LuaPauseMenu )

-- The created event.
function LuaPauseMenu:created( owner )
	local game = owner:getGame()
	local window = owner
	local menu = nil
	
	-- Initialize owner.
	owner.location = pool.get_v3f( 0.0, 0.0, 10.0 )
	owner.size     = constants.scene.world_size
	owner.hidden   = true

	-- Menu: we anchor it to an inner shell to 
	--       make the animations easier.
	do
		local shell = owner:spawn( Window() )
		
		shell.hidden = true
		shell.anchor = make_anchor( Anchor.Anchor_B, Anchor.Anchor_B, pool.get_v3f( 0.0, 0.0, 0.0 ) )
		menu         = shell:spawn( chunks.window.spawn( Window() ) )
		
		textures.setTexture( menu, assets.textures.pause_menu )
		
		shell.size = menu.size
		
		if engine.default_language_enabled then
			local paused_overlay = utility.createWindowWithTexture( menu, assets.textures.paused_menu_text_locale )
			paused_overlay.location = pool.get_v3f( 0.0, 110.0, 11.0 )
          
		    local paused_or_overlay = utility.createWindowWithTexture( menu, assets.textures.paused_menu_or_locale )
			paused_or_overlay.location = pool.get_v3f( 0.0, -15.0, 11.0 )
		else
			utility.createText( menu, lstring( "paused" ), 
				{"ninja-text-large", 80.0, pool.get_colorf(1, 1, 1, 1), 200.0, TextAlign.Center, 0, 120, 7} )
			
			utility.createText( menu, lstring( "or" ), 
				{"ninja-text", 40.0, pool.get_colorf(1, 0, 0, 1), 200.0, TextAlign.Center, 0, -5, 7} )
		end
	end

	-- Play button.
	do
		-- Transition out and unpause.
		local on_release = function( inner )
			self:transitionOut( function( o ) game.lua:onUnpause() end )
		end
		
		local button = menu.lua:addButton( on_release )
		
		button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, 40.0, 1.0 ) )
		textures.setTexture( button, assets.textures.continue_button )
	end
	
	-- Quit button.
	do
		-- Transition out and unpause.
		local on_release = function( inner )
			self:transitionOut( function( o ) game.lua:onQuit() end )
		end
		
		local button = nil
		
		if engine.default_language_enabled then
			button =  menu.lua:addButton( on_release )
			textures.setTexture( button, assets.textures.menu_button )
		else
			local text = lstring( "menu" )
			local format = {"ninja-text", 20.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 42.0, TextAlign.Center, -1.0, 3.0}
			button = utility.createButtonWithTextAndTexture(menu.lua.owner, text, format, assets.textures.menu_button_locale )
			button.on_release = on_release
		end
		
		button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -60.0, 1.0 ) )
		
	end
	
	-- The item store button.
    if not GameService.get():zeb() then
    	do
    
    		local on_release = function( inner )	
    		    if chunks.iap.purchasingAvailable() then  
    			    self:openItemStore((UserData.get():getShields() == 0))
    			else
    			    chunks.iap.displayPurchasingUnavailable()
    			end
    		end	
    		
    		local button = nil
    		if engine.default_language_enabled then
    			button = menu.lua:addButton( on_release )
    			textures.setTexture( button, assets.textures.item_store )
    		else
    			local shields_text = lstring( "shields" )
    			local format = {"ninja-text", 35.0, pool.get_colorf(0.86, 0.98, 1.0, 1.0), 105.0, TextAlign.Center, 0.0, 6.0}
    			local fx = {drop_shadow = true, shadow_color = colorf( 0.0, 0.27, 0.48, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 100000}
    			button = utility.createButtonWithTextAndTexture(menu.lua.owner, shields_text, format, assets.textures.item_store_locale, fx )
    			button.on_release = on_release
    		end
    		
    		button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -100.0, 1.0 ) )
    		
    	end
    end
	
	-- Transition in.
	self:transitionIn()
end

-- Transition in.
function LuaPauseMenu:transitionIn()
	local height = self.owner.size.y
	local s      = SequenceAction()	
	local s2     = SplitAction()
	
	s:addAction( MoveOffsetAction( 0.0, InterpAction.Smooth, pool.get_v3f( 0.0, -height, 0.0 ) ) )
	s:addAction( AlphaAction( 0.0, InterpAction.Smooth, 0.0 ) )
	s2:addAction( MoveOffsetAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.0,  height, 0.0 ) ) )
	s2:addAction( AlphaAction( 0.4, InterpAction.Smooth, 1.0 ) )
	s:addAction( s2 )
	
	self.owner:addAction( s )
end

-- Transition out.
function LuaPauseMenu:transitionOut( callback )
	local height = self.owner.size.y
	local s      = SequenceAction()	
	local s2     = SplitAction()
	
	s2:addAction( MoveOffsetAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.0, -height, 0.0 ) ) )
	s2:addAction( AlphaAction( 0.4, InterpAction.Smooth, 0.0 ) )
	s:addAction( s2 )
	s:addAction( LuaAction( 0.0, callback ) )
	s:addAction( DestroyAction( 0.0 ) )
	
	self.owner:addAction( s )
end

function LuaPauseMenu:openItemStore(ask_use)
    self:transitionOut( function( o ) self.owner:getOwner().lua:openItemStore( ask_use ) end )
end
