------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Pawn               --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks

-- LuaPawn --
LuaPawn = chunks.class.new
{
	-- visuals --
	visuals = nil,	-- the table of possible visuals
	visual  = nil 	-- the current visual
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaPawn )

-- The created method.
function LuaPawn:created( owner )
	-- Initialize the owner.
	owner.hidden = true
	
	-- Create visuals table.
	self.visuals = {}
end

-- Set the current visual.
function LuaPawn:setVisual( visual )
	-- First, hide all visuals.
	for k,v in pairs( self.visuals ) do
		v.hidden = true
	end
	
	-- Unhide the specified visual.
	self.visual        = self.visuals[ visual ]
	self.visual.hidden = false
	
	-- If animated texture reset.
	if self.visual and self.visual.texture then
        local animation = self.visual.texture.animation

        if animation then
            animation:reset()
        end
	end
	
	return self.visual
end

-- Returns true if the actors collide.
function LuaPawn:collides( other )
	if self.visual == nil or other.lua.visual == nil then
		return false
	end
	
	local p1 = CollidableActor.fast_cast( self.visual )
	local p2 = CollidableActor.fast_cast( other.lua.visual )
	
	if p1 == nil or p2 == nil then
		return false
	end
	
	return p1:collides( p2 )
end

-- The collided event.
function LuaPawn:onCollided( other )
	print( "missing collided method!" )
end
