------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Precache           --
------------------------

-- module --
module(...,package.seeall)

local sound     = require( "shared.sound" )
local resources = Resources.get()

--------------
-- textures --
--------------

-- Load textures.
local textures_load = function( assets )
    for k,v in pairs( assets.textures ) do
        Resources.get():load( v.path, TextureResource.class_info() )
    end
end

-- Unload textures.
local textures_unload = function( assets )
    for k,v in pairs( assets.textures ) do
        Resources.get():unload( v.path )
    end
end

------------
-- images --
------------

-- Load images.
local images_load = function( assets )
	local manager = ImageManager.get()
	
	for k,v in pairs( assets.textures ) do
		if v.collides == true then
			manager:loadImage( URI( v.path ) )
		end
	end
end
 
-- Unload images.
local images_unload = function( assets )
	local manager = ImageManager.get()
	
	for k,v in pairs( assets.textures ) do
		if v.collides == true then
			manager:destroyImage( URI( v.path ) )
		end
	end
end

------------
-- sounds --
------------

-- Load sounds.
local sounds_load = function( assets )
	for k,v in pairs( assets.sounds ) do
		if v.path == nil then
			for k2,v2 in ipairs( v ) do
                sound.precache( v2 )
			end	
		else
            sound.precache( v )
		end
	end
end

-- Unload sounds.
local sounds_unload = function( assets )
	for k,v in pairs( assets.sounds ) do
		if v.path == nil then
			for k2,v2 in ipairs( v ) do
                sound.unload( v2 )
			end	
		else
            sound.unload( v )
		end
	end	
end

---------
-- all --
---------

-- Loads textures, images, sounds.
function load( assets )
	textures_load( assets )
	images_load( assets )
	sounds_load( assets )
end	

-- Unloads textures, images, sounds.
function unload( assets )
	textures_unload( assets )
	images_unload( assets )
	sounds_unload( assets )
end
