------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Scenarios          --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local assets = chunks.assets
local pool   = chunks.pool

---------------
-- scenarios --
---------------

scenarios = {}

-- Empty.
function scenarios.empty( root )
	return { delay = 0.75, lifetime = 0.0, z = 0.0 }
end

-- Shield.
function scenarios.shieldpack( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.shieldpack.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Foliage.
function scenarios.foliage( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 0.5 )
	
	scenario:spawn( chunks.foliage.spawn() )
	
	return { delay = 0.5, lifetime = time }
end

-- Rope.
function scenarios.rope( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, -55.0 )
	
	scenario:spawn( chunks.rope.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Mast.
function scenarios.mast( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.mast.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Mast hard.
function scenarios.mast_hard( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.mast.spawn() )
	
	return { delay = 0.65, lifetime = time }
end

-- Crows nest.
function scenarios.crows_nest( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.crows_nest.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Crows nest hard.
function scenarios.crows_nest_hard( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.crows_nest.spawn() )
	
	return { delay = 0.65, lifetime = time }
end

-- Cannon easy.
function scenarios.cannon_easy( root )
	local time   = nil
	local cannon = nil
	
	-- Cannon.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time   = scenario.lua:scroll( -0.75, 1.0 )
		cannon = scenario:spawn( chunks.cannon.spawn() )
	end

	-- Cannonballs.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.lua:scroll( -0.75, 0.5 )
		cannon.lua:easy( scenario )
	end
	
	return { delay = 0.9, lifetime = time }
end

-- Cannon hard.
function scenarios.cannon_hard( root )
	local time   = nil
	local cannon = nil
	
	-- Cannon.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time   = scenario.lua:scroll( -0.75, 1.0 )
		cannon = scenario:spawn( chunks.cannon.spawn() )
	end

	-- Cannonballs.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.lua:scroll( -0.75, 0.5 )
		cannon.lua:hard( scenario )
	end
	
	return { delay = 0.8, lifetime = time }
end

-- Parrot easy.
function scenarios.parrot_easy( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local parrot   = scenario:spawn( chunks.parrot.spawn() )
	
	scenario.location = pool.get_v3f( 0.0, 0.0, 3.0 )
	parrot.lua:easy()
	
	return { delay = 1.8, lifetime = 4.5 }
end

-- Parrot hard.
function scenarios.parrot_hard( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local parrot   = scenario:spawn( chunks.parrot.spawn() )
	
	scenario.location = pool.get_v3f( 0.0, 0.0, 3.0 )
	parrot.lua:hard()
	
	return { delay = 1.7, lifetime = 4.5 }
end

-- Hooker easy.
function scenarios.hooker_easy( root )
	local time = nil
	
	-- Rope.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time = scenario.lua:scroll( -0.75, -55.0 )
		scenario:spawn( chunks.rope.spawn() )
	end
	
	-- Hooker.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local time     = scenario.lua:scroll( -0.75, 1.0 )
		local base     = scenario:spawn( ActorBase() )
		local hooker   = base:spawn( chunks.hooker.spawn() )
		
		base.location = pool.get_v3f( 0.0, -15.5, 0.0 )
		hooker.lua:easy()
	end
	
	return { delay = 0.7, lifetime = time }
end

-- Hooker hard.
function scenarios.hooker_hard( root )
	local time = nil
	
	-- Rope.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time = scenario.lua:scroll( -0.75, -55.0 )
		scenario:spawn( chunks.rope.spawn() )
	end
	
	-- Hooker.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local time     = scenario.lua:scroll( -0.75, 1.0 )
		local base     = scenario:spawn( ActorBase() )
		local hooker   = base:spawn( chunks.hooker.spawn() )
		
		base.location = pool.get_v3f( 0.0, -15.5, 0.0 )
		hooker.lua:hard()
	end
	
	return { delay = 0.6, lifetime = time }
end

-- Kraken easy.
function scenarios.kraken_easy( root )
	local kraken                   = nil
	local scenario_tentacle_bottom = nil
	local scenario_tentacle_top    = nil
	
	-- Head.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.location = pool.get_v3f( 0.0, 0.0, -55.0 )
		kraken            = scenario:spawn( chunks.kraken.spawn() ) 
	end
	
	-- Tentacle bottom.
	do
		scenario_tentacle_bottom          = root:spawn( chunks.scenario.spawn() )
		scenario_tentacle_bottom.location = pool.get_v3f( 0.0, 0.0, 17.0 )
		
	end
	
	-- Tentacle top.
	do
		scenario_tentacle_top          = root:spawn( chunks.scenario.spawn() )
		scenario_tentacle_top.location = pool.get_v3f( 0.0, 0.0, 18.0 )
	end
	
	kraken.lua:easy( scenario_tentacle_bottom, scenario_tentacle_top )
	
	return { delay = 3.8, lifetime = 4.5 }
end

-- Kraken hard.
function scenarios.kraken_hard( root )
	local kraken                   = nil
	local scenario_tentacle_bottom = nil
	local scenario_tentacle_top    = nil
	
	-- Head.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.location = pool.get_v3f( 0.0, 0.0, -55.0 )
		kraken            = scenario:spawn( chunks.kraken.spawn() ) 
	end
	
	-- Tentacle bottom.
	do
		scenario_tentacle_bottom          = root:spawn( chunks.scenario.spawn() )
		scenario_tentacle_bottom.location = pool.get_v3f( 0.0, 0.0, 17.0 )
		
	end
	
	-- Tentacle top.
	do
		scenario_tentacle_top          = root:spawn( chunks.scenario.spawn() )
		scenario_tentacle_top.location = pool.get_v3f( 0.0, 0.0, 18.0 )
	end
	
	kraken.lua:hard( scenario_tentacle_bottom, scenario_tentacle_top )
	
	return { delay = 2.0, lifetime = 2.5 }
end

-- Kraken hardest.
function scenarios.kraken_hardest( root )
	local kraken                   = nil
	local scenario_tentacle_bottom = nil
	local scenario_tentacle_top    = nil
	
	-- Head.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.location = pool.get_v3f( 0.0, 0.0, -55.0 )
		kraken            = scenario:spawn( chunks.kraken.spawn() ) 
	end
	
	-- Tentacle bottom.
	do
		scenario_tentacle_bottom          = root:spawn( chunks.scenario.spawn() )
		scenario_tentacle_bottom.location = pool.get_v3f( 0.0, 0.0, 17.0 )
		
	end
	
	-- Tentacle top.
	do
		scenario_tentacle_top          = root:spawn( chunks.scenario.spawn() )
		scenario_tentacle_top.location = pool.get_v3f( 0.0, 0.0, 18.0 )
	end
	
	-- Setup the kraken.
	kraken.lua:hard( scenario_tentacle_bottom, scenario_tentacle_top )
	
	-- Ledges.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local time     = scenario.lua:scroll( -2.2, 1.0 )
		
		-- First.
		do
			local base       = scenario:spawn( ActorBase() )
			local crows_nest = base:spawn( chunks.crows_nest.spawn() )
		
			base.location = pool.get_v3f( 0.0, 80.0, 0.0 )
		end
		
		-- Second.
		do
			local base       = scenario:spawn( ActorBase() )
			local crows_nest = base:spawn( chunks.crows_nest.spawn() )
		
			base.location = pool.get_v3f( 0.0, 280.0, 0.0 )
			base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		end
	end
	
	return { delay = 2.0, lifetime = 3.0 }
end

-- Swinger easy.
function scenarios.swinger_easy( root )
	local swinger = nil
	
	-- Swinger.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.location = pool.get_v3f( 0.0, 0.0, 3.0 )
		swinger           = scenario:spawn( chunks.swinger.spawn() )
	end
	
	-- Rope.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.location = pool.get_v3f( 0.0, 0.0, 2.0 )
		
		swinger.lua:easy( scenario )
	end
	
	return { delay = 1.5, lifetime = 2.4 }
end

-- Swinger hard.
function scenarios.swinger_hard( root )
	local swinger = nil
	
	-- Swinger.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.location = pool.get_v3f( 0.0, 0.0, 3.0 )
		swinger           = scenario:spawn( chunks.swinger.spawn() )
	end
	
	-- Rope.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.location = pool.get_v3f( 0.0, 0.0, 2.0 )
		
		swinger.lua:hard( scenario )
	end
	
	return { delay = 1.4, lifetime = 1.8 }
end

-- Gauntlet.
function scenarios.gauntlet( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -1.75, 1.0 )
	
	-- First.
	do
		scenario:spawn( chunks.mast.spawn() )
	end
	
	-- Second.
	do
		local base     = scenario:spawn( ActorBase() )
		local overhang = base:spawn( chunks.crows_nest.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 85 * 2, 0.0 )
	end
	
	-- Third.
	do
		local base     = scenario:spawn( ActorBase() )
		local overhang = base:spawn( chunks.crows_nest.spawn() )
		
		--base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 85 * 2 + 75.0, 0.0 )
	end
	
	-- Forth.
	do
		local base     = scenario:spawn( ActorBase() )
		local overhang = base:spawn( chunks.mast.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 85 * 4 + 75.0, 0.0 )
	end

	return { delay = 0.75 * 3, lifetime = time }
end

-- Mast close call.
function scenarios.mast_close_call( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.85, 1.0 )
	
	-- First.
	do
		scenario:spawn( chunks.mast.spawn() )
	end
	
	-- Second.
	do
		local base    = scenario:spawn( ActorBase() )
		local balcony = base:spawn( chunks.mast.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 128.0, 0.0 )
	end
	
	return { delay = 1.0, lifetime = time }
end

-- Crows nest close call.
function scenarios.crows_nest_close_call( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.8, 1.0 )
	
	-- First.
	do
		scenario:spawn( chunks.crows_nest.spawn() )
	end
	
	-- Second.
	do
		local base    = scenario:spawn( ActorBase() )
		local balcony = base:spawn( chunks.crows_nest.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 75.0, 0.0 )
	end
	
	return { delay = 1.0, lifetime = time }
end

-- New high score.
function scenarios.new_high_score( root )
	local scenario       = root:spawn( chunks.scenario.spawn() )
	local time           = scenario.lua:scroll( -0.75, -55.0 )
	local new_high_score = scenario:spawn( chunks.new_high_score.spawn() )
	
	new_high_score.lua:celebrate()
	
	return { delay = 1.5, lifetime = time, z = -55.0 }
end

-----------------
-- transitions --
-----------------

transitions =
{
	-------------
	-- TESTING --
	-------------
	
	--[[
	{
		height    = 100000.0,
		scenarios =
		{
			{ prob = 0.5, f = scenarios.empty },
			{ prob = 0.5, f = scenarios.shieldpack },
			--{ prob = 0.5, f = scenarios.rope },
			--{ prob = 0.5, f = scenarios.mast },
			--{ prob = 0.5, f = scenarios.crows_nest },
			--{ prob = 0.5, f = scenarios.mast_hard },
			--{ prob = 0.5, f = scenarios.crows_nest_hard },
			--{ prob = 0.5, f = scenarios.cannon_easy },
			--{ prob = 0.5, f = scenarios.cannon_hard },
			--{ prob = 0.5, f = scenarios.parrot_easy },
			--{ prob = 0.5, f = scenarios.parrot_hard },
			--{ prob = 0.5, f = scenarios.hooker_easy },
			--{ prob = 0.5, f = scenarios.hooker_hard },
			--{ prob = 0.5, f = scenarios.kraken_easy },
			--{ prob = 0.5, f = scenarios.kraken_hard },
			--{ prob = 0.5, f = scenarios.kraken_hardest },
			--{ prob = 0.5, f = scenarios.swinger_easy },
			--{ prob = 0.5, f = scenarios.swinger_hard },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.mast_close_call },
			--{ prob = 0.5, f = scenarios.crows_nest_close_call },
		}
	},
	--]]

	----------------
	-- PRODUCTION --
	----------------
	
	-- At the very beginning, spawn some empty and rope.
	{
		height    = 102.0,
		scenarios =
		{
			{ prob = 0.8, f = scenarios.empty },
			--{ prob = 0.5, f = scenarios.shieldpack },
			{ prob = 0.3, f = scenarios.rope },
			--{ prob = 0.5, f = scenarios.mast },
			--{ prob = 0.5, f = scenarios.crows_nest },
			--{ prob = 0.5, f = scenarios.mast_hard },
			--{ prob = 0.5, f = scenarios.crows_nest_hard },
			--{ prob = 0.5, f = scenarios.cannon_easy },
			--{ prob = 0.5, f = scenarios.cannon_hard },
			--{ prob = 0.5, f = scenarios.parrot_easy },
			--{ prob = 0.5, f = scenarios.parrot_hard },
			--{ prob = 0.5, f = scenarios.hooker_easy },
			--{ prob = 0.5, f = scenarios.hooker_hard },
			--{ prob = 0.5, f = scenarios.kraken_easy },
			--{ prob = 0.5, f = scenarios.kraken_hard },
			--{ prob = 0.4, f = scenarios.kraken_hardest },
			--{ prob = 0.5, f = scenarios.swinger_easy },
			--{ prob = 0.5, f = scenarios.swinger_hard },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.mast_close_call },
			--{ prob = 0.5, f = scenarios.crows_nest_close_call },
		}
	},
	
	-- Introduce easy cannon, parrot, hooker.  Also the shieldpack.
	{
		height    = 1122.0,
		scenarios =
		{
			{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			{ prob = 0.3, f = scenarios.rope },
			--{ prob = 0.5, f = scenarios.mast },
			--{ prob = 0.5, f = scenarios.crows_nest },
			--{ prob = 0.5, f = scenarios.mast_hard },
			--{ prob = 0.5, f = scenarios.crows_nest_hard },
			{ prob = 0.2, f = scenarios.cannon_easy },
			--{ prob = 0.5, f = scenarios.cannon_hard },
			{ prob = 0.2, f = scenarios.parrot_easy },
			--{ prob = 0.5, f = scenarios.parrot_hard },
			{ prob = 0.2, f = scenarios.hooker_easy },
			--{ prob = 0.5, f = scenarios.hooker_hard },
			--{ prob = 0.5, f = scenarios.kraken_easy },
			--{ prob = 0.5, f = scenarios.kraken_hard },
			--{ prob = 0.4, f = scenarios.kraken_hardest },
			--{ prob = 0.5, f = scenarios.swinger_easy },
			--{ prob = 0.5, f = scenarios.swinger_hard },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.mast_close_call },
			--{ prob = 0.5, f = scenarios.crows_nest_close_call },
		}
	},
	
	-- Introduce crows nest, rare mast, rare swinger.
	{
		height    = 2142.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.rope },
			{ prob = 0.2, f = scenarios.mast },
			{ prob = 0.4, f = scenarios.crows_nest },
			--{ prob = 0.5, f = scenarios.mast_hard },
			--{ prob = 0.5, f = scenarios.crows_nest_hard },
			{ prob = 0.3, f = scenarios.cannon_easy },
			--{ prob = 0.5, f = scenarios.cannon_hard },
			{ prob = 0.3, f = scenarios.parrot_easy },
			--{ prob = 0.5, f = scenarios.parrot_hard },
			{ prob = 0.3, f = scenarios.hooker_easy },
			--{ prob = 0.5, f = scenarios.hooker_hard },
			--{ prob = 0.5, f = scenarios.kraken_easy },
			--{ prob = 0.5, f = scenarios.kraken_hard },
			--{ prob = 0.4, f = scenarios.kraken_hardest },
			{ prob = 0.15, f = scenarios.swinger_easy },
			--{ prob = 0.5, f = scenarios.swinger_hard },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.mast_close_call },
			--{ prob = 0.5, f = scenarios.crows_nest_close_call },
		}
	},
	
	-- Increase masts, introduce kraken, increase swingers.
	{
		height    = 3162.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.rope },
			{ prob = 0.4, f = scenarios.mast },
			{ prob = 0.3, f = scenarios.crows_nest },
			--{ prob = 0.5, f = scenarios.mast_hard },
			--{ prob = 0.5, f = scenarios.crows_nest_hard },
			{ prob = 0.3, f = scenarios.cannon_easy },
			--{ prob = 0.5, f = scenarios.cannon_hard },
			{ prob = 0.3, f = scenarios.parrot_easy },
			--{ prob = 0.5, f = scenarios.parrot_hard },
			{ prob = 0.3, f = scenarios.hooker_easy },
			--{ prob = 0.5, f = scenarios.hooker_hard },
			{ prob = 0.1, f = scenarios.kraken_easy },
			--{ prob = 0.5, f = scenarios.kraken_hard },
			--{ prob = 0.4, f = scenarios.kraken_hardest },
			{ prob = 0.3, f = scenarios.swinger_easy },
			--{ prob = 0.5, f = scenarios.swinger_hard },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.mast_close_call },
			--{ prob = 0.5, f = scenarios.crows_nest_close_call },
		}
	},
	
	-- Introduce hard enemies, partially.  Increase kraken, cannon.
	{
		height    = 4182.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.rope },
			{ prob = 0.5, f = scenarios.mast },
			{ prob = 0.3, f = scenarios.crows_nest },
			--{ prob = 0.5, f = scenarios.mast_hard },
			--{ prob = 0.5, f = scenarios.crows_nest_hard },
			{ prob = 0.1, f = scenarios.cannon_easy },
			{ prob = 0.3, f = scenarios.cannon_hard },
			{ prob = 0.1, f = scenarios.parrot_easy },
			{ prob = 0.2, f = scenarios.parrot_hard },
			{ prob = 0.1, f = scenarios.hooker_easy },
			{ prob = 0.2, f = scenarios.hooker_hard },
			{ prob = 0.3, f = scenarios.kraken_easy },
			--{ prob = 0.5, f = scenarios.kraken_hard },
			--{ prob = 0.4, f = scenarios.kraken_hardest },
			{ prob = 0.1, f = scenarios.swinger_easy },
			{ prob = 0.2, f = scenarios.swinger_hard },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.mast_close_call },
			--{ prob = 0.5, f = scenarios.crows_nest_close_call },
		}
	},
	
	-- Introduce all hard enemies, hard ledges.
	{
		height    = 5202.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.rope },
			--{ prob = 0.5, f = scenarios.mast },
			--{ prob = 0.3, f = scenarios.crows_nest },
			{ prob = 0.5, f = scenarios.mast_hard },
			{ prob = 0.3, f = scenarios.crows_nest_hard },
			--{ prob = 0.1, f = scenarios.cannon_easy },
			{ prob = 0.3, f = scenarios.cannon_hard },
			--{ prob = 0.1, f = scenarios.parrot_easy },
			{ prob = 0.3, f = scenarios.parrot_hard },
			--{ prob = 0.1, f = scenarios.hooker_easy },
			{ prob = 0.3, f = scenarios.hooker_hard },
			{ prob = 0.3, f = scenarios.kraken_easy },
			--{ prob = 0.5, f = scenarios.kraken_hard },
			--{ prob = 0.4, f = scenarios.kraken_hardest },
			--{ prob = 0.1, f = scenarios.swinger_easy },
			{ prob = 0.3, f = scenarios.swinger_hard },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.mast_close_call },
			--{ prob = 0.5, f = scenarios.crows_nest_close_call },
		}
	},
	
	-- Introduce gauntlet, some hard kraken.
	{
		height    = 6222.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.rope },
			--{ prob = 0.5, f = scenarios.mast },
			--{ prob = 0.3, f = scenarios.crows_nest },
			{ prob = 0.3, f = scenarios.mast_hard },
			{ prob = 0.3, f = scenarios.crows_nest_hard },
			--{ prob = 0.1, f = scenarios.cannon_easy },
			{ prob = 0.3, f = scenarios.cannon_hard },
			--{ prob = 0.1, f = scenarios.parrot_easy },
			{ prob = 0.3, f = scenarios.parrot_hard },
			--{ prob = 0.1, f = scenarios.hooker_easy },
			{ prob = 0.3, f = scenarios.hooker_hard },
			{ prob = 0.1, f = scenarios.kraken_easy },
			{ prob = 0.2, f = scenarios.kraken_hard },
			--{ prob = 0.4, f = scenarios.kraken_hardest },
			--{ prob = 0.1, f = scenarios.swinger_easy },
			{ prob = 0.3, f = scenarios.swinger_hard },
			{ prob = 0.2, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.mast_close_call },
			--{ prob = 0.5, f = scenarios.crows_nest_close_call },
		}
	},
	
	-- Add mast close calls, all hard krakens.
	{
		height    = 7242.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.rope },
			--{ prob = 0.5, f = scenarios.mast },
			--{ prob = 0.3, f = scenarios.crows_nest },
			{ prob = 0.3, f = scenarios.mast_hard },
			{ prob = 0.35, f = scenarios.crows_nest_hard },
			--{ prob = 0.1, f = scenarios.cannon_easy },
			{ prob = 0.35, f = scenarios.cannon_hard },
			--{ prob = 0.1, f = scenarios.parrot_easy },
			{ prob = 0.35, f = scenarios.parrot_hard },
			--{ prob = 0.1, f = scenarios.hooker_easy },
			{ prob = 0.35, f = scenarios.hooker_hard },
			--{ prob = 0.1, f = scenarios.kraken_easy },
			{ prob = 0.35, f = scenarios.kraken_hard },
			--{ prob = 0.4, f = scenarios.kraken_hardest },
			--{ prob = 0.1, f = scenarios.swinger_easy },
			{ prob = 0.35, f = scenarios.swinger_hard },
			{ prob = 0.1, f = scenarios.gauntlet },
			{ prob = 0.2, f = scenarios.mast_close_call },
			--{ prob = 0.5, f = scenarios.crows_nest_close_call },
		}
	},
	
	-- Add crows nest close calls, hardest kraken.
	{
		height    =  8262.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.rope },
			--{ prob = 0.5, f = scenarios.mast },
			--{ prob = 0.3, f = scenarios.crows_nest },
			{ prob = 0.2, f = scenarios.mast_hard },
			{ prob = 0.2, f = scenarios.crows_nest_hard },
			--{ prob = 0.1, f = scenarios.cannon_easy },
			{ prob = 0.35, f = scenarios.cannon_hard },
			--{ prob = 0.1, f = scenarios.parrot_easy },
			{ prob = 0.35, f = scenarios.parrot_hard },
			--{ prob = 0.1, f = scenarios.hooker_easy },
			{ prob = 0.35, f = scenarios.hooker_hard },
			--{ prob = 0.1, f = scenarios.kraken_easy },
			{ prob = 0.2, f = scenarios.kraken_hard },
			{ prob = 0.4, f = scenarios.kraken_hardest },
			--{ prob = 0.1, f = scenarios.swinger_easy },
			{ prob = 0.35, f = scenarios.swinger_hard },
			{ prob = 0.1, f = scenarios.gauntlet },
			{ prob = 0.1, f = scenarios.mast_close_call },
			{ prob = 0.2, f = scenarios.crows_nest_close_call },
		}
	},
}	
