------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Scenarios          --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local assets = chunks.assets
local pool   = chunks.pool

---------------
-- scenarios --
---------------

scenarios = {}

-- Empty.
function scenarios.empty( root )
	return { delay = 0.75, lifetime = 0.0, z = 0.0 }
end

-- Shield.
function scenarios.shieldpack( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.shieldpack.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Bridge.
function scenarios.bridge( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, -55.0 )
	
	scenario:spawn( chunks.bridge.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Branch.
function scenarios.branch( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.branch.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Branch hard.
function scenarios.branch_hard( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.branch.spawn() )
	
	return { delay = 0.65, lifetime = time }
end

-- Flytrap.
function scenarios.flytrap( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.flytrap.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Flytrap hard.
function scenarios.flytrap_hard( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.flytrap.spawn() )
	
	return { delay = 0.65, lifetime = time }
end

-- Foliage.
function scenarios.foliage( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 15.0 )
	
	scenario:spawn( chunks.foliage.spawn() )
	
	return { delay = 0.9, lifetime = time }
end

-- Frog easy.
function scenarios.frog_easy( root )
	local time = nil
	
	-- Bridge.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time = scenario.lua:scroll( -0.75, -55.0 )
		scenario:spawn( chunks.bridge.spawn() )
	end
	
	-- Frog.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local time     = scenario.lua:scroll( -0.75, 1.0 )
		local base     = scenario:spawn( ActorBase() )
		local frog     = base:spawn( chunks.frog.spawn() )
		
		base.location = pool.get_v3f( 0.0, 24.0, 0.0 )
		frog.lua:easy()
	end
	
	return { delay = 0.8, lifetime = time }
end

-- Frog hard.
function scenarios.frog_hard( root )
	local time = nil
	
	-- Bridge.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time = scenario.lua:scroll( -0.75, -55.0 )
		scenario:spawn( chunks.bridge.spawn() )
	end
	
	-- Frog.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local time     = scenario.lua:scroll( -0.75, 1.0 )
		local base     = scenario:spawn( ActorBase() )
		local frog     = base:spawn( chunks.frog.spawn() )
		
		base.location = pool.get_v3f( 0.0, 24.0, 0.0 )
		frog.lua:hard()
	end
	
	return { delay = 0.7, lifetime = time }
end

-- Witch easy.
function scenarios.witch_easy( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local witch    = scenario:spawn( chunks.witch.spawn() )
	
	witch.lua:easy()
	
	return { delay = 0.75, lifetime = time }
end

-- Witch hard.
function scenarios.witch_hard( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local witch    = scenario:spawn( chunks.witch.spawn() )
	
	witch.lua:hard()
	
	return { delay = 0.65, lifetime = time }
end

-- Monkey easy.
function scenarios.monkey_easy( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local monkey   = scenario:spawn( chunks.monkey.spawn() )
	
	monkey.lua:easy()
	
	return { delay = 0.8, lifetime = time }
end

-- Beetle easy.
function scenarios.beetle_easy( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local beetle   = scenario:spawn( chunks.beetle.spawn() )
	
	scenario.location = pool.get_v3f( 0.0, 0.0, 3.0 )
	beetle.lua:easy()
	
	return { delay = 1.4, lifetime = 4.5 }
end

-- Beetle hard.
function scenarios.beetle_hard( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local beetle   = scenario:spawn( chunks.beetle.spawn() )
	
	scenario.location = pool.get_v3f( 0.0, 0.0, 3.0 )
	beetle.lua:hard()
	
	return { delay = 1.2, lifetime = 4.5 }
end

-- Snake short.
function scenarios.snake_short( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local snake    = scenario:spawn( chunks.snake.spawn() )
	
	snake.lua:short()
	
	return { delay = 0.8, lifetime = time }
end

-- Snake medium.
function scenarios.snake_medium( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local snake    = scenario:spawn( chunks.snake.spawn() )
	
	snake.lua:medium()
	
	return { delay = 0.9, lifetime = time }
end

-- Snake long.
function scenarios.snake_long( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local snake    = scenario:spawn( chunks.snake.spawn() )
	
	snake.lua:long()
	
	return { delay = 0.95, lifetime = time }
end

-- Gauntlet.
function scenarios.gauntlet( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -1.75, 1.0 )
	
	-- First.
	do
		scenario:spawn( chunks.branch.spawn() )
	end
	
	-- Second.
	do
		local base     = scenario:spawn( ActorBase() )
		local overhang = base:spawn( chunks.branch.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 85 * 2, 0.0 )
	end
	
	-- Third.
	do
		local base     = scenario:spawn( ActorBase() )
		local overhang = base:spawn( chunks.branch.spawn() )
		
		base.location = pool.get_v3f( 0.0, 85 * 4, 0.0 )
	end
	
	-- Forth.
	do
		local base     = scenario:spawn( ActorBase() )
		local overhang = base:spawn( chunks.branch.spawn() )
		
		base.location = pool.get_v3f( 0.0, 85 * 6, 0.0 )
	end
	
	return { delay = 0.75 * 3, lifetime = time }
end

-- Branch close call.
function scenarios.branch_close_call( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.85, 1.0 )
	
	-- First.
	do
		scenario:spawn( chunks.branch.spawn() )
	end
	
	-- Second.
	do
		local base    = scenario:spawn( ActorBase() )
		local balcony = base:spawn( chunks.branch.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 135.0, 0.0 )
	end
	
	return { delay = 1.0, lifetime = time }
end

-- Flytrap close call.
function scenarios.flytrap_close_call( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.8, 1.0 )
	
	-- First.
	do
		scenario:spawn( chunks.flytrap.spawn() )
	end
	
	-- Second.
	do
		local base    = scenario:spawn( ActorBase() )
		local balcony = base:spawn( chunks.flytrap.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 65.0, 0.0 )
	end
	
	return { delay = 1.0, lifetime = time }
end

-- New high score.
function scenarios.new_high_score( root )
	local scenario       = root:spawn( chunks.scenario.spawn() )
	local time           = scenario.lua:scroll( -0.75, -55.0 )
	local new_high_score = scenario:spawn( chunks.new_high_score.spawn() )
	
	new_high_score.lua:celebrate()
	
	return { delay = 1.5, lifetime = time, z = -55.0 }
end

-----------------
-- transitions --
-----------------

transitions =
{
	-------------
	-- TESTING --
	-------------
	
	--[[
	{
		height    = 100000.0,
		scenarios =
		{
			{ prob = 0.5, f = scenarios.empty },
			{ prob = 0.5, f = scenarios.shieldpack },
			--{ prob = 0.5, f = scenarios.bridge },
			--{ prob = 0.5, f = scenarios.branch },
			--{ prob = 0.5, f = scenarios.flytrap },
			--{ prob = 0.5, f = scenarios.branch_hard },
			--{ prob = 0.5, f = scenarios.flytrap_hard },
			--{ prob = 0.5, f = scenarios.frog_easy },
			--{ prob = 0.5, f = scenarios.frog_hard },
			--{ prob = 0.5, f = scenarios.witch_easy },
			--{ prob = 0.5, f = scenarios.witch_hard },
			--{ prob = 0.5, f = scenarios.monkey_easy },
			--{ prob = 0.5, f = scenarios.beetle_easy },
			--{ prob = 0.5, f = scenarios.beetle_hard },
			--{ prob = 0.5, f = scenarios.snake_short },
			--{ prob = 0.5, f = scenarios.snake_medium },
			--{ prob = 0.5, f = scenarios.snake_long },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.branch_close_call },
			--{ prob = 0.5, f = scenarios.flytrap_close_call },
		}
	},
	--]]

	----------------
	-- PRODUCTION --
	----------------
	
	-- At the very beginning, spawn some empty and bridges.
	{
		height    = 102.0,
		scenarios =
		{
			{ prob = 0.8, f = scenarios.empty },
			--{ prob = 0.2, f = scenarios.shieldpack },
			{ prob = 0.3, f = scenarios.bridge },
			--{ prob = 0.5, f = scenarios.branch },
			--{ prob = 0.5, f = scenarios.flytrap },
			--{ prob = 0.5, f = scenarios.branch_hard },
			--{ prob = 0.5, f = scenarios.flytrap_hard },
			--{ prob = 0.5, f = scenarios.frog_easy },
			--{ prob = 0.5, f = scenarios.frog_hard },
			--{ prob = 0.5, f = scenarios.witch_easy },
			--{ prob = 0.5, f = scenarios.witch_hard },
			--{ prob = 0.5, f = scenarios.monkey_easy },
			--{ prob = 0.5, f = scenarios.beetle_easy },
			--{ prob = 0.5, f = scenarios.beetle_hard },
			--{ prob = 0.5, f = scenarios.snake_short },
			--{ prob = 0.5, f = scenarios.snake_medium },
			--{ prob = 0.5, f = scenarios.snake_long },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.branch_close_call },
			--{ prob = 0.5, f = scenarios.flytrap_close_call },
		}
	},
	
	-- Introduce easy frogs/witches, and small snakes.  Also the shieldpack.
	{
		height    = 1122.0,
		scenarios =
		{
			{ prob = 0.1, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			{ prob = 0.1, f = scenarios.bridge },
			--{ prob = 0.5, f = scenarios.branch },
			--{ prob = 0.5, f = scenarios.flytrap },
			--{ prob = 0.5, f = scenarios.branch_hard },
			--{ prob = 0.5, f = scenarios.flytrap_hard },
			{ prob = 0.3, f = scenarios.frog_easy },
			--{ prob = 0.5, f = scenarios.frog_hard },
			{ prob = 0.3, f = scenarios.witch_easy },
			--{ prob = 0.5, f = scenarios.witch_hard },
			--{ prob = 0.5, f = scenarios.monkey_easy },
			--{ prob = 0.5, f = scenarios.beetle_easy },
			--{ prob = 0.5, f = scenarios.beetle_hard },
			{ prob = 0.3, f = scenarios.snake_short },
			--{ prob = 0.5, f = scenarios.snake_medium },
			--{ prob = 0.5, f = scenarios.snake_long },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.branch_close_call },
			--{ prob = 0.5, f = scenarios.flytrap_close_call },
		}
	},
	
	-- Introduce flytrap, rare branch, rare beetle.
	{
		height    = 2142.0,
		scenarios =
		{
			--{ prob = 0.1, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.1, f = scenarios.bridge },
			{ prob = 0.2, f = scenarios.branch },
			{ prob = 0.4, f = scenarios.flytrap },
			--{ prob = 0.5, f = scenarios.branch_hard },
			--{ prob = 0.5, f = scenarios.flytrap_hard },
			{ prob = 0.3, f = scenarios.frog_easy },
			--{ prob = 0.5, f = scenarios.frog_hard },
			{ prob = 0.3, f = scenarios.witch_easy },
			--{ prob = 0.5, f = scenarios.witch_hard },
			--{ prob = 0.5, f = scenarios.monkey_easy },
			{ prob = 0.15, f = scenarios.beetle_easy },
			--{ prob = 0.5, f = scenarios.beetle_hard },
			{ prob = 0.3, f = scenarios.snake_short },
			--{ prob = 0.15, f = scenarios.snake_medium },
			--{ prob = 0.5, f = scenarios.snake_long },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.branch_close_call },
			--{ prob = 0.5, f = scenarios.flytrap_close_call },
		}
	},
	
	-- Introduce medium snakes, and rare monkies.  Increase branches.
	{
		height    = 3162.0,
		scenarios =
		{
			--{ prob = 0.1, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.1, f = scenarios.bridge },
			{ prob = 0.4, f = scenarios.branch },
			{ prob = 0.3, f = scenarios.flytrap },
			--{ prob = 0.5, f = scenarios.branch_hard },
			--{ prob = 0.5, f = scenarios.flytrap_hard },
			{ prob = 0.3, f = scenarios.frog_easy },
			--{ prob = 0.5, f = scenarios.frog_hard },
			{ prob = 0.3, f = scenarios.witch_easy },
			--{ prob = 0.5, f = scenarios.witch_hard },
			{ prob = 0.1, f = scenarios.monkey_easy },
			{ prob = 0.3, f = scenarios.beetle_easy },
			--{ prob = 0.5, f = scenarios.beetle_hard },
			{ prob = 0.1, f = scenarios.snake_short },
			{ prob = 0.2, f = scenarios.snake_medium },
			--{ prob = 0.5, f = scenarios.snake_long },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.branch_close_call },
			--{ prob = 0.5, f = scenarios.flytrap_close_call },
		}
	},
	
	-- Introduce hard enemies, partially.  Increase monkies.
	{
		height    = 4182.0,
		scenarios =
		{
			--{ prob = 0.1, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.1, f = scenarios.bridge },
			{ prob = 0.5, f = scenarios.branch },
			{ prob = 0.3, f = scenarios.flytrap },
			--{ prob = 0.5, f = scenarios.branch_hard },
			--{ prob = 0.5, f = scenarios.flytrap_hard },
			{ prob = 0.1, f = scenarios.frog_easy },
			{ prob = 0.2, f = scenarios.frog_hard },
			{ prob = 0.1, f = scenarios.witch_easy },
			{ prob = 0.2, f = scenarios.witch_hard },
			{ prob = 0.3, f = scenarios.monkey_easy },
			{ prob = 0.1, f = scenarios.beetle_easy },
			{ prob = 0.2, f = scenarios.beetle_hard },
			{ prob = 0.1, f = scenarios.snake_short },
			{ prob = 0.2, f = scenarios.snake_medium },
			--{ prob = 0.5, f = scenarios.snake_long },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.branch_close_call },
			--{ prob = 0.5, f = scenarios.flytrap_close_call },
		}
	},
	
	-- Introduce all hard enemies, long snakes, hard ledges.
	{
		height    = 5202.0,
		scenarios =
		{
			--{ prob = 0.1, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.1, f = scenarios.bridge },
			--{ prob = 0.5, f = scenarios.branch },
			--{ prob = 0.3, f = scenarios.flytrap },
			{ prob = 0.5, f = scenarios.branch_hard },
			{ prob = 0.3, f = scenarios.flytrap_hard },
			--{ prob = 0.1, f = scenarios.frog_easy },
			{ prob = 0.3, f = scenarios.frog_hard },
			--{ prob = 0.1, f = scenarios.witch_easy },
			{ prob = 0.3, f = scenarios.witch_hard },
			{ prob = 0.35, f = scenarios.monkey_easy },
			--{ prob = 0.1, f = scenarios.beetle_easy },
			{ prob = 0.3, f = scenarios.beetle_hard },
			--{ prob = 0.1, f = scenarios.snake_short },
			{ prob = 0.15, f = scenarios.snake_medium },
			{ prob = 0.15, f = scenarios.snake_long },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.branch_close_call },
			--{ prob = 0.5, f = scenarios.flytrap_close_call },
		}
	},
	
	-- Introduce gauntlet.
	{
		height    = 6222.0,
		scenarios =
		{
			--{ prob = 0.1, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.1, f = scenarios.bridge },
			--{ prob = 0.5, f = scenarios.branch },
			--{ prob = 0.3, f = scenarios.flytrap },
			{ prob = 0.3, f = scenarios.branch_hard },
			{ prob = 0.3, f = scenarios.flytrap_hard },
			--{ prob = 0.1, f = scenarios.frog_easy },
			{ prob = 0.3, f = scenarios.frog_hard },
			--{ prob = 0.1, f = scenarios.witch_easy },
			{ prob = 0.3, f = scenarios.witch_hard },
			{ prob = 0.3, f = scenarios.monkey_easy },
			--{ prob = 0.1, f = scenarios.beetle_easy },
			{ prob = 0.3, f = scenarios.beetle_hard },
			--{ prob = 0.1, f = scenarios.snake_short },
			{ prob = 0.1, f = scenarios.snake_medium },
			{ prob = 0.2, f = scenarios.snake_long },
			{ prob = 0.2, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.branch_close_call },
			--{ prob = 0.5, f = scenarios.flytrap_close_call },
		}
	},
	
	-- Add branch close calls.
	{
		height    = 7242.0,
		scenarios =
		{
			--{ prob = 0.1, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.1, f = scenarios.bridge },
			--{ prob = 0.5, f = scenarios.branch },
			--{ prob = 0.3, f = scenarios.flytrap },
			{ prob = 0.3, f = scenarios.branch_hard },
			{ prob = 0.35, f = scenarios.flytrap_hard },
			--{ prob = 0.1, f = scenarios.frog_easy },
			{ prob = 0.35, f = scenarios.frog_hard },
			--{ prob = 0.1, f = scenarios.witch_easy },
			{ prob = 0.35, f = scenarios.witch_hard },
			{ prob = 0.35, f = scenarios.monkey_easy },
			--{ prob = 0.1, f = scenarios.beetle_easy },
			{ prob = 0.35, f = scenarios.beetle_hard },
			--{ prob = 0.1, f = scenarios.snake_short },
			{ prob = 0.15, f = scenarios.snake_medium },
			{ prob = 0.2, f = scenarios.snake_long },
			{ prob = 0.1, f = scenarios.gauntlet },
			{ prob = 0.2, f = scenarios.branch_close_call },
			--{ prob = 0.5, f = scenarios.flytrap_close_call },
		}
	},
	
	-- Add flytrap close calls.
	{
		height    =  8262.0,
		scenarios =
		{
			--{ prob = 0.1, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.1, f = scenarios.bridge },
			--{ prob = 0.5, f = scenarios.branch },
			--{ prob = 0.3, f = scenarios.flytrap },
			{ prob = 0.2, f = scenarios.branch_hard },
			{ prob = 0.2, f = scenarios.flytrap_hard },
			--{ prob = 0.1, f = scenarios.frog_easy },
			{ prob = 0.35, f = scenarios.frog_hard },
			--{ prob = 0.1, f = scenarios.witch_easy },
			{ prob = 0.35, f = scenarios.witch_hard },
			{ prob = 0.35, f = scenarios.monkey_easy },
			--{ prob = 0.1, f = scenarios.beetle_easy },
			{ prob = 0.35, f = scenarios.beetle_hard },
			--{ prob = 0.1, f = scenarios.snake_short },
			{ prob = 0.15, f = scenarios.snake_medium },
			{ prob = 0.2, f = scenarios.snake_long },
			{ prob = 0.1, f = scenarios.gauntlet },
			{ prob = 0.1, f = scenarios.branch_close_call },
			{ prob = 0.2, f = scenarios.flytrap_close_call },
		}
	},
}	
