------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Scenarios          --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local assets = chunks.assets
local pool   = chunks.pool

---------------
-- scenarios --
---------------

scenarios = {}

-- Empty.
function scenarios.empty( root )
	return { delay = 0.75, lifetime = 0.0, z = 0.0 }
end

-- Laundry.
function scenarios.laundry( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, -55.0 )
	
	scenario:spawn( chunks.laundry.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Shield.
function scenarios.shieldpack( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.shieldpack.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Balcony.
function scenarios.balcony( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.balcony.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Balcony hard.
function scenarios.balcony_hard( root )
	local result = scenarios.balcony( root )
	
	return { delay = 0.6, lifetime = result.lifetime } 
end

-- Overhang.
function scenarios.overhang( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.overhang.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Overhang hard.
function scenarios.overhang_hard( root )
	local result = scenarios.overhang( root )
	
	return { delay = 0.6, lifetime = result.lifetime }
end

-- Thrower single.
function scenarios.thrower_single( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local thrower  = scenario:spawn( chunks.thrower.spawn() )
	
	thrower.lua:single()
	
	return { delay = 0.75, lifetime = time }
end

-- Thrower single hard.
function scenarios.thrower_single_hard( root )
	local result = scenarios.thrower_single( root )
	
	return { delay = 0.6, lifetime = result.lifetime }
end

-- Thrower double.
function scenarios.thrower_double( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local thrower  = scenario:spawn( chunks.thrower.spawn() )
	
	thrower.lua:double()
	
	return { delay = 0.75, lifetime = time }
end

-- Thrower double hard.
function scenarios.thrower_double_hard( root )
	local result = scenarios.thrower_double( root )
	
	return { delay = 0.6, lifetime = result.lifetime }
end

-- Squirrel easy.
function scenarios.squirrel_easy( root )
	local time = nil
	
	-- Laundry.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time = scenario.lua:scroll( -0.75, -55.0 )
		scenario:spawn( chunks.laundry.spawn() )
	end
	
	-- Squirrel.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local time     = scenario.lua:scroll( -0.75, 1.0 )
		local base     = scenario:spawn( ActorBase() )
		local squirrel = base:spawn( chunks.squirrel.spawn() )
		
		base.location = pool.get_v3f( 0.0, 28.0, 0.0 )
		squirrel.lua:easy()
	end
	
	return { delay = 0.75, lifetime = time }
end

-- Squirrel hard.
function scenarios.squirrel_hard( root )
	local time = nil
	
	-- Laundry.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time = scenario.lua:scroll( -0.75, -55.0 )
		scenario:spawn( chunks.laundry.spawn() )
	end
	
	-- Squirrel.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local time     = scenario.lua:scroll( -0.75, 1.0 )
		local base     = scenario:spawn( ActorBase() )
		local squirrel = base:spawn( chunks.squirrel.spawn() )
		
		base.location = pool.get_v3f( 0.0, 28.0, 0.0 )
		squirrel.lua:hard()
	end
	
	return { delay = 0.75, lifetime = time }
end

-- Squirrel hardest.
function scenarios.squirrel_hardest( root )
	local result = scenarios.squirrel_hard( root )
	
	return { delay = 0.6, lifetime = result.lifetime }
end

-- Bird easy.
function scenarios.bird_easy( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local bird     = scenario:spawn( chunks.bird.spawn() )
	
	scenario.location = pool.get_v3f( 0.0, 0.0, 3.0 )
	bird.lua:easy()
	
	return { delay = 1.8, lifetime = 4.5 }
end

-- Bird hard.
function scenarios.bird_hard( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local bird     = scenario:spawn( chunks.bird.spawn() )
	
	scenario.location = pool.get_v3f( 0.0, 0.0, 3.0 )
	bird.lua:hard()
	
	return { delay = 1.7, lifetime = 4.5 }
end

-- Climber.
function scenarios.climber( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local climber  = scenario:spawn( chunks.climber.spawn() )
	
	climber.lua:easy()
	
	return { delay = 1.2, lifetime = time }
end

-- Thrower bomb.
function scenarios.thrower_bomb( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local thrower  = scenario:spawn( chunks.thrower.spawn() )
	
	thrower.lua:bomb()
	
	return { delay = 0.85, lifetime = time }
end

-- Thrower bomb hard.
function scenarios.thrower_bomb_hard( root )
	local result = scenarios.thrower_bomb( root )
	
	return { delay = 0.7, lifetime = result.lifetime }
end

-- Gauntlet.
function scenarios.gauntlet( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -1.75, 1.0 )
	
	-- First.
	do
		scenario:spawn( chunks.overhang.spawn() )
	end
	
	-- Second.
	do
		local base     = scenario:spawn( ActorBase() )
		local overhang = base:spawn( chunks.overhang.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 80 * 2, 0.0 )
	end
	
	-- Third.
	do
		local base     = scenario:spawn( ActorBase() )
		local overhang = base:spawn( chunks.overhang.spawn() )
		
		base.location = pool.get_v3f( 0.0, 80 * 4, 0.0 )
	end
	
	-- Forth.
	do
		local base     = scenario:spawn( ActorBase() )
		local overhang = base:spawn( chunks.overhang.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 80 * 6, 0.0 )
	end
	
	return { delay = 0.75 * 3, lifetime = time }
end

-- Overhang close call.
function scenarios.overhang_close_call( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.8, 1.0 )
	
	-- First.
	do
		scenario:spawn( chunks.overhang.spawn() )
	end
	
	-- Second.
	do
		local base    = scenario:spawn( ActorBase() )
		local balcony = base:spawn( chunks.overhang.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 120.0, 0.0 )
	end
	
	return { delay = 1.0, lifetime = time }
end

-- Balcony close call.
function scenarios.balcony_close_call( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.8, 1.0 )
	
	-- First.
	do
		scenario:spawn( chunks.balcony.spawn() )
	end
	
	-- Second.
	do
		local base    = scenario:spawn( ActorBase() )
		local balcony = base:spawn( chunks.balcony.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 70.0, 0.0 )
	end
	
	return { delay = 1.0, lifetime = time }
end

-- New high score.
function scenarios.new_high_score( root )
	local scenario       = root:spawn( chunks.scenario.spawn() )
	local time           = scenario.lua:scroll( -0.75, -55.0 )
	local new_high_score = scenario:spawn( chunks.new_high_score.spawn() )
	
	new_high_score.lua:celebrate()
	
	return { delay = 1.5, lifetime = time, z = -55.0 }
end

-----------------
-- transitions --
-----------------

transitions =
{
	-------------
	-- TESTING --
	-------------
	
	--[[
	{
		height    = 100000.0,
		scenarios =
		{
			{ prob = 0.5, f = scenarios.empty },
			{ prob = 0.5, f = scenarios.laundry },
			--{ prob = 0.5, f = scenarios.shieldpack },
			--{ prob = 0.9, f = scenarios.balcony },
			--{ prob = 0.5, f = scenarios.overhang },
			--{ prob = 0.5, f = scenarios.thrower_single },
			--{ prob = 0.5, f = scenarios.thrower_double },
			--{ prob = 0.5, f = scenarios.squirrel_easy },
			--{ prob = 0.5, f = scenarios.squirrel_hard },
			--{ prob = 0.5, f = scenarios.bird_easy },
			--{ prob = 0.5, f = scenarios.bird_hard },
			--{ prob = 0.5, f = scenarios.climber },
			--{ prob = 0.5, f = scenarios.thrower_bomb },
			--{ prob = 0.5, f = scenarios.new_high_score },
			--{ prob = 0.5, f = scenarios.balcony_hard },
			--{ prob = 0.5, f = scenarios.overhang_hard },
			--{ prob = 0.5, f = scenarios.thrower_single_hard },
			--{ prob = 0.5, f = scenarios.thrower_double_hard },
			--{ prob = 0.5, f = scenarios.squirrel_hardest },
			--{ prob = 0.5, f = scenarios.thrower_bomb_hard },
			--{ prob = 0.5, f = scenarios.gauntlet },
			--{ prob = 0.5, f = scenarios.overhang_close_call },
			--{ prob = 0.5, f = scenarios.balcony_close_call },
		}
	},
	--]]
	
	----------------
	-- PRODUCTION --
	----------------
	---[[
	-- At the very beginning, spawn some empty and laundry.
	{
		height    = 102.0,
		scenarios =
		{
			{ prob = 0.8, f = scenarios.empty },
			{ prob = 0.3, f = scenarios.laundry },
			--{ prob = 0.0, f = scenarios.shieldpack },
			--{ prob = 0.0, f = scenarios.balcony },
			--{ prob = 0.0, f = scenarios.balcony_hard },
			--{ prob = 0.0, f = scenarios.overhang },
			--{ prob = 0.0, f = scenarios.overhang_hard },
			--{ prob = 0.0, f = scenarios.thrower_single },
			--{ prob = 0.0, f = scenarios.thrower_single_hard },
			--{ prob = 0.0, f = scenarios.thrower_double },
			--{ prob = 0.0, f = scenarios.thrower_double_hard },
			--{ prob = 0.0, f = scenarios.squirrel_easy },
			--{ prob = 0.0, f = scenarios.squirrel_hard },
			--{ prob = 0.0, f = scenarios.squirrel_hardest },
			--{ prob = 0.0, f = scenarios.bird_easy },
			--{ prob = 0.0, f = scenarios.bird_hard },
			--{ prob = 0.0, f = scenarios.climber },
			--{ prob = 0.0, f = scenarios.thrower_bomb },
			--{ prob = 0.0, f = scenarios.thrower_bomb_hard },
			--{ prob = 0.0, f = scenarios.gauntlet },
			--{ prob = 0.0, f = scenarios.balcony_close_call },
			--{ prob = 0.0, f = scenarios.overhang_close_call },
		}
	},
	
	-- Introduce easy ninja throwers, birds and squirrels.  Also the shieldpack.
	{
		height    = 1122.0,
		scenarios =
		{
			{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.3, f = scenarios.laundry },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.0, f = scenarios.balcony },
			--{ prob = 0.0, f = scenarios.balcony_hard },
			--{ prob = 0.0, f = scenarios.overhang },
			--{ prob = 0.0, f = scenarios.overhang_hard },
			{ prob = 0.2, f = scenarios.thrower_single },
			--{ prob = 0.0, f = scenarios.thrower_single_hard },
			--{ prob = 0.0, f = scenarios.thrower_double },
			--{ prob = 0.0, f = scenarios.thrower_double_hard },
			{ prob = 0.2, f = scenarios.squirrel_easy },
			--{ prob = 0.0, f = scenarios.squirrel_hard },
			--{ prob = 0.0, f = scenarios.squirrel_hardest },
			{ prob = 0.2, f = scenarios.bird_easy },
			--{ prob = 0.0, f = scenarios.bird_hard },
			--{ prob = 0.0, f = scenarios.climber },
			--{ prob = 0.0, f = scenarios.thrower_bomb },
			--{ prob = 0.0, f = scenarios.thrower_bomb_hard },
			--{ prob = 0.0, f = scenarios.gauntlet },
			--{ prob = 0.0, f = scenarios.balcony_close_call },
			--{ prob = 0.0, f = scenarios.overhang_close_call },
		}
	},
	
	-- Introduce double ninja throwers, small overhang.
	{
		height    = 2142.0,
		scenarios =
		{
			--{ prob = 0.0, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.laundry },
			{ prob = 0.2, f = scenarios.shieldpack },
			{ prob = 0.5, f = scenarios.balcony },
			--{ prob = 0.0, f = scenarios.balcony_hard },
			--{ prob = 0.0, f = scenarios.overhang },
			--{ prob = 0.0, f = scenarios.overhang_hard },
			--{ prob = 0.3, f = scenarios.thrower_single },
			--{ prob = 0.0, f = scenarios.thrower_single_hard },
			{ prob = 0.4, f = scenarios.thrower_double },
			--{ prob = 0.0, f = scenarios.thrower_double_hard },
			{ prob = 0.3, f = scenarios.squirrel_easy },
			--{ prob = 0.0, f = scenarios.squirrel_hard },
			--{ prob = 0.0, f = scenarios.squirrel_hardest },
			{ prob = 0.3, f = scenarios.bird_easy },
			--{ prob = 0.0, f = scenarios.bird_hard },
			--{ prob = 0.0, f = scenarios.climber },
			--{ prob = 0.0, f = scenarios.thrower_bomb },
			--{ prob = 0.0, f = scenarios.thrower_bomb_hard },
			--{ prob = 0.0, f = scenarios.gauntlet },
			--{ prob = 0.0, f = scenarios.balcony_close_call },
			--{ prob = 0.0, f = scenarios.overhang_close_call },
		}
	},
	
	-- Introduce large overhangs, climbing ninjas.
	{
		height    = 3162.0,
		scenarios =
		{
			--{ prob = 0.0, f = scenarios.empty },
			--{ prob = 0.2, f = scenarios.laundry },
			{ prob = 0.2, f = scenarios.shieldpack },
			{ prob = 0.4, f = scenarios.balcony },
			--{ prob = 0.0, f = scenarios.balcony_hard },
			{ prob = 0.5, f = scenarios.overhang },
			--{ prob = 0.0, f = scenarios.overhang_hard },
			--{ prob = 0.3, f = scenarios.thrower_single },
			--{ prob = 0.0, f = scenarios.thrower_single_hard },
			{ prob = 0.3, f = scenarios.thrower_double },
			--{ prob = 0.0, f = scenarios.thrower_double_hard },
			{ prob = 0.3, f = scenarios.squirrel_easy },
			--{ prob = 0.0, f = scenarios.squirrel_hard },
			--{ prob = 0.0, f = scenarios.squirrel_hardest },
			{ prob = 0.3, f = scenarios.bird_easy },
			--{ prob = 0.0, f = scenarios.bird_hard },
			{ prob = 0.4, f = scenarios.climber },
			--{ prob = 0.0, f = scenarios.thrower_bomb },
			--{ prob = 0.0, f = scenarios.thrower_bomb_hard },
			--{ prob = 0.0, f = scenarios.gauntlet },
			--{ prob = 0.0, f = scenarios.balcony_close_call },
			--{ prob = 0.0, f = scenarios.overhang_close_call },
		}
	},
	
	-- Introduce firecracker ninjas, hard birds, hard squirrels, hard single ninja throwers.
	{
		height    = 4182.0,
		scenarios =
		{
			--{ prob = 0.0, f = scenarios.empty },
			--{ prob = 0.2, f = scenarios.laundry },
			{ prob = 0.2, f = scenarios.shieldpack },
			{ prob = 0.3, f = scenarios.balcony },
			--{ prob = 0.0, f = scenarios.balcony_hard },
			{ prob = 0.4, f = scenarios.overhang },
			--{ prob = 0.0, f = scenarios.overhang_hard },
			--{ prob = 0.3, f = scenarios.thrower_single },
			{ prob = 0.3, f = scenarios.thrower_single_hard },
			--{ prob = 0.3, f = scenarios.thrower_double },
			--{ prob = 0.0, f = scenarios.thrower_double_hard },
			--{ prob = 0.3, f = scenarios.squirrel_easy },
			{ prob = 0.3, f = scenarios.squirrel_hard },
			--{ prob = 0.0, f = scenarios.squirrel_hardest },
			--{ prob = 0.3, f = scenarios.bird_easy },
			{ prob = 0.3, f = scenarios.bird_hard },
			{ prob = 0.3, f = scenarios.climber },
			{ prob = 0.5, f = scenarios.thrower_bomb },
			--{ prob = 0.0, f = scenarios.thrower_bomb_hard },
			--{ prob = 0.0, f = scenarios.gauntlet },
			--{ prob = 0.0, f = scenarios.balcony_close_call },
			--{ prob = 0.0, f = scenarios.overhang_close_call },
		}
	},
	
	-- Introduce hard double ninja throwers, hard single ledges, harder squirrels.
	{
		height    = 5202.0,
		scenarios =
		{
			--{ prob = 0.0, f = scenarios.empty },
			--{ prob = 0.2, f = scenarios.laundry },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.balcony },
			{ prob = 0.5, f = scenarios.balcony_hard },
			{ prob = 0.4, f = scenarios.overhang },
			--{ prob = 0.0, f = scenarios.overhang_hard },
			--{ prob = 0.3, f = scenarios.thrower_single },
			--{ prob = 0.3, f = scenarios.thrower_single_hard },
			--{ prob = 0.3, f = scenarios.thrower_double },
			{ prob = 0.4, f = scenarios.thrower_double_hard },
			--{ prob = 0.3, f = scenarios.squirrel_easy },
			--{ prob = 0.3, f = scenarios.squirrel_hard },
			{ prob = 0.4, f = scenarios.squirrel_hardest },
			--{ prob = 0.3, f = scenarios.bird_easy },
			{ prob = 0.3, f = scenarios.bird_hard },
			{ prob = 0.3, f = scenarios.climber },
			{ prob = 0.3, f = scenarios.thrower_bomb },
			--{ prob = 0.0, f = scenarios.thrower_bomb_hard },
			--{ prob = 0.0, f = scenarios.gauntlet },
			--{ prob = 0.0, f = scenarios.balcony_close_call },
			--{ prob = 0.0, f = scenarios.overhang_close_call },
		}
	},
	
	-- Introduce hard big ledges, hard firecracker ninjas, gauntlet.
	{
		height    = 6222.0,
		scenarios =
		{
			--{ prob = 0.0, f = scenarios.empty },
			--{ prob = 0.2, f = scenarios.laundry },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.balcony },
			{ prob = 0.4, f = scenarios.balcony_hard },
			--{ prob = 0.3, f = scenarios.overhang },
			{ prob = 0.5, f = scenarios.overhang_hard },
			--{ prob = 0.3, f = scenarios.thrower_single },
			--{ prob = 0.3, f = scenarios.thrower_single_hard },
			--{ prob = 0.3, f = scenarios.thrower_double },
			{ prob = 0.4, f = scenarios.thrower_double_hard },
			--{ prob = 0.3, f = scenarios.squirrel_easy },
			--{ prob = 0.3, f = scenarios.squirrel_hard },
			{ prob = 0.4, f = scenarios.squirrel_hardest },
			--{ prob = 0.3, f = scenarios.bird_easy },
			{ prob = 0.3, f = scenarios.bird_hard },
			{ prob = 0.3, f = scenarios.climber },
			--{ prob = 0.3, f = scenarios.thrower_bomb },
			{ prob = 0.4, f = scenarios.thrower_bomb_hard },
			{ prob = 0.2, f = scenarios.gauntlet },
			--{ prob = 0.0, f = scenarios.balcony_close_call },
			--{ prob = 0.0, f = scenarios.overhang_close_call },
		}
	},
	
	-- Introduce small crazy ledges.  
	{
		height    = 7242.0,
		scenarios =
		{
			--{ prob = 0.0, f = scenarios.empty },
			--{ prob = 0.2, f = scenarios.laundry },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.balcony },
			{ prob = 0.4, f = scenarios.balcony_hard },
			--{ prob = 0.3, f = scenarios.overhang },
			{ prob = 0.5, f = scenarios.overhang_hard },
			--{ prob = 0.3, f = scenarios.thrower_single },
			--{ prob = 0.3, f = scenarios.thrower_single_hard },
			--{ prob = 0.3, f = scenarios.thrower_double },
			{ prob = 0.4, f = scenarios.thrower_double_hard },
			--{ prob = 0.3, f = scenarios.squirrel_easy },
			--{ prob = 0.3, f = scenarios.squirrel_hard },
			{ prob = 0.4, f = scenarios.squirrel_hardest },
			--{ prob = 0.3, f = scenarios.bird_easy },
			{ prob = 0.3, f = scenarios.bird_hard },
			{ prob = 0.3, f = scenarios.climber },
			--{ prob = 0.3, f = scenarios.thrower_bomb },
			{ prob = 0.4, f = scenarios.thrower_bomb_hard },
			{ prob = 0.1, f = scenarios.gauntlet },
			--{ prob = 0.0, f = scenarios.balcony_close_call },
			{ prob = 0.3, f = scenarios.overhang_close_call },
		}
	},
	
	-- Introduce final crazy ledge.  Rinse and repeat.
	{
		height    = 8262.0,
		scenarios =
		{
			--{ prob = 0.0, f = scenarios.empty },
			--{ prob = 0.2, f = scenarios.laundry },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.balcony },
			{ prob = 0.4, f = scenarios.balcony_hard },
			--{ prob = 0.3, f = scenarios.overhang },
			{ prob = 0.5, f = scenarios.overhang_hard },
			--{ prob = 0.3, f = scenarios.thrower_single },
			--{ prob = 0.3, f = scenarios.thrower_single_hard },
			--{ prob = 0.3, f = scenarios.thrower_double },
			{ prob = 0.4, f = scenarios.thrower_double_hard },
			--{ prob = 0.3, f = scenarios.squirrel_easy },
			--{ prob = 0.3, f = scenarios.squirrel_hard },
			{ prob = 0.4, f = scenarios.squirrel_hardest },
			--{ prob = 0.3, f = scenarios.bird_easy },
			{ prob = 0.3, f = scenarios.bird_hard },
			{ prob = 0.3, f = scenarios.climber },
			--{ prob = 0.3, f = scenarios.thrower_bomb },
			{ prob = 0.4, f = scenarios.thrower_bomb_hard },
			{ prob = 0.1, f = scenarios.gauntlet },
			{ prob = 0.3, f = scenarios.balcony_close_call },
			{ prob = 0.2, f = scenarios.overhang_close_call },
		}
	}
	--]]
}	
