------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Scenarios          --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local assets = chunks.assets
local pool   = chunks.pool

---------------
-- scenarios --
---------------

scenarios = {}

-- Empty.
function scenarios.empty( root )
	return { delay = 0.75, lifetime = 0.0, z = 0.0 }
end

-- Shield.
function scenarios.shieldpack( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	
	scenario:spawn( chunks.shieldpack.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Drawbridge.
function scenarios.drawbridge( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 ) 
	
	scenario:spawn( chunks.drawbridge.spawn() )
	
	return { delay = 1.2, lifetime = time }
end

-- Knight.
function scenarios.knight( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local knight   = scenario:spawn( chunks.knight.spawn() )
	
	knight.lua:easy()
	
	return { delay = 1.2, lifetime = time }
end

-- Horse.
function scenarios.horse( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local horse    = scenario:spawn( chunks.horse.spawn() )
	
	horse.lua:easy()
	
	return { delay = 1.2, lifetime = time }
end

-- Chain.
function scenarios.chain( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, -55.0 )
	
	scenario:spawn( chunks.chain.spawn() )
	
	return { delay = 0.75, lifetime = time }
end

-- Troll easy.
function scenarios.troll_easy( root )
	local time = nil
	
	-- Bridge.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time = scenario.lua:scroll( -0.75, -55.0 )
		scenario:spawn( chunks.bridge.spawn() )
	end
	
	-- Troll.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local time     = scenario.lua:scroll( -0.75, 1.0 )
		local base     = scenario:spawn( ActorBase() )
		local troll    = base:spawn( chunks.troll.spawn() )
		
		base.location = pool.get_v3f( 0.0, 64.0, 0.0 )
		troll.lua:easy()
	end
	
	return { delay = 0.9, lifetime = time }
end

-- Troll hard.
function scenarios.troll_hard( root )
	local time = nil
	
	-- Bridge.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time = scenario.lua:scroll( -0.75, -55.0 )
		scenario:spawn( chunks.bridge.spawn() )
	end
	
	-- Troll.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local time     = scenario.lua:scroll( -0.75, 1.0 )
		local base     = scenario:spawn( ActorBase() )
		local troll    = base:spawn( chunks.troll.spawn() )
		
		base.location = pool.get_v3f( 0.0, 64.0, 0.0 )
		troll.lua:hard()
	end
	
	return { delay = 0.9, lifetime = time }
end

-- Archer easy.
function scenarios.archer_easy( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local archer   = scenario:spawn( chunks.archer.spawn() )
	
	archer.lua:easy()
	
	return { delay = 1.0, lifetime = time }
end

-- Archer hard.
function scenarios.archer_hard( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
	local archer   = scenario:spawn( chunks.archer.spawn() )
	
	archer.lua:hard()
	
	return { delay = 0.9, lifetime = time }
end

-- Gargoyle easy.
function scenarios.gargoyle_easy( root )
	local time     = nil
	local callback = nil
	
	-- Gargoyle.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local gargoyle = scenario:spawn( chunks.gargoyle.spawn() )
		
		callback = function( instigator )
			gargoyle.lua:easy( instigator )
		end
	end
	
	-- Frozen.
	do
		local scenario        = root:spawn( chunks.scenario.spawn() )
		local gargoyle_frozen = scenario:spawn( chunks.gargoyle.spawn_frozen() )
		
		scenario.lua:scroll( -0.75, 1.0 ) 
		
		gargoyle_frozen.lua:easy( callback )
	end
	
	return { delay = 1.2, lifetime = 4.2 }  
end

-- Gargoyle hard.
function scenarios.gargoyle_hard( root )
	local time     = nil
	local callback = nil
	
	-- Gargoyle.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local gargoyle = scenario:spawn( chunks.gargoyle.spawn() )
		
		callback = function( instigator )
			gargoyle.lua:hard( instigator )
		end
	end
	
	-- Frozen.
	do
		local scenario        = root:spawn( chunks.scenario.spawn() )
		local gargoyle_frozen = scenario:spawn( chunks.gargoyle.spawn_frozen() )
		
		scenario.lua:scroll( -0.75, 1.0 ) 
		
		gargoyle_frozen.lua:easy( callback )
	end
	
	return { delay = 1.1, lifetime = 3.2 }  
end

-- Catapult easy.
function scenarios.catapult_easy( root )
	local time     = nil
	local catapult = nil
	
	-- Cannon.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time     = scenario.lua:scroll( -0.75, 1.0 )
		catapult = scenario:spawn( chunks.catapult.spawn() )
	end

	-- Cannonballs.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.lua:scroll( -0.75, 0.5 )
		catapult.lua:easy( scenario )
	end
	
	return { delay = 1.0, lifetime = time }
end

-- Catapult hard.
function scenarios.catapult_hard( root )
	local time     = nil
	local catapult = nil
	
	-- Cannon.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		time     = scenario.lua:scroll( -0.75, 1.0 )
		catapult = scenario:spawn( chunks.catapult.spawn() )
	end

	-- Cannonballs.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		
		scenario.lua:scroll( -0.75, 0.5 )
		catapult.lua:hard( scenario )
	end
	
	return { delay = 1.0, lifetime = time }
end

-- Dragon easy.
function scenarios.dragon_easy( root )
	local fireball_layer          = nil
	local fireball_particle_layer = nil
	
	-- Fireball particle layer.
	do
		fireball_particle_layer          = root:spawn( chunks.scenario.spawn() )
		fireball_particle_layer.location = pool.get_v3f( 0.0, 0.0, 3.5 )
	end
	
	-- Fireball layer.
	do
		fireball_layer          = root:spawn( chunks.scenario.spawn() )
		fireball_layer.location = pool.get_v3f( 0.0, 0.0, 4.0 )
	end

	-- Dragon.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local dragon   = scenario:spawn( chunks.dragon.spawn() )
		
		scenario.location = pool.get_v3f( 0.0, 0.0, 6.0 )
		
		dragon.lua:easy( fireball_layer, fireball_particle_layer )
	end

	return { delay = 3.0, lifetime = 5.0 }
end

-- Dragon hard.
function scenarios.dragon_hard( root )
	local fireball_layer          = nil
	local fireball_particle_layer = nil
	
	-- Fireball particle layer.
	do
		fireball_particle_layer          = root:spawn( chunks.scenario.spawn() )
		fireball_particle_layer.location = pool.get_v3f( 0.0, 0.0, 3.5 )
	end
	
	-- Fireball layer.
	do
		fireball_layer          = root:spawn( chunks.scenario.spawn() )
		fireball_layer.location = pool.get_v3f( 0.0, 0.0, 4.0 )
	end

	-- Dragon.
	do
		local scenario = root:spawn( chunks.scenario.spawn() )
		local dragon   = scenario:spawn( chunks.dragon.spawn() )
		
		scenario.location = pool.get_v3f( 0.0, 0.0, 6.0 )
		
		dragon.lua:hard( fireball_layer, fireball_particle_layer )
	end

	return { delay = 2.6, lifetime = 4.0 }
end

-- Dragon cheap shot.
function scenarios.dragon_cheap_shot( root )
	-- Catapult.
	do
		local catapult = nil
	
		-- Cannon.
		do
			local scenario = root:spawn( chunks.scenario.spawn() )
			local base     = scenario:spawn( ActorBase() )
			
			scenario.lua:scroll( -0.75, 1.0 )
			base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
			catapult      = base:spawn( chunks.catapult.spawn() )
		end
	
		-- Cannonballs.
		do
			local scenario = root:spawn( chunks.scenario.spawn() )
			local base     = scenario:spawn( ActorBase() )
			
			scenario.lua:scroll( -0.75, 0.5 )
			base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
			catapult.lua:hard( base )
		end
	end

	-- Dragon (use dragon timings).
	return scenarios.dragon_hard( root )
end

-- Dragon cheaper shot.
function scenarios.dragon_cheaper_shot( root )
	local sleep_time = 1.4
	
	-- Catapult.
	do
		local scenario_catapult = root:spawn( chunks.scenario.spawn() )
		local scenario_boulder  = root:spawn( chunks.scenario.spawn() )
		local spawn_catapult    = function( inner )
			local catapult = nil
		
			-- Cannon.
			do
				local base = scenario_catapult:spawn( ActorBase() )
				
				scenario_catapult.lua:scroll( -0.75, 1.0 )
				base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
				catapult      = base:spawn( chunks.catapult.spawn() )
			end
		
			-- Cannonballs.
			do
				local base = scenario_boulder:spawn( ActorBase() )
				
				scenario_boulder.lua:scroll( -0.75, 0.5 )
				base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
				catapult.lua:hard( base )
			end
		end

		-- Actions.
		do
			local s = SequenceAction()
			
			s:addAction( LuaAction( sleep_time, spawn_catapult ) )
			
			root:addAction( s )
		end
	end

	-- Dragon (use dragon timings).
	scenarios.dragon_hard( root )
	
	return { delay = 2.6 + sleep_time * 0.2, lifetime = 4.0 } 
end

-- Knight-mare.
function scenarios.knightmare( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -0.75, 1.0 )
		
	-- Left knight.
	do
		local knight = scenario:spawn( chunks.knight.spawn() )
		
		knight.lua:easy()
	end
	
	-- Right knight.
	do
		local base   = scenario:spawn( ActorBase() )
		local knight = base:spawn( chunks.knight.spawn() )
		
		base.location = pool.get_v3f( 0.0, 100.0, 0.0 )
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		knight.lua:easy()
	end
	
	return { delay = 1.3, lifetime = time }
end

-- Horsing around.
function scenarios.horsing_around( root )
	local scenario = root:spawn( chunks.scenario.spawn() )
	local time     = scenario.lua:scroll( -1.5, 1.0 )
	
	-- Left horse.
	do
		local horse = scenario:spawn( chunks.horse.spawn() )
		
		horse.lua:horsingAround()
	end
	
	-- Right horse.
	do
		local base  = scenario:spawn( ActorBase() )
		local horse = base:spawn( chunks.horse.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 175.0, 0.0 )
		horse.lua:horsingAround()
	end	
	
	-- Left horse.
	do
		local base  = scenario:spawn( ActorBase() )
		local horse = base:spawn( chunks.horse.spawn() )
		
		base.location = pool.get_v3f( 0.0, 175.0 * 2.0, 0.0 )
		horse.lua:horsingAround()
	end	
	
	-- Right horse.
	do
		local base  = scenario:spawn( ActorBase() )
		local horse = base:spawn( chunks.horse.spawn() )
		
		base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
		base.location = pool.get_v3f( 0.0, 175.0 * 3.0, 0.0 )
		horse.lua:horsingAround()
	end	
	
	return { delay = 2.0, lifetime = 3.0 }
end	

-- Gargizzle.
function scenarios.gargizzle( root )
	local spawn_gargizzle = function( scenario_gargoyle, scenario_frozen, flipped )
		local callback = nil
	
		-- Gargoyle.
		do
			local base     = scenario_gargoyle:spawn( ActorBase() )
			local gargoyle = base:spawn( chunks.gargoyle.spawn() )
			
			if flipped then
				base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
			end
			
			callback = function( instigator )
				gargoyle.lua:hard( instigator )
			end
		end
		
		-- Frozen.
		do
			local base            = scenario_frozen:spawn( chunks.scenario.spawn() )
			local gargoyle_frozen = base:spawn( chunks.gargoyle.spawn_frozen() )
			
			if flipped then
				base.rotation = pool.get_v3f( 0.0, 180.0, 0.0 )
			end
			
			scenario_frozen.lua:scroll( -0.75, 1.0 ) 
			gargoyle_frozen.lua:easy( callback )
		end
	end
	local ledge_scenario = root:spawn( chunks.scenario.spawn() )
	local spawn_ledge    = function( inner )
		ledge_scenario.lua:scroll( -0.75, 1.0 ) 
		ledge_scenario:spawn( chunks.drawbridge.spawn() )
	end
	
	local sleep_time = 0.6
	
	-- Spawn the gargoyles.
	spawn_gargizzle( root:spawn( chunks.scenario.spawn() ), root:spawn( chunks.scenario.spawn() ) )
	spawn_gargizzle( root:spawn( chunks.scenario.spawn() ), root:spawn( chunks.scenario.spawn() ), true )
	
	-- Wait a bit to spawn the second ones.
	do
		local s = SequenceAction()
		
		s:addAction( LuaAction( sleep_time, spawn_ledge ) )
		
		root:addAction( s )
	end
	
	return { delay = 1.7, lifetime = 3.3 }  
end

-- New high score.
function scenarios.new_high_score( root )
	local scenario       = root:spawn( chunks.scenario.spawn() )
	local time           = scenario.lua:scroll( -0.75, -55.0 )
	local new_high_score = scenario:spawn( chunks.new_high_score.spawn() )
	
	new_high_score.lua:celebrate()
	
	return { delay = 1.5, lifetime = time, z = -55.0 }
end

-----------------
-- transitions --
-----------------

transitions =
{
	-------------
	-- TESTING --
	-------------
	
	--[[
	{
		height    = 100000.0,
		scenarios =
		{
			{ prob = 0.5, f = scenarios.empty },
			{ prob = 0.5, f = scenarios.shieldpack },
			--{ prob = 0.5, f = scenarios.drawbridge },
			--{ prob = 0.5, f = scenarios.knight },
			--{ prob = 0.5, f = scenarios.horse },
			--{ prob = 0.5, f = scenarios.chain },
			--{ prob = 0.5, f = scenarios.troll_easy },
			--{ prob = 0.5, f = scenarios.troll_hard },
			--{ prob = 0.5, f = scenarios.archer_easy },
			--{ prob = 0.5, f = scenarios.archer_hard },
			--{ prob = 0.5, f = scenarios.gargoyle_easy },
			--{ prob = 0.5, f = scenarios.gargoyle_hard },
			--{ prob = 0.5, f = scenarios.catapult_easy },
			--{ prob = 0.5, f = scenarios.catapult_hard },
			--{ prob = 0.5, f = scenarios.dragon_easy },
			--{ prob = 0.5, f = scenarios.dragon_hard },
			--{ prob = 0.5, f = scenarios.dragon_cheap_shot },
			--{ prob = 0.5, f = scenarios.knightmare },
			--{ prob = 0.5, f = scenarios.gargizzle },
			--{ prob = 0.5, f = scenarios.horsing_around },
			--{ prob = 0.5, f = scenarios.dragon_cheaper_shot },
		}
	},
	--]]

	----------------
	-- PRODUCTION --
	----------------
	
	-- At the very beginning, spawn some empty and chain.
	{
		height    = 102.0,
		scenarios =
		{
			{ prob = 0.8, f = scenarios.empty },
			--{ prob = 0.5, f = scenarios.shieldpack },
			{ prob = 0.3, f = scenarios.chain },
			--{ prob = 0.5, f = scenarios.drawbridge },
			--{ prob = 0.5, f = scenarios.knight },
			--{ prob = 0.5, f = scenarios.horse },
			--{ prob = 0.5, f = scenarios.troll_easy },
			--{ prob = 0.5, f = scenarios.troll_hard },
			--{ prob = 0.5, f = scenarios.archer_easy },
			--{ prob = 0.5, f = scenarios.archer_hard },
			--{ prob = 0.5, f = scenarios.gargoyle_easy },
			--{ prob = 0.5, f = scenarios.gargoyle_hard },
			--{ prob = 0.5, f = scenarios.catapult_easy },
			--{ prob = 0.5, f = scenarios.catapult_hard },
			--{ prob = 0.5, f = scenarios.dragon_easy },
			--{ prob = 0.5, f = scenarios.dragon_hard },
			--{ prob = 0.5, f = scenarios.dragon_cheap_shot },
			--{ prob = 0.5, f = scenarios.knightmare },
			--{ prob = 0.5, f = scenarios.gargizzle },
			--{ prob = 0.5, f = scenarios.horsing_around },
			--{ prob = 0.5, f = scenarios.dragon_cheaper_shot },
		}
	},
	
	-- Introduce knight, easy catapult, easy troll.  Also the shieldpack.
	{
		height    = 1122.0,
		scenarios =
		{
			{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			{ prob = 0.3, f = scenarios.chain },
			--{ prob = 0.5, f = scenarios.drawbridge },
			{ prob = 0.2, f = scenarios.knight },
			--{ prob = 0.5, f = scenarios.horse },
			{ prob = 0.2, f = scenarios.troll_easy },
			--{ prob = 0.5, f = scenarios.troll_hard },
			--{ prob = 0.5, f = scenarios.archer_easy },
			--{ prob = 0.5, f = scenarios.archer_hard },
			--{ prob = 0.5, f = scenarios.gargoyle_easy },
			--{ prob = 0.5, f = scenarios.gargoyle_hard },
			{ prob = 0.2, f = scenarios.catapult_easy },
			--{ prob = 0.5, f = scenarios.catapult_hard },
			--{ prob = 0.5, f = scenarios.dragon_easy },
			--{ prob = 0.5, f = scenarios.dragon_hard },
			--{ prob = 0.5, f = scenarios.dragon_cheap_shot },
			--{ prob = 0.5, f = scenarios.knightmare },
			--{ prob = 0.5, f = scenarios.gargizzle },
			--{ prob = 0.5, f = scenarios.horsing_around },
			--{ prob = 0.5, f = scenarios.dragon_cheaper_shot },
		}
	},
	
	-- Introduce drawbridge, archer.
	{
		height    = 2142.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.chain },
			{ prob = 0.4, f = scenarios.drawbridge },
			{ prob = 0.3, f = scenarios.knight },
			--{ prob = 0.5, f = scenarios.horse },
			{ prob = 0.3, f = scenarios.troll_easy },
			--{ prob = 0.5, f = scenarios.troll_hard },
			{ prob = 0.4, f = scenarios.archer_easy },
			--{ prob = 0.5, f = scenarios.archer_hard },
			--{ prob = 0.5, f = scenarios.gargoyle_easy },
			--{ prob = 0.5, f = scenarios.gargoyle_hard },
			{ prob = 0.3, f = scenarios.catapult_easy },
			--{ prob = 0.5, f = scenarios.catapult_hard },
			--{ prob = 0.5, f = scenarios.dragon_easy },
			--{ prob = 0.5, f = scenarios.dragon_hard },
			--{ prob = 0.5, f = scenarios.dragon_cheap_shot },
			--{ prob = 0.5, f = scenarios.knightmare },
			--{ prob = 0.5, f = scenarios.gargizzle },
			--{ prob = 0.5, f = scenarios.horsing_around },
			--{ prob = 0.5, f = scenarios.dragon_cheaper_shot },
		}
	},
	
	-- Introduce gargoyles, introduce rare dragons.
	{
		height    = 3162.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.chain },
			{ prob = 0.3, f = scenarios.drawbridge },
			{ prob = 0.2, f = scenarios.knight },
			{ prob = 0.4, f = scenarios.horse },
			{ prob = 0.3, f = scenarios.troll_easy },
			--{ prob = 0.5, f = scenarios.troll_hard },
			{ prob = 0.3, f = scenarios.archer_easy },
			--{ prob = 0.5, f = scenarios.archer_hard },
			{ prob = 0.4, f = scenarios.gargoyle_easy },
			--{ prob = 0.5, f = scenarios.gargoyle_hard },
			{ prob = 0.3, f = scenarios.catapult_easy },
			--{ prob = 0.5, f = scenarios.catapult_hard },
			{ prob = 0.1, f = scenarios.dragon_easy },
			--{ prob = 0.5, f = scenarios.dragon_hard },
			--{ prob = 0.5, f = scenarios.dragon_cheap_shot },
			--{ prob = 0.5, f = scenarios.knightmare },
			--{ prob = 0.5, f = scenarios.gargizzle },
			--{ prob = 0.5, f = scenarios.horsing_around },
			--{ prob = 0.5, f = scenarios.dragon_cheaper_shot },
		}
	},
	
	-- Introduce hard enemies, partially.
	{
		height    = 4182.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.chain },
			{ prob = 0.3, f = scenarios.drawbridge },
			{ prob = 0.2, f = scenarios.knight },
			{ prob = 0.3, f = scenarios.horse },
			{ prob = 0.1, f = scenarios.troll_easy },
			{ prob = 0.2, f = scenarios.troll_hard },
			{ prob = 0.1, f = scenarios.archer_easy },
			{ prob = 0.2, f = scenarios.archer_hard },
			{ prob = 0.1, f = scenarios.gargoyle_easy },
			{ prob = 0.2, f = scenarios.gargoyle_hard },
			{ prob = 0.1, f = scenarios.catapult_easy },
			{ prob = 0.2, f = scenarios.catapult_hard },
			{ prob = 0.3, f = scenarios.dragon_easy },
			--{ prob = 0.5, f = scenarios.dragon_hard },
			--{ prob = 0.5, f = scenarios.dragon_cheap_shot },
			--{ prob = 0.5, f = scenarios.knightmare },
			--{ prob = 0.5, f = scenarios.gargizzle },
			--{ prob = 0.5, f = scenarios.horsing_around },
			--{ prob = 0.5, f = scenarios.dragon_cheaper_shot },
		}
	},
	
	-- Introduce all hard enemies, more dragon.
	{
		height    = 5202.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.chain },
			{ prob = 0.3, f = scenarios.drawbridge },
			{ prob = 0.2, f = scenarios.knight },
			{ prob = 0.3, f = scenarios.horse },
			--{ prob = 0.1, f = scenarios.troll_easy },
			{ prob = 0.3, f = scenarios.troll_hard },
			--{ prob = 0.1, f = scenarios.archer_easy },
			{ prob = 0.3, f = scenarios.archer_hard },
			--{ prob = 0.1, f = scenarios.gargoyle_easy },
			{ prob = 0.3, f = scenarios.gargoyle_hard },
			--{ prob = 0.1, f = scenarios.catapult_easy },
			{ prob = 0.3, f = scenarios.catapult_hard },
			{ prob = 0.4, f = scenarios.dragon_easy },
			--{ prob = 0.5, f = scenarios.dragon_hard },
			--{ prob = 0.5, f = scenarios.dragon_cheap_shot },
			--{ prob = 0.5, f = scenarios.knightmare },
			--{ prob = 0.5, f = scenarios.gargizzle },
			--{ prob = 0.5, f = scenarios.horsing_around },
			--{ prob = 0.5, f = scenarios.dragon_cheaper_shot },
		}
	},
	
	-- Introduce hard dragon, dragon cheap shot, knightmare.
	{
		height    = 6222.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.chain },
			{ prob = 0.3, f = scenarios.drawbridge },
			{ prob = 0.1, f = scenarios.knight },
			{ prob = 0.3, f = scenarios.horse },
			--{ prob = 0.1, f = scenarios.troll_easy },
			{ prob = 0.3, f = scenarios.troll_hard },
			--{ prob = 0.1, f = scenarios.archer_easy },
			{ prob = 0.3, f = scenarios.archer_hard },
			--{ prob = 0.1, f = scenarios.gargoyle_easy },
			{ prob = 0.3, f = scenarios.gargoyle_hard },
			--{ prob = 0.1, f = scenarios.catapult_easy },
			{ prob = 0.3, f = scenarios.catapult_hard },
			{ prob = 0.1, f = scenarios.dragon_easy },
			{ prob = 0.2, f = scenarios.dragon_hard },
			{ prob = 0.2, f = scenarios.dragon_cheap_shot },
			{ prob = 0.2, f = scenarios.knightmare },
			--{ prob = 0.5, f = scenarios.gargizzle },
			--{ prob = 0.5, f = scenarios.horsing_around },
			--{ prob = 0.5, f = scenarios.dragon_cheaper_shot },
		}
	},
	
	-- Introduce horsing around.
	{
		height    = 7242.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.chain },
			{ prob = 0.3, f = scenarios.drawbridge },
			{ prob = 0.1, f = scenarios.knight },
			{ prob = 0.3, f = scenarios.horse },
			--{ prob = 0.1, f = scenarios.troll_easy },
			{ prob = 0.3, f = scenarios.troll_hard },
			--{ prob = 0.1, f = scenarios.archer_easy },
			{ prob = 0.3, f = scenarios.archer_hard },
			--{ prob = 0.1, f = scenarios.gargoyle_easy },
			{ prob = 0.3, f = scenarios.gargoyle_hard },
			--{ prob = 0.1, f = scenarios.catapult_easy },
			{ prob = 0.3, f = scenarios.catapult_hard },
			{ prob = 0.1, f = scenarios.dragon_easy },
			{ prob = 0.2, f = scenarios.dragon_hard },
			{ prob = 0.2, f = scenarios.dragon_cheap_shot },
			{ prob = 0.2, f = scenarios.knightmare },
			--{ prob = 0.5, f = scenarios.gargizzle },
			{ prob = 0.2, f = scenarios.horsing_around },
			--{ prob = 0.5, f = scenarios.dragon_cheaper_shot },
		}
	},
	
	-- Introduce gargizzle, dragon cheaper shot.
	{
		height    = 8262.0,
		scenarios =
		{
			--{ prob = 0.3, f = scenarios.empty },
			{ prob = 0.2, f = scenarios.shieldpack },
			--{ prob = 0.3, f = scenarios.chain },
			{ prob = 0.3, f = scenarios.drawbridge },
			{ prob = 0.1, f = scenarios.knight },
			{ prob = 0.3, f = scenarios.horse },
			--{ prob = 0.1, f = scenarios.troll_easy },
			{ prob = 0.3, f = scenarios.troll_hard },
			--{ prob = 0.1, f = scenarios.archer_easy },
			{ prob = 0.3, f = scenarios.archer_hard },
			--{ prob = 0.1, f = scenarios.gargoyle_easy },
			{ prob = 0.3, f = scenarios.gargoyle_hard },
			--{ prob = 0.1, f = scenarios.catapult_easy },
			{ prob = 0.3, f = scenarios.catapult_hard },
			{ prob = 0.1, f = scenarios.dragon_easy },
			{ prob = 0.2, f = scenarios.dragon_hard },
			{ prob = 0.2, f = scenarios.dragon_cheap_shot },
			{ prob = 0.2, f = scenarios.knightmare },
			{ prob = 0.2, f = scenarios.gargizzle },
			{ prob = 0.2, f = scenarios.horsing_around },
			{ prob = 0.2, f = scenarios.dragon_cheaper_shot },
		}
	},
}	
