------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Shield             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local pawn      = chunks.pawn
local textures  = chunks.textures

-- LuaShield --
LuaShield = chunks.class.inherit( pawn.LuaPawn )
{
	equipped = false
}

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaShield )

-- The created method.
function LuaShield:created( owner )
	-- Initialize super.
	pawn.LuaPawn.created( self, owner )
	
	-- Initilize owner.
	owner.location = pool.get_v3f( 0.0, 0.0, 1.0 )
	
	-- Balcony.
	do
		local actor = owner:spawn( SpriteActor() )
		
		actor.scale = pool.get_v3f( 0.0, 0.0, 1.0 )
		
		textures.setTexture( actor, assets.textures.shield )

		self.visuals.shield = actor
	end
	
	self:setVisual( "shield" )
end

-- Equip the shield.
function LuaShield:equip()
	-- If we already have the shield equipped, exit.
	if self.equipped then
		return false
	else
		self.equipped = true
	end
	
	local actor = self.visual
	
	-- Actions.
	do
		local s     = SequenceAction()
		local l     = LoopAction()
		local split = SplitAction()
		local s1    = SequenceAction()
		local s2    = SequenceAction()
		
		s:addAction( ScaleAction( 0.0, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
		s:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.65, 0.65, 1.0 ) ) )
		s1:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.5, 0.5, 1.0 ) ) )
		s1:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.65, 0.65, 1.0 ) ) )
		s2:addAction( RotateOffsetAction( 0.4, InterpAction.Lerp, pool.get_v3f( 0.0, 0.0, 180.0 ) ) )
		s2:addAction( RotateOffsetAction( 0.4, InterpAction.Lerp, pool.get_v3f( 0.0, 0.0, 180.0 ) ) )
		split:addAction( s1 )
		split:addAction( s2 )
		l:addAction( split )
		s:addAction( l )
		
		actor:clearActions()
		actor:addAction( s )
	end
	
	return true
end

-- Lose the shield.
function LuaShield:lose()
	local actor = self.visual
	local s     = SequenceAction()
	
	s:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
		
	actor:clearActions()
	actor:addAction( s )
	
	self.equipped = false
end	

-- Start bonus.
function LuaShield:bonusStart( scale )
	local actor = self.owner
	
	-- Actions.
	do
		local s = SequenceAction()
		
		s:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( scale, scale, 1.0 ) ) )
		
		actor:clearActions()
		actor:addAction( s )
	end
end

-- Stop bonus.
function LuaShield:bonusStop()
	local actor = self.owner
	
	-- Actions.
	do
		local s = SequenceAction()
		
		s:addAction( ScaleAction( 0.4, InterpAction.Smooth, pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
		
		actor:clearActions()
		actor:addAction( s )
	end
end

--These methods are exclusively used to support the IAP shield system

--Check that the shield is not already equipped
function LuaShield:canEquip()
	return not self.equipped
end

-- Equip the shield without any visual effects, for while the flying shield is in transit.
function LuaShield:silentEquip()
	self.equipped = true
end

-- Unequip the shield without any visual effects, for while the flying shield is in transit.
function LuaShield:silentUnequip()
	self.equipped = false
end

