-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local assets_shared    = chunks.assets_shared
local globals   = chunks.globals
local pool      = chunks.pool
local util      = chunks.utility
local textures = chunks.textures
local events = chunks.events

local white = colorf(1, 1, 1, 1)
local active_color = colorf(1.0, 0.89, 0.28, 1.0)
local inactive_color = colorf(0.75, 0.75, 0.75, 1.0)
local current_wave = 0
local scroll_view_inset = v2f(1, 4)

LuaShieldInfo = chunks.class.new
{
	m_selection_box = nil,
	m_popup_time = 0.3,
	m_prices = nil,
	m_info_window = nil,
	m_spawning_ground = nil,
	m_shield_meter = nil,
	m_window = nil,
	m_overlay_window = nil,
	m_spawned_from_main_menu = nil,
	m_store_button = nil,
	m_store_text = nil,
	m_flying_shield = nil
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaShieldInfo )

function LuaShieldInfo:created(window)
	
	
				
	self.m_window = window

	local overlay_window = window:spawn( Window() )
	overlay_window.size = constants.scene.world_size
	overlay_window.receive_input = true
	overlay_window.location = pool.get_v3f(0, 0, 50)
	overlay_window.color = pool.get_colorf(0, 0, 0, 0)
    overlay_window.lua   = {}
	overlay_window:addAction(AlphaAction(self.m_popup_time, InterpAction.Smooth, 0.75))
	self.m_overlay_window = overlay_window
	
	--Blank Window from which to spawn the store, for input issues
	local spawning_ground = window:spawn( Window() )
	spawning_ground.size = pool.get_v2f(0, 0)
	spawning_ground.location = pool.get_v3f(0, 0, 60)
	self.m_spawning_ground = spawning_ground


	--The background window
	local info_window = window:spawn( Window() )
	textures.setTexture( info_window, assets.textures.shieldstore_popup )
	info_window.force_color = true
	info_window.receive_input = true
	info_window.color = pool.get_colorf(1,1,1,1)
	info_window.location = pool.get_v3f(0, -constants.scene.world_size.y , 52)
	info_window.scale = pool.get_v3f(1, 1, 1)
	info_window:addAction(MoveOffsetAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, constants.scene.world_size.y, 0)))
	info_window:addAction(LuaAction(0, function() self:animateFreeShields() end))
	self.m_info_window = info_window
	
	if not engine.default_language_enabled then
		local format = {"ninja-text", 30.0, pool.get_colorf(0.86, 0.98, 1.0, 1), 100, TextAlign.Center, -10, 106}

		local label = info_window:spawn(Text())	
		label.style = format[1]
		label.options = {height = format[2], max_width = format[4] or 0.0, align = format[5], char_spacing = format[8] or 0} 	 	
 		label.color = format[3]

		local text_offset_x = format[6] or 0.0
		local text_offset_y = format[7] or 0.0
 	
 		label.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(text_offset_x, text_offset_y, 55) }
 	
		label.text = lstring("shields")
	
		local fx = {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}
		label.fx = fx;

	else
		local upgrade_overlay = info_window:spawn( Window() )
		textures.setTexture( upgrade_overlay, assets.textures.shieldstore_popup_locale )
		upgrade_overlay.force_color = true
		upgrade_overlay.color = pool.get_colorf(1,1,1,1)
		upgrade_overlay.location = pool.get_v3f(-12, 106, 54)
		upgrade_overlay.scale = pool.get_v3f(1, 1, 1)
	end
	
	--Back button
	local back_button = util.createButtonWithTexture(info_window, assets_shared.textures.exit_button)
	back_button.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_TR, pool.get_v3f(-20, -24, 53))
	back_button.on_release = function(owner)
		--assets.playSound(assets.sounds.button_click_wav)
		self:dismiss()
	end
	
	local format = {"sans-text", 20.0, pool.get_colorf(1, 1, 1, 1), 300, TextAlign.Center, 0, 0}
	
	
	if (engine.locale_language == "ja" or engine.locale_language == "ko") then

        local button_x
        local line_2_b_x
        
        if(engine.locale_language == "ja")then
            button_x = -40
            line_2_b_x = 45
        else
            button_x = -40
            line_2_b_x = 45
        end

	    local x_to_max_width = 45.0
	
    	local fx = {drop_shadow = false}

	    format[4] = 120.0 -- max_width
    	local line_1 = util.createText( info_window, lstring("asian_shield_info_line_1"), format, fx )
	    line_1.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 65, 3))	

	    --pic of the shield button
    	local button = info_window:spawn(Window())	
	    button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( button_x, 35.0, 3.0 ) )		
    	textures.setTexture( button, assets_shared.textures.shield_button )

	    --Shield count in ticker form
    	local shield_ticker = button:spawn( chunks.ticker.spawn() )		
	    shield_ticker.location = pool.get_v3f( 11.0, 0.0, 1.0 )
    	shield_ticker.scale    = pool.get_v3f( 0.7, 0.7, 1.0 )
	    local shields = UserData.get():getShields()  
    	if(shields > 9999) then shields = 9999 end
	    if(shields > 0) then
	        shield_ticker.value    = shields - 1
    	else
	        shield_ticker.value    = shields
	    end
	    self.m_shield_meter = shield_ticker

    	format[4] = 180.0 -- max_width
	    local line_2b = util.createText( info_window, lstring("asian_shield_info_line_2_b"), format, fx )
    	line_2b.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(line_2_b_x, 35, 3))	
	else
	    --Explanatory text, lines 1-4

	    
	    -- Because of the placement of the shields button in the middle of the text we need to switch
	    -- how text is displayed based on language
	    local x_anchor_button_offset = 10.0
	    local x_anchor_text = 75
	    local x_to_max_width = 45.0
        if engine.locale_language == "de" then
            format[2] = 12.0
        end
    	if engine.locale_language == "es" then
            format[2] = 20.0
        end
	
    	local fx = {drop_shadow = false}
	
	    format[4] = 60.0 -- max_width
    	local line_1 = util.createText( info_window, lstring("tap"), format, fx )
	    line_1.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(-65, 65, 3))	
	
    	format[4] = x_to_max_width -- max_width
	    local line_1b = util.createText( info_window, lstring("to"), format, fx )
    	line_1b.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(x_anchor_text, 65, 3))
	
	    --pic of the shield button
    	local button = info_window:spawn(Window())	
	    button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( x_anchor_button_offset, 65.0, 3.0 ) )		
    	textures.setTexture( button, assets_shared.textures.shield_button )
	
	    --Shield count in ticker form
    	local shield_ticker = button:spawn( chunks.ticker.spawn() )		
	    shield_ticker.location = pool.get_v3f( 11.0, 0.0, 1.0 )
    	shield_ticker.scale    = pool.get_v3f( 0.7, 0.7, 1.0 )
	    local shields = UserData.get():getShields()  
    	if(shields > 9999) then shields = 9999 end
	    if(shields > 0) then
	        shield_ticker.value    = shields - 1
    	else
	        shield_ticker.value    = shields
	    end
	    self.m_shield_meter = shield_ticker

    	format[4] = 180.0 -- max_width
	    local line_2 = util.createText( info_window, lstring("use_a_shield"), format, fx )
    	line_2.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 35, 3))	
	end
	
	-- The item store button.
	if not GameService.get():zeb() then
	
		-- leave out mentions of the store in zeb builds
		local line_3 = util.createText( info_window, lstring("increase_shields"), format, fx )
		line_3.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 5, 3))	
	
		local line_4 = util.createText( info_window, lstring("per_game_in_the"), format, fx )
		line_4.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, -25, 3))	
	
		local line_5 = util.createText( info_window, lstring("shield_store"), format, fx )
		line_5.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, -55, 3))	
	
		do
			local on_release = function( inner )
			    print("Opening item store")	
				self:openItemStore()
			end	
			
			print("Creating item store button")
			
			local button = nil
			if engine.default_language_enabled then
				button = util.createButtonWithTexture(info_window, assets_shared.textures.item_store)	
			else
				local shields_text = lstring( "shields" )
				local format = {"ninja-text", 40.0, pool.get_colorf(0.86, 0.98, 1.0, 1.0), 105.0, TextAlign.Center, 0.0, 6.0}
				local fx = {drop_shadow = true, shadow_color = colorf( 0.0, 0.27, 0.48, 1.0 ), shadow_offset = v3f( 1, -1, 0), shadow_blur_amount = 100000}
				button, label = util.createButtonWithTextAndTexture(info_window, shields_text, format, assets.textures.item_store_locale, fx )
			    self.m_store_text = label
			end
			button.on_release = on_release
			button.anchor = make_anchor( Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -95.0, 60.0 ) )
	        self.m_store_button = button
		end
	end

	local input_block = window:spawn( Button() )
	textures.setTexture( input_block, assets.textures.shieldstore_popup )
	input_block.force_color = true
	input_block.color = pool.get_colorf(1,1,1,0)
	input_block.location = pool.get_v3f(0, 0 , 100)
	input_block.scale = pool.get_v3f(1, 1, 1)
	
	--This overlay button prevents input from leaking to windows underneath the shield store, also causes a tap
	--outside of the menu region to dismiss it.
	local overlay_window = window:spawn( Button() )
	overlay_window.size = constants.scene.world_size
	overlay_window.location = pool.get_v3f(0, 0, 50)
	overlay_window.color = pool.get_colorf(0, 0, 0, 0)
	overlay_window.on_release = function(owner)
		self:dismiss()
	end
	
	UserData.get():setShields(UserData.get():getFreeShieldCapacity() + UserData.get():getShieldCapacity())
	UserData.get():setFreeShieldsIssued()
	
	
end

function LuaShieldInfo:setSpawnedFromMainMenu()
	self.m_overlay_window.lua.m_spawned_from_main_menu = true
	self.m_store_button.hidden = true
	events.on_shield_store_appeared()
	if(self.m_store_text) then
	    self.m_store_text.hidden = true
	end
end

function LuaShieldInfo:dismiss(delay)
	delay = delay or 0
	self.m_info_window:addAction(SleepAction(delay))
	self.m_info_window:addAction(MoveOffsetAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, -constants.scene.world_size.y, 0)))		
	self.owner:addAction(SleepAction(delay))
	self.owner:addAction(AlphaAction(self.m_popup_time, InterpAction.Smooth, 0))
	self.owner:addAction(DestroyAction(0))
	
	
	if(self.m_overlay_window.lua.m_spawned_from_main_menu) then
		self.owner:getOwner():getOwner().lua:openItemStore()
	else
		self.owner:getOwner():getGame().lua:forceUnpause()
	end

end

function LuaShieldInfo:openItemStore()
    if chunks.iap.purchasingAvailable() then   
	    local shield_store = self.m_spawning_ground:spawn( chunks.shield_store.spawn() )
	    self.m_overlay_window:addAction(AlphaAction(self.m_popup_time, InterpAction.Smooth, 0.0))
	    shield_store.lua:setSpawnedFromInfoMenu()
	else
        chunks.iap.displayPurchasingUnavailable()
    end
end


function LuaShieldInfo:animateFreeShields()

	
	local destination = v3f(-16.0, 63.0, 200)
	
	if(not self.m_overlay_window.lua.m_spawned_from_main_menu) then
		local flying_shield = self.m_window:spawn( Window() )
		flying_shield.location = pool.get_v3f(-41, -79, 200)
		flying_shield.scale    = pool.get_v3f( 0.52, 0.52, 1.0 )
		textures.setTexture( flying_shield, assets.textures.shieldpack )

	
		local fly = SplitAction() 
		fly:addAction( ScaleAction( 0.5, InterpAction.Ramp, pool.get_v3f( 0.4, 0.4, 1.0 ) ) )
		fly:addAction( MoveAction( 0.5, InterpAction.Smooth, destination ) )	
		flying_shield:addAction(fly)
		flying_shield:addAction( DestroyAction(0.0) )
		flying_shield:addAction( LuaAction(0.0, function() 
													if(self.m_shield_meter.value < 9999) then
														self.m_shield_meter.value = self.m_shield_meter.value + 1 
													end
												end) )
	else
		self.m_shield_meter.value = self.m_shield_meter.value + 1 
	end

end



function LuaShieldInfo:restoreOverlay()
	self.m_overlay_window:addAction(AlphaAction(self.m_popup_time, InterpAction.Smooth, 0.75))
	self.m_shield_meter.value = UserData.get():getShields()
end

