------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Shieldpack         --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaShieldPack --
LuaShieldPack = chunks.class.inherit( enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaShieldPack )

-- The created method.
function LuaShieldPack:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "shieldpack"
	owner.location = pool.get_v3f( 110.0, 0.0, 0.0 )
	
	-- Balcony.
	do
		local actor = owner:spawn( CollidableActor() )
		
		actor.scale = pool.get_v3f( 0.65, 0.65, 1.0 )
		
		textures.setTexture( actor, assets.textures.shieldpack )

		self.visuals.shieldpack = actor
	end
	
	self:setVisual( "shieldpack" )
end

-- The collided event.
function LuaShieldPack:onCollided( other )
	-- Spawn particles.
	do
		particles.spawn_normal( self.owner:getGame().lua.root, 
		                        assets.textures.particles_shield, 
		                        self.visual:getWorldLocation() )
	end
	
	actions.shrink_and_kill( self.visual )

	-- Play sound.
	sound.play( assets.sounds.shieldpack ) 
end
