-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local assets_shared    = chunks.assets_shared
local globals   = chunks.globals
local pool      = chunks.pool
local util      = chunks.utility
local textures = chunks.textures
local events = chunks.events

local white = colorf(1, 1, 1, 1)
local active_color = colorf(1.0, 0.89, 0.28, 1.0)
local inactive_color = colorf(0.75, 0.75, 0.75, 1.0)
local current_wave = 0
local scroll_view_inset = v2f(1, 4)

LuaShieldStore = chunks.class.new
{
	m_selection_box = nil,
	m_popup_time = 0.3,
	m_prices = nil,
	m_upgrade_window = nil,
	m_immediate_use = nil,
	m_force_immediate_use = nil,
	m_use_now_menu = nil,
	m_overlay_window = nil,
	m_window = nil,
	m_shield_meter = nil,
    m_dismissed = false,
    m_iap_handle = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaShieldStore )

function LuaShieldStore:created(window)

	self.m_window = window

	self.m_immediate_use = false


	--The background window
	local upgrade_window = window:spawn( Window() )
	textures.setTexture( upgrade_window, assets.textures.shieldstore_popup )
	upgrade_window.force_color = true
	upgrade_window.color = pool.get_colorf(1,1,1,1)
	upgrade_window.location = pool.get_v3f(0, -constants.scene.world_size.y , 52)
	upgrade_window.scale = pool.get_v3f(1, 1, 1)
	upgrade_window:addAction(MoveOffsetAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, constants.scene.world_size.y, 0)))
	self.m_upgrade_window = upgrade_window
	
	if not engine.default_language_enabled then
		local format = {"ninja-text", 30.0, pool.get_colorf(0.86, 0.98, 1.0, 1), 100, TextAlign.Center, -10, 106}

		local label = upgrade_window:spawn(Text())	
		label.style = format[1]
		label.options = {height = format[2], max_width = format[4] or 0.0, align = format[5], char_spacing = format[8] or 0} 	 	
 		label.color = format[3]

		local text_offset_x = format[6] or 0.0
		local text_offset_y = format[7] or 0.0
 	
 		label.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(text_offset_x, text_offset_y, 55) }
 	
		label.text = lstring("shields")
	
		local fx = {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}
		label.fx = fx;

	else
		local upgrade_overlay = upgrade_window:spawn( Window() )
		textures.setTexture( upgrade_overlay, assets.textures.shieldstore_popup_locale )
		upgrade_overlay.force_color = true
		upgrade_overlay.color = pool.get_colorf(1,1,1,1)
		upgrade_overlay.location = pool.get_v3f(-12, 106, 54)
		upgrade_overlay.scale = pool.get_v3f(1, 1, 1)
	end

	--Back button
	local back_button = util.createButtonWithTexture(upgrade_window, assets_shared.textures.exit_button)
	back_button.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_TR, pool.get_v3f(-20, -24, 53))
	back_button.on_release = function(owner)
		self:dismiss()
	end
	
	--The "Use shield now?" menu.  Spawns off screen
	local use_now = window:spawn( Window() )
	textures.setTexture( use_now, assets.textures.use_now )
	use_now.location = pool.get_v3f(0, -constants.scene.world_size.y , 52)
	self.m_use_now_menu = use_now
	
	local format = {"sans-text", 18.0, pool.get_colorf(1, 1, 1, 1), 100, TextAlign.Center, 0, 0}
	local fx = {drop_shadow = false}
	
	local line_1 = util.createText( use_now, lstring("use_shield"), format, fx )
	local line_2 = util.createText( use_now, lstring("immediately"), format, fx )
	
	line_1.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 10, 3))			
	line_2.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, -10, 3))
	
	-- The shield display
	
		
			local display = upgrade_window:spawn( Window() )		
			display.location = pool.get_v3f( 0.0, -140.0, 100.0 ) 	
			textures.setTexture( display, assets.textures.shield_button )
			display.scale = pool.get_v3f(1, -1, 1)
							
			local format = TextFormat("font://ohitashi_rg.ttf", 22.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))	
	
			--Shield count in ticker form
			local shield_ticker = display:spawn( chunks.ticker.spawn() )		
			shield_ticker.location = pool.get_v3f( 11.0, 1.0, 1.0 )
			shield_ticker.scale    = pool.get_v3f( 0.7, -0.7, 1.0 )
			shield_ticker.lua.delay = 0
			shield_ticker.value    = UserData.get():getShieldCapacity() + UserData.get():getFreeShieldCapacity()

		
			
		
			self.m_shield_meter = shield_ticker	
	--Yes button
	local yes_button = nil
	if engine.default_language_enabled then
		yes_button = util.createButtonWithTexture(use_now, assets_shared.textures.yes)
	else
		local text = lstring( "yes" )
		local format = {"ninja-text-large", 52.0 * 1.6, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 100.0, TextAlign.Center, 0.0, 25.0}
		yes_button = util.createButtonWithTextAndTexture( use_now, text, format, assets_shared.textures.yes_locale )
	end

	yes_button.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(35, -75, 53))
	yes_button.scale = pool.get_v3f(0.65, 0.65, 1.0)

	yes_button.on_release = function(owner)
		local activated_from_store = true
		self.owner:getOwner():getGame().lua:getHud().lua:activateShield(activated_from_store)
		self:dismiss()
	end
	
	--No button
	local no_button = nil
	if engine.default_language_enabled then
		no_button = util.createButtonWithTexture(use_now, assets_shared.textures.no)
	else
		local text = lstring( "no" )
		local format = {"ninja-text", 42.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 50.0, TextAlign.Center, 0.0, 10.0}
		no_button = util.createButtonWithTextAndTexture( use_now, text, format, assets_shared.textures.no_locale )
	end

	no_button.anchor = make_anchor(Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(-45, -75, 53))
	no_button.on_release = function(owner)
		self:dismiss()
	end
	
	local format = {"sans-text", 36.0, pool.get_colorf(1, 1, 1, 1), 300, TextAlign.Center, 0, 0}
	local fx = {drop_shadow = false}
	
	local info = util.createText( upgrade_window, lstring("increase_shields_per_game"), format, fx )
	info.anchor = make_anchor(Anchor.Anchor_B, Anchor.Anchor_B, pool.get_v3f(0, -60, 3))	
	
	
	
	--local format = TextFormat("font://Helvetica", 36.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
	--info.format = format
	--info.scale = pool.get_v3f(0.5, 0.5, 1)
	--info.text = "Increase shields per game!"	
	
    local format = TextFormat("font://ohitashi_rg.ttf", 50.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
    -- local price_format = TextFormat("font://ohitashi_rg.ttf", 40.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))

    local text_objs = {}
	for i, id in ipairs(chunks.iap.PRODUCT_DISPLAY_ORDER) do
        local item = chunks.iap.PRODUCTS[id]
		
		local shields = item.value
		local price = chunks.iap.getPriceString(id)
		print("Creating IAP button ", i, "with ID ", id, " with price ", price)
		
		local asset = assets.textures.shieldstore_button
		local button = util.createButtonWithTexture(upgrade_window, asset)
		button.anchor = make_anchor(Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(-1, -45-asset.size.y*(i-1), 5))

		local shield_icon = util.createWindowWithTexture(button, assets.textures.shieldpack)
		shield_icon.anchor = make_anchor(Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(6, 1.5, 5))
		shield_icon.size = shield_icon.size * 0.45
		
		--Shield count in ticker form
		local plus_sign = shield_icon:spawn( Label() )
		plus_sign.location = pool.get_v3f( 20.0, 0.0, 1.0 )
		plus_sign.format = format
		plus_sign.text = "+"
		plus_sign.scale = pool.get_v3f(0.4, 0.4, 1)
		
		local shield_ticker = shield_icon:spawn( chunks.ticker.spawn() )
		
		--in order to get the shield numbers to appear to be left-justified, i am hardcoding
		-- these displacement values.  the nj_ticker class does not seem to have the ability to do anything 
		--other than center-justify
		
		if(i==1) then
			shield_ticker.location = pool.get_v3f( 34.0, 0.0, 1.0 )
		elseif(i==2) then
			shield_ticker.location = pool.get_v3f( 34.0, 0.0, 1.0 )
		elseif(i==3) then
			shield_ticker.location = pool.get_v3f( 39.0, 0.0, 1.0 )
		elseif(i==4) then
			shield_ticker.location = pool.get_v3f( 39.0, 0.0, 1.0 )
		elseif(i==5) then
			shield_ticker.location = pool.get_v3f( 44.0, 0.0, 1.0 )
		end
		
		
		shield_ticker.scale    = pool.get_v3f( 0.8, 0.8, 1.0 )
		shield_ticker.value    = (shields)
		
		local price_label = button:spawn(Text())

		price_label.anchor = make_anchor(Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(-10, 4, 3))
		--price_label.format = price_format
		price_label.style = "currency"
        price_label.options = {height = 40, max_width = 180}
        price_label.text = lstring(price)
		price_label.scale = pool.get_v3f(0.5, 0.5, 1)
		price_label.color = pool.get_colorf(1.0, 1.0, 1.0, 1.0)
        table.insert(text_objs, price_label)

		button.on_release = function(inner)
            local response_complete_callback = function()
                self.m_iap_handle = nil
			end

            local product_consumed_callback = function()
				if(self.m_immediate_use and not window:getOwner():getGame().lua:getNinja().lua:getShield().lua.equipped) then
					self:askForImmediateUse()
				else
					self.m_window:clearActions()
					self:animateShieldPurchase(i, asset, 0.0)
					self:animateShieldPurchase(i, asset, 0.25)
					self:animateShieldPurchase(i, asset, 0.25)
				end
				
			end

            self.m_iap_handle = chunks.iap.purchaseProduct(id, response_complete_callback, product_consumed_callback)
		end

	end

    chunks.utility.unifyTextSizes(text_objs)

	--This overlay button prevents input from leaking to windows underneath the shield store, also causes a tap
	--outside of the menu region to dismiss the shield store.
	local overlay_window = window:spawn( Button() )
	overlay_window.size = constants.scene.world_size
	overlay_window.location = pool.get_v3f(0, 0, 50)
	overlay_window.color = pool.get_colorf(0, 0, 0, 0)
    overlay_window.lua   = {}
	local darken = SequenceAction()
	darken:addAction( AlphaAction( self.m_popup_time, InterpAction.Smooth, 0.75) )
	overlay_window:addAction( darken )
	self.m_overlay_window = overlay_window
	
	overlay_window.on_release = 
    function(owner)
        if not self.m_dismissed then
            self:dismiss()
            self.m_dismissed = true
        end
	end
	
	events.on_shield_store_appeared()
end

function LuaShieldStore:tick()
	self:incrementShieldCounter()
end

function LuaShieldStore:dismiss(delay)
	delay = delay or 0
	self.m_upgrade_window:addAction(SleepAction(delay))
	self.m_upgrade_window:addAction(MoveOffsetAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, -constants.scene.world_size.y, 0)))
		
	self.owner:addAction(SleepAction(delay))
	self.owner:addAction(AlphaAction(self.m_popup_time, InterpAction.Smooth, 0))
	self.owner:addAction(DestroyAction(0))
	
	if(not self.m_overlay_window.lua.m_spawned_from_info_menu) then
		if(not self.owner:getOwner():getOwner().lua:isMainMenu()) then
			if(self.m_force_immediate_use) then
				self.owner:getOwner():getGame().lua:onUnpause()
				self.owner:getOwner():getGame().lua:getHud().lua:activateShield()
			elseif(self.m_immediate_use) then
				self.owner:getOwner():getGame().lua:onUnpause()
                self.owner:getOwner():getGame().lua:onPause()
			else
				if(self.owner:getOwner():getGame().lua:isGameOver())then
					self.owner:getOwner():getGame().lua:onGameOver(false)
				else
					self.owner:getOwner():getGame().lua:onUnpause()
					self.owner:getOwner():getGame().lua:onPause()
				end
			end
		end
	else
		self.owner:getOwner():getOwner().lua:restoreOverlay()
	end
	
	events.on_shield_store_disappeared()

    if self.m_iap_handle then
        self.m_iap_handle:cancel()
    end
end

function LuaShieldStore:setImmediateUseFlag()
	self.m_immediate_use = true
end



function LuaShieldStore:askForImmediateUse()
	self.m_immediate_use = false
	self.m_upgrade_window:addAction(MoveOffsetAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, -constants.scene.world_size.y, 0)))
	self.m_use_now_menu:addAction(MoveOffsetAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, constants.scene.world_size.y, 0)))	
end

function LuaShieldStore:returnToStore()
	self.m_upgrade_window:addAction(MoveOffsetAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, constants.scene.world_size.y, 0)))
	self.m_use_now_menu:addAction(MoveOffsetAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, -constants.scene.world_size.y, 0)))	
end

function LuaShieldStore:setSpawnedFromInfoMenu()
	self.m_overlay_window.lua.m_spawned_from_info_menu = true
end

function LuaShieldStore:animateShieldPurchase(button_index, button_asset, delay)

	self.m_window:addAction( LuaAction( delay, function() 
		local flying_shield = self.m_window:spawn( Window() )
		flying_shield.location = pool.get_v3f(-78, 105-button_asset.size.y*(button_index), 100)
		flying_shield.scale    = pool.get_v3f( 0.45, 0.45, 1.0 )
		textures.setTexture( flying_shield, assets.textures.shieldpack )
		local destination = v3f(-26, -135 , 200)
	
		local fly = SplitAction() 
		fly:addAction( ScaleAction( 0.3, InterpAction.Ramp, pool.get_v3f( 0.4, 0.4, 1.0 ) ) )
		fly:addAction( MoveAction(0.5, InterpAction.Smooth, destination ) )	
		flying_shield:addAction(fly)
		flying_shield:addAction( DestroyAction(0.0) )
	end
	) )
	
end

function LuaShieldStore:incrementShieldCounter()
	local shield_ticker = self.m_shield_meter
	
	local shields = UserData.get():getShieldCapacity() + UserData.get():getFreeShieldCapacity()
	if(shields > 9999) then shields = 9999 end
	
	if(shield_ticker.lua.delay==0) then
		if(shield_ticker.value < shields) then
			local difference = (shields - shield_ticker.value)
			if (difference > 100) then
				shield_ticker.value = shield_ticker.value + 100
			elseif (difference > 10) then
				shield_ticker.value = shield_ticker.value + 10
			elseif (difference >= 1) then
				shield_ticker.value = shield_ticker.value + 1
			end
			shield_ticker.lua.delay = 2
		else
			shield_ticker.value = shields
			shield_ticker.lua.delay = 2
		end
	else
		shield_ticker.lua.delay = shield_ticker.lua.delay - 1
	end
end



