------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Sounds             --
------------------------

module(...,package.seeall)

local class   = require( "shared.class" )
local pool    = require( "shared.pool" )
local engine  = engine.instance
local manager = engine.audio.sound

-- Precache a sound.
function precache( sound )
   if sound.path then
      manager:precache( sound.path )
   end
end

-- Unload a sound.
function unload( sound )
    if sound.path then
        manager:unload( sound.path )
    end
end

-- Play a sound.
function play( sound )
    local channel = nil

    if sound.path then
        channel = manager:play( sound.path, true )
    end

    if channel then
        -- Loops?
        if sound.loops then
            channel.loop_count = -1
        end

        -- Priority?
        if sound.priority then
            channel.priority = sound.priority
        end

        channel.paused = false
    end

    return channel
end

-- Play a random sound given a list.
function play_random( sound )
	local s = sound[ math.random( 1, #sound ) ]
	
	play( s )
end

-----------------
-- sound loops --
-----------------

LuaSoundLoops = class.new
{
	sounds = {}
}

-- Add a loop.
function LuaSoundLoops:add( loop )
	self.sounds[ loop ] = loop
end

-- Remove a loop.
function LuaSoundLoops:remove( loop )
	self.sounds[ loop ] = nil
end

-- Mute all sound loops.
function LuaSoundLoops:mute()
	for k,v in pairs( self.sounds ) do
		v.volume = 0.0
	end
end

-- Unmute all sound loops.
function LuaSoundLoops:unmute()
	for k,v in pairs( self.sounds ) do
		v.volume = 1.0
	end
end