------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Star               --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaStar --
LuaStar = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaStar )

-- The created method.
function LuaStar:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label = "star"
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.star )

		self.visuals.star = actor
	end
	
	self:setVisual( "star" )
end

-- The collided event.
function LuaStar:onCollided( other, is_bonus )
	local game    = self.owner:getGame()
	local ninja   = game.lua.ninja
	local spawner = is_bonus and particles.spawn_uber or particles.spawn_normal
	
	-- Spawn particles.
	do
		spawner( game.lua.root, 
		         assets.textures.particles_spark, 
		         self.visual:getWorldLocation() )
	end
	
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_star )
end

-- The bonus setup method.
function LuaStar:bonus( ninja )
	local left_offset = ninja.lua.sides.left.offset
	local offset      = v3f( -80.0, -70.0, ninja.location.z )
	local ldist       = math.abs( ninja.location.x - offset.x )
	local rdist       = math.abs( ninja.location.x + offset.x )
	local dir         = ldist > rdist and -1 or 1
	local delay       = 0.7
	local set_rot     = function( inner, d )
		local r = ( d == -1 ) and 180.0 or 0.0
		
		ninja.lua:setRotation( v3f( 0.0, r, 0.0 ), v3f( 0.0, r + 180.0, 0.0 ) )
	end
	
	-- Set visual.
	textures.setTexture( ninja.lua.visual, assets.textures.bonus_star )
	
	-- Enlarge shield.
	ninja.lua.shield.lua:bonusStart( 1.7 )
	
	-- Actions.
	do
		local s = SequenceAction()
		
		for i = 1,3 do
			s:addAction( LuaAction( 0.0, set_rot, dir ) )
			s:addAction( MoveAction( delay, InterpAction.Smooth, 
									 pool.get_v3f( math.abs( offset.x ) * dir, offset.y, offset.z ) ) )                    
			dir = dir * -1
		end
		
		-- The last point.
		s:addAction( LuaAction( 0.0, set_rot, dir ) )
		s:addAction( MoveAction( delay, InterpAction.Smooth,
								 pool.get_v3f( math.abs( left_offset.x ) * dir, left_offset.y, left_offset.z ) ) )
		s:addAction( LuaMemberAction( 0.0, "bonusStop" ) )	
		
		ninja:addAction( s )
	end
	
	-- Play sounds.
	sound.play( assets.sounds.bonus_trigger_star )
	sound.play( assets.sounds.bonus_loop_star )
	
	return delay * 4
end

-- Update the ninja's trailer.
function LuaStar:updateTrailer( trailer )
	local s = SequenceAction()
			
	s:addAction( ColorAction( 0.4, InterpAction.Smooth, pool.get_colorf( 1.0, 1.0, 1.0, 1.0 ) ) )
			
	trailer:addAction( s )	
end
