------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- LuaTutorialMenu    --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local constants = engine.platform.constants
local chunks    = engine.chunks
local assets    = chunks.assets
local assets_shared    = chunks.assets_shared
local pool      = chunks.pool
local textures  = chunks.textures
local fade      = chunks.fade
local events    = chunks.events
local utility   = chunks.utility

-- LuaTutorialMenu --
LuaTutorialMenu = chunks.class.inherit( chunks.window.LuaWindow )
{
    -- Whether or not this menu is shown 
    -- when you click the play button.
    m_play = false,
    m_theme = nil
}
	
-- The spawner.
spawn = chunks.class.spawner( Window, LuaTutorialMenu )

function LuaTutorialMenu:draw_text( owner, text, size_height, size_width, position_x, position_y )
	utility.createText( owner, text, {"sans-text", size_height, pool.get_colorf(0, 0, 0, 1), size_width, TextAlign.Center, position_x, position_y}, {drop_shadow = false} )
end

-- The created method.
function LuaTutorialMenu:created( owner )
	local game = owner:getGame()
	
	-- Background.
	do
		textures.setTexture( owner, assets.textures.tutorial )
		
        local text_height = 14.0

        if engine.locale_language == "de" then
			text_height = 11.0
        end

        if engine.locale_language == "es" or engine.locale_language == "pt" or engine.locale_language == "fr" or engine.locale_language == "it" then
			text_height = 10.0
        end
        
        if string.sub(engine.locale_language,1,string.len("zh")) == "zh" then
		    text_height = 20.0
        end

		if constants.WIDESCREEN then
			self:draw_text( owner, lstring( "tap_to_jump_and_attack"), text_height, 180, 40, 180 )
		
			self:draw_text( owner, lstring( "hit_three_of_a_kind_for"), text_height, 175, -65, 80 )
			self:draw_text( owner, lstring( "a_bonus_jump"), text_height, 140, -73, 63 )

			self:draw_text( owner, lstring( "avoid_obstacles"), text_height, 120, -84, -232 )
			self:draw_text( owner, lstring( "on_walls"), text_height, 120, -84, -248 )

			self:draw_text( owner, lstring( "a_shield"), text_height, 120, 85, -63 )
			self:draw_text( owner, lstring( "blocks_one_hit"), text_height, 120, 85, -81 )
		else
			self:draw_text( owner, lstring( "tap_to_jump_and_attack"), text_height, 180, 40, 146 )
		
			self:draw_text( owner, lstring( "hit_three_of_a_kind_for"), text_height, 175, -65, 71 )
			self:draw_text( owner, lstring( "a_bonus_jump"), text_height, 140, -73, 54 )

			self:draw_text( owner, lstring( "avoid_obstacles"), text_height, 120, -84, -202 )
			self:draw_text( owner, lstring( "on_walls"), text_height, 120, -84, -218 )

			self:draw_text( owner, lstring( "a_shield"), text_height, 120, 85, -53 )
			self:draw_text( owner, lstring( "blocks_one_hit"), text_height, 120, 85, -71 )
		end
	end

	-- Fade in.
	do
		fade.fade_in( owner )
	end
	
	-- Analytics.
	events.on_tutorial_menu()
end

-- The options tutorial menu.
function LuaTutorialMenu:options()
	local owner    = self.owner
	local game     = owner:getGame()
	local x_offset = 5.0
	local y_offset = -5.0
	
	-- Back.
	do
		local on_release = function( inner )
			local callback = function( o )
				game:getWindow():spawn( chunks.options.spawn() )
				owner:destroy()
			end
			
			fade.fade_out( owner, callback )
		end
		
		local actor  = self:addButton( on_release )
		actor.anchor = make_anchor( Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f( x_offset, y_offset, 1.0 ) )
		
		textures.setTexture( actor, assets_shared.textures.back_button )

		
	end
end

-- The play tutorial menu.
function LuaTutorialMenu:play( theme )
	local owner    = self.owner
	local game     = owner:getGame()
	local x_offset = -10.0
	local y_offset = 15.0
	local delay    = 0.35

	self.m_play = true
	self.m_theme = theme
	
	-- Play.
	do
		local on_release = function( inner )
			local callback = function( o )
				game.lua:onPlay( theme )
			end
			
			fade.fade_out_loading( owner, callback )
		end
		
		local actor = nil
		if engine.default_language_enabled then
			actor  = self:addButton( on_release )
			textures.setTexture( actor, assets.textures.tutorial_play_button )
			actor.anchor = make_anchor( Anchor.Anchor_BR, Anchor.Anchor_BR, pool.get_v3f( x_offset, y_offset, 1.0 ) )
		else
			local options_text = lstring( "play" )
			local format = {"ninja-text", 36.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 80.0, TextAlign.Center, -2.0, 4.0}
			actor = utility.createButtonWithTextAndTexture(self.owner, options_text, format, assets.textures.play_button_locale )
			actor.on_release = on_release
			actor.anchor = make_anchor( Anchor.Anchor_BR, Anchor.Anchor_BR, pool.get_v3f( x_offset - 10.0, y_offset + -5.0, 1.0 ) )
		end
		
		-- Actions.
		do
			local l = LoopAction()
			
			l:addAction( ScaleAction( delay, InterpAction.Smooth,
								  pool.get_v3f( 1.1, 1.1, 1.0 ) ) )
			l:addAction( ScaleAction( delay, InterpAction.Smooth,
								  pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
			
			actor:addAction( l )
		end
	end
end

function LuaTutorialMenu:onBack()

    local owner    = self.owner
    local game     = owner:getGame()
    local callback = function( o )
        if(not self.m_play)then
            game:getWindow():spawn( chunks.options.spawn() )
            owner:destroy()
        else
			game.lua:onPlay( self.m_theme )
        end
    end
			
    fade.fade_out( owner, callback )
end
