------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Utility            --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local textures  = chunks.textures
local pool      = chunks.pool

-- Generates a random float in the range [min, max].
function frandom( min, max )
	return min + ( max - min ) * math.random()
end

-- Generates a random normal in angle range [min, max].
function random_normal( min, max )
	local angle = frandom( math.rad( min ), math.rad( max ) )
	
	return v2f( math.cos( angle ), math.sin( angle ) ):normalized()
end

function createWindowWithTexture(parent, texture, size, receives_input)
	local window = parent:spawn(Window())
	window.texture = texture.path
	window.size = size or texture.size
	window.receive_input = receives_input or false
	return window
end

function normalize(vector)

	local length = ((vector.x)^2 + (vector.y)^2 + (vector.z)^2)^0.5
	local normalized_vector = vector / length
	return normalized_vector, length
	
end

function createButtonWithTexture(parent, texture)
	local button = parent:spawn(Button())
	chunks.textures.setTexture(button, texture)

	-- Dim the button.
	button.on_enter = function(owner)
		owner.color = colorf( 0.7, 0.7, 0.7, 1.0 )
	end

	-- Undim the button.
	button.on_exit = function(owner)
		owner.color = colorf( 1.0, 1.0, 1.0, 1.0 )
	end

	button.on_enable = function(owner)
		button.color = pool.get_colorf(1.0, 1.0, 1.0, 1.0)
	end

	button.on_disable = function(owner)
		button.color = colorf( 0.7, 0.7, 0.7, 1.0 )
	end

	return button
end

 
function createText(parent, text, input_format, text_fx, wrapping)
	local label = parent:spawn(Text())
	local format = input_format or {"Debug Font", 24.0, colorf(0.0, 0.0, 1.0, 1.0), 0.0, TextAlign.Center, 0.0, 0.0, 0.0, 0.1}
	local alignment = format[5] or TextAlign.Center
	
	label.style = format[1]
	
 	if(wrapping)then
 	    label.options = {height = format[2] * engine.locale_font_scale, max_width = format[4] or 0.0, align = alignment, char_spacing = format[9] or 0, locale_scale_to_bottom = false or format[10], localization_scaling_limit = 0.1, ok_to_split_line = true, max_lines = 3, wrapping = wrapping} 	 	
 	else
 	    label.options = {height = format[2] * engine.locale_font_scale, max_width = format[4] or 0.0, align = alignment, char_spacing = format[9] or 0, locale_scale_to_bottom = false or format[10], localization_scaling_limit = 0.1} 	 	
 	end
 	
 	label.color = format[3]

	local text_offset_x = format[6] or 0.0
	local text_offset_y = format[7] or 0.0
 	local text_offset_z = format[8] or 0.1
 	
 	-- The alignment of the text in 'format' determines the anchoring in this case as well
 	if ( alignment == TextAlign.Left ) then
 		label.anchor = { Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(text_offset_x, text_offset_y, text_offset_z) }
 	elseif ( alignment == TextAlign.Right ) then
 		label.anchor = { Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(text_offset_x, text_offset_y, text_offset_z) }
 	else
 		label.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(text_offset_x, text_offset_y, text_offset_z) }
 	end
	
	label.text = text or ""
	
	local fx = text_fx or {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}
	label.fx = fx;

	return label
end

function createButtonWithTextAndTexture(parent, text, input_format, texture, text_fx, wrapping)
	local button = parent:spawn(Button())
	textures.setTexture(button, texture)
	
	local label = createText(button, text, input_format, text_fx, wrapping)
	button.color = pool.get_colorf(1.0, 1.0, 1.0, 1.0)

	scale_button = input_format[9] or false
	if scale_button then
		button.size = label.size + pool.get_v2f(6, 0)
	end

	-- Dim the button.
	button.on_enter = function(owner)
		owner.color = colorf( 0.7, 0.7, 0.7, 1.0 )
	end

	-- Undim the button.	
	button.on_exit = function(owner)
		owner.color = colorf( 1.0, 1.0, 1.0, 1.0 )
	end

	return button, label
end

function createMultiLineLabel(parent, text, format, max_width)
	print("starting MLL")
	--local container = parent:spawn(Window())

		local label = parent:spawn(Text())
		--label.format = format
		label.style = format[1]
		label.options = {height = format[2] * engine.locale_font_scale, align = TextAlign.Left, wrapping = max_width}
		label.color = format[3] or pool.get_colorf(1,1,1,1)
		
		if(type(text) == "string") then
			print("Start Multilinelabel with", text, "with style", format[1])
			label.text = lstring(text)
		else
			print("Start Multilinelabel with an lstring ", "with style", format[1])
			label.text = text
		end
	
	
	return label
end

function spawnSpriteActor(parent, size, offset_x, offset_y, color)
	local sprite = parent:spawn(SpriteActor())
	sprite.texture = "builtin://white"
	sprite.size = size

	if color then
		sprite.color = color
	else
		sprite.color = colorf(1.0, 0.0, 0.0, 1.0)
	end

	--sprite.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(offset_x, offset_y, 0.1) }

	return sprite
end


function showDialog(title, text, button_text)
    local ui = PlatformUIService.get()
    if ui then
        ui:showDialog(title, text, button_text, function() end);
    end
end

function unifyTextSizes(text_objs)
    local min = 1000000  -- Just needs to be arbitrarily large.

    for _,obj in ipairs(text_objs) do
        min = math.min(min, obj.size.y)
    end

    for _,obj in pairs(text_objs) do
        local options = obj.options

        options.height = min
        obj.options = options
    end
end



