------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- LuaWindow          --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local pool   = chunks.pool
local assets = chunks.assets
local sound  = chunks.sound

-- LuaWindow --
LuaWindow = chunks.class.new()

-- The spawner.
spawn = chunks.class.partial_spawner( LuaWindow )

-- Add a button to a window.  Among other things, handles:
-- * highlighting
-- * click sound
function LuaWindow:addButton( on_release )
	local button = self.owner:spawn( Button() )
	
	-- Dim the button.
	local on_enter = function( owner )
		owner.color = pool.get_colorf( 0.7, 0.7, 0.7, 1.0 )
	end	
		
	-- Undim the button.
	local on_exit = function( owner )
		owner.color = pool.get_colorf( 1.0, 1.0, 1.0, 1.0 )
	end

	-- On release, play the click sound and
	-- call their release function.
	local the_on_release = function( owner )
		sound.play( assets.sounds.click )
		
		on_release( owner )
	end

	button.on_enter   = on_enter
	button.on_exit    = on_exit
	button.on_release = the_on_release
	
	return button
end
