------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Witch              --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local pool      = chunks.pool
local enemy     = chunks.enemy
local particles = chunks.particles
local actions   = chunks.actions
local effects   = chunks.effects
local utility   = chunks.utility
local textures  = chunks.textures
local sound     = chunks.sound

-- LuaWitch --
LuaWitch = chunks.class.inherit( chunks.enemy.LuaEnemy )()

-- The spawner.
spawn = chunks.class.spawner( ActorBase, LuaWitch )

-- The created method.
function LuaWitch:created( owner )
	-- Initialize super.
	enemy.LuaEnemy.created( self, owner )
	
	-- Initialize owner.
	owner.label    = "witch"
	owner.location = pool.get_v3f( -110.0, 0.0, 0.0 )
	
	-- Thrower.
	do
		local actor = owner:spawn( CollidableActor() )
		
		textures.setTexture( actor, assets.textures.witch )

		self.visuals.witch = actor
	end
	
	self:setVisual( "witch" )
	
	-- We're not killable.
	self.killable = false
end

-- The collided event.
function LuaWitch:onCollided( other )
	local game  = self.owner:getGame()
	local ninja = game.lua.ninja

	-- Spawn particles.
	do
		particles.spawn_normal( self.owner:getGame().lua.root, 
								assets.textures.particles_leaf, 
								self.visual:getWorldLocation() )
	end
	
	-- Spawn slice.
	do
		effects.spawn_flash( self.owner:getGame().lua.root, 
							 assets.textures.effects_slice, 
							 self.visual:getWorldLocation() )
	end
	
	self.owner:clearActions()
	actions.shrink_and_kill( self.visual )
	
	-- Play sound.
	sound.play_random( assets.sounds.hit_enemy )
end

-- Setup easy thrower.
function LuaWitch:easy()
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local projectile = actor:spawn( chunks.dart.spawn() )
			
			-- Actions
				do
					local s2 = SequenceAction()
					
					s2:addAction( MoveOffsetAction( utility.frandom( 2.0, 2.5 ), 
								  InterpAction.Lerp, pool.get_v3f( 350.0, 0.0, 0.0 ) ) )
					
					projectile:addAction( s2 )
				end
			
			-- Play sound.
			sound.play_random( assets.sounds.shoot_dart )
		end
		
		s:addAction( SleepAction( 0.3 ) )
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end
end

-- Setup hard thrower.
function LuaWitch:hard()
	local actor = self.owner
	
	-- Actions
	do
		local s = SequenceAction()
		local f = function( inner )
			local projectile = actor:spawn( chunks.dart.spawn() )
			
			-- Actions
				do
					local s2 = SequenceAction()
					
					s2:addAction( MoveOffsetAction( utility.frandom( 1.4, 1.8 ), 
								  InterpAction.Lerp, pool.get_v3f( 350.0, 0.0, 0.0 ) ) )
					
					projectile:addAction( s2 )
				end
			
			-- Play sound.
			sound.play_random( assets.sounds.shoot_dart )
		end
		
		s:addAction( SleepAction( 0.5 ) )
		s:addAction( LuaAction( 0.0, f ) )
		
		actor:addAction( s )
	end
end


